; Copyright (C)  Michael Holzrichter 1992,  All rights reserved.

        title   VSINCOS.ASM:  Compute the Sine or Cosine of a Vector
        page    55,132

; Call with:    vsin (float* a, int str_a, float* b, int n)
;               vcos (float* a, int str_a, float* b, int n)
;
; Returns:      b [i*str_b] = sin (a [i*str_a]) for i = 0 to n-1
;               b [i*str_b] = cos (a [i*str_a]) for i = 0 to n-1 
;
; (Angles are in radians.)
;
; Destroys:     nothing

.286
.287

;.MODEL SMALL
.MODEL LARGE

.DATA

piby4     dq      3fe921fb54442d18h       ; constant pi / 4
piby2     dq      3ff921fb54442d18h       ; constant pi / 2

odda      dq      1.0000000000000e+000
oddb      dq     -1.6666728019714e-001
oddc      dq      8.3358026981354e-003
oddd      dq     -1.9902997970581e-004

evena     dq      1.0000000000000e+000
evenb     dq     -5.0000000000000e-001
evenc     dq      4.1666667938232e-002
evend     dq     -1.3761875391006e-003

.CODE

FLAGS     equ     bp-12
DS_SAV    equ     bp-10
PIDV4     equ     bp-8
PIDV2     equ     bp-4

N_ELM     equ     bp+6
B_STR     equ     bp+8
B_OFF     equ     bp+10
B_SEG     equ     bp+12
A_STR     equ     bp+14
A_OFF     equ     bp+16
A_SEG     equ     bp+18

          PUBLIC  _vsin__FPfiPfii, _vcos__FPfiPfii

_vsin__FPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp, sp                  ; Set stack framepointer

          sub     sp, 12

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx

          mov ax, SEG DGROUP              ; Get segment of constants
          mov [DS_SAV], ax
          mov ds, [DS_SAV]

          fld     piby2                   ; Store constants locally
          fstp    dword ptr [PIDV2]
          fld     piby4
          fstp    dword ptr [PIDV4]

          fld     evenc                   ; Load constants
          fld     evend

          fld     oddc
          fld     oddd

          fld     piby2

          mov     si, [A_OFF]             ; Get address of A array

          mov     di, [B_OFF]             ; Get address of B array
          mov     es, [B_SEG]

          shl     word ptr [A_STR], 2     ; Compute stride of arrays
          shl     word ptr [B_STR], 2

          mov     bx, 4002h

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jg      sloop
          jmp     serror

sloop:    mov     ds, [A_SEG]
          fld     dword ptr ds:[si]       ; Load angle   
          mov     ds, [DS_SAV]

          ftst
          fstsw   [FLAGS]
          fabs

          fadd    dword ptr [PIDV4]       ; Sub pi/4

          fprem                           ; reduce to 0<=angle<pi/4
          fstsw   ax                      ; Get FPREM status

          fsub    dword ptr [PIDV4]       ; ST(0) = reduced angle

          mov     al, ah                  ; AL ~= 0 means odd
          and     ax, bx                  ; AH ~= 0 negative

          or      al, al
          jz      sodd                    ; branch to correct sequence

                                          ; 0    1    2    3    4    5    6    7
          fmul    st(0), st(0)            ; x*x  pi2  od   oc   ed   ec
          fld     st(0)                   ; x*x  x*x  pi2  od   oc   ed   ec
          fmul    st(0), st(5)            ; ---  x*x  pi2  od   oc   ed   ec
          fadd    st(0), st(6)            ; ---  x*x  pi2  od   oc   ed   ec
          fmul    st(0), st(1)            ; ---  x*x  pi2  od   oc   ed   ec
          fadd    evenb                   ; ---  x*x  pi2  od   oc   ed   ec
          fmulp   st(1), st(0)            ; ---  pi2  od   oc   ed   ec
          fadd    evena

          jmp     ssign1

                                          ; 0    1    2    3    4    5    6    7
sodd:     fld     st(0)                   ; x    x    pi2  od   oc   ed   ec
          fmul    st(0), st(0)            ; x*x  x    pi2  od   oc   ed   ec
          fld     st(0)                   ; x*x  x*x  x    pi2  od   oc   ed  ec
          fmul    st(0), st(4)            ; ---  x*x  x    pi2  od   oc   ed  ec
          fadd    st(0), st(5)            ; ---  x*x  x    pi2  od   oc   ed  ec
          fmul    st(0), st(1)            ; ---  x*x  x    pi2  od   oc   ed  ec
          fadd    oddb                    ; ---  x*x  x    pi2  od   oc   ed  ec
          fmulp   st(1), st(0)            ; ---  x    pi2  od   oc   ed   ec
          fadd    odda                    ; ---  x    pi2  od   oc   ed   ec
          fmulp   st(1), st(0)            ; ---  pi2  od   oc   ed   eb

ssign1:   or      ah, ah                  ; Set sign based on quadrant of angle
          jz      ssign2

          fchs

ssign2:   mov     ax, [FLAGS]             ; Set sign based on sign of angle
          sahf
          jae     ssave

          fchs

ssave:    fstp    dword ptr es:[di]       ; Store result

          add     si, [A_STR]             ; Increment pointers
          add     di, [B_STR]

          loop    sloop                   ; Repeat until cx is 0

          fstp    st                      ; Pop constants off stack
          fstp    st
          fstp    st
          fstp    st
          fstp    st

          fwait                           ; Wait for processor to catch up

          xor     ax, ax                  ; Return code is 0

sexit:    pop     bx                      ; Restore registers
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          mov     sp, bp                  ; Release memory for local variables
          pop     bp

          RET     14                      ; Return

serror:   mov     ax, 1
          jmp     sexit

_vsin__FPfiPfii ENDP

_vcos__FPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp, sp                  ; Set stack framepointer

          sub     sp, 12

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx

          mov ax, SEG DGROUP              ; Get segment of constants
          mov [DS_SAV], ax
          mov ds, [DS_SAV]

          fld     piby2                   ; Store constants locally
          fstp    dword ptr [PIDV2]
          fld     piby4
          fstp    dword ptr [PIDV4]

          fld     evenc                   ; Load constants
          fld     evend

          fld     oddc
          fld     oddd

          fld     piby2

          mov     si, [A_OFF]             ; Get address of A array

          mov     di, [B_OFF]             ; Get address of B array
          mov     es, [B_SEG]

          shl     word ptr [A_STR], 2     ; Compute stride of arrays
          shl     word ptr [B_STR], 2

          mov     bx, 4002h

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jg      cloop
          jmp     cerror

cloop:    mov     ds, [A_SEG]
          fld     dword ptr ds:[si]       ; Load angle   
          mov     ds, [DS_SAV]
 
          fabs

          fadd    dword ptr [PIDV4]       ; Sub pi/4

          fprem                           ; reduce to 0<=angle<pi/4
          fstsw   ax                      ; Get FPREM status

          fsub    dword ptr [PIDV4]       ; ST(0) = reduced angle

          mov     al, ah                  ; AL ~= 0 means odd
          and     ax, bx                  ; AH ~= 0 negative

          or      al, al
          jnz     codd                    ; branch to correct sequence

                                          ; 0    1    2    3    4    5    6    7
          fmul    st(0), st(0)            ; x*x  pi2  od   oc   ed   ec
          fld     st(0)                   ; x*x  x*x  pi2  od   oc   ed   ec
          fmul    st(0), st(5)            ; ---  x*x  pi2  od   oc   ed   ec
          fadd    st(0), st(6)            ; ---  x*x  pi2  od   oc   ed   ec
          fmul    st(0), st(1)            ; ---  x*x  pi2  od   oc   ed   ec
          fadd    evenb                   ; ---  x*x  pi2  od   oc   ed   ec
          fmulp   st(1), st(0)            ; ---  pi2  od   oc   ed   ec
          fadd    evena

          jmp     csign1

                                          ; 0    1    2    3    4    5    6    7
codd:     fld     st(0)                   ; x    x    pi2  od   oc   ed   ec
          fmul    st(0), st(0)            ; x*x  x    pi2  od   oc   ed   ec
          fld     st(0)                   ; x*x  x*x  x    pi2  od   oc   ed  ec
          fmul    st(0), st(4)            ; ---  x*x  x    pi2  od   oc   ed  ec
          fadd    st(0), st(5)            ; ---  x*x  x    pi2  od   oc   ed  ec
          fmul    st(0), st(1)            ; ---  x*x  x    pi2  od   oc   ed  ec
          fadd    oddb                    ; ---  x*x  x    pi2  od   oc   ed  ec
          fmulp   st(1), st(0)            ; ---  x    pi2  od   oc   ed   ec
          fadd    odda                    ; ---  x    pi2  od   oc   ed   ec
          fmulp   st(1), st(0)            ; ---  pi2  od   oc   ed   eb

csign1:   or      ah, ah                  ; Set sign based on quadrant of angle
          jz      csign2

          fchs

csign2:   or      al, al
          jz      csave

          fchs

csave:    fstp    dword ptr es:[di]       ; Store result

          add     si, [A_STR]             ; Increment pointers
          add     di, [B_STR]

          loop    cloop                   ; Repeat until cx is 0

          fstp    st                      ; Pop constants off stack
          fstp    st
          fstp    st
          fstp    st
          fstp    st

          fwait                           ; Wait for processor to catch up

          xor     ax, ax                  ; Return code is 0

cexit:    pop     bx                      ; Restore registers
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          mov     sp, bp                  ; Release memory for local variables
          pop     bp

          RET     14                      ; Return

cerror:   mov     ax, 1
          jmp     cexit

_vcos__FPfiPfii ENDP
          END
