/* Copyright (C)  Michael Holzrichter 1992,  All rights reserved. */


#include "zfloat.hpp"
#include <math.h>

unsigned int zitof (long far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia];
   oa [0] = ia [0];

   return 0;
}


unsigned int zftoi (float far *ia, int s_ia, long far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia];

   return 0;
}


unsigned int zvssub (float far *ia, int s_ia, float ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia] - ib;

   return 0;
}


unsigned int zvsdiv (float far *ia, int s_ia, float ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia] / ib;

   return 0;
}


unsigned int zsvadd (float ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia + ib [i*s_ib];

   return 0;
}


unsigned int zsvsub (float ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia - ib [i*s_ib];

   return 0;
}


unsigned int zsvmul (float ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia * ib [i*s_ib];

   return 0;
}


unsigned int zsvdiv (float ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia / ib [i*s_ib];

   return 0;
}


unsigned int zvvadd (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia] + ib [i*s_ib];

   return 0;
}


unsigned int zvvsub (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia] - ib [i*s_ib];

   return 0;
}


unsigned int zvvmul (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia] * ib [i*s_ib];

   return 0;
}


unsigned int zvvdiv (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia] / ib [i*s_ib];

   return 0;
}


unsigned int zvsmnmx (float far *ia, int s_ia, float far *oa, int n_e)
{
   int i;
   float ftmp;

   if (n_e < 0) return 1;

   ftmp = ia [0];

   oa [0] = ftmp;
   oa [1] = ftmp;

   for (i=1; i<n_e; ++i) {
       ftmp = ia [i*s_ia];

       if (oa [0] > ftmp) oa [0] = ftmp;
       if (oa [1] < ftmp) oa [1] = ftmp;
   }

   return 0;
}


unsigned int zvamnmx (float far *ia, int s_ia, float far *oa, int n_e)
{
   int i;
   float ftmp;

   if (n_e < 0) return 1;

   ftmp = ia [0];
   if (ftmp < 0.0) ftmp = -ftmp;

   oa [0] = ftmp;
   oa [1] = ftmp;

   for (i=1; i<n_e; ++i) {
       ftmp = ia [i*s_ia];
       if (ftmp < 0.0) ftmp = -ftmp;

       if (oa [0] > ftmp) oa [0] = ftmp;
       if (oa [1] < ftmp) oa [1] = ftmp;
   }

   return 0;
}


unsigned int zvesum (float far *ia, int s_ia, float far *oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   oa [0] = ia [0];

   for (i=1; i<n_e; ++i) oa [0] += ia [i*s_ia];

   return 0;
}


unsigned int zvsqrt (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = sqrt (ia [i*s_ia]);

   return 0;
}


unsigned int zvsin (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = sin (ia [i*s_ia]);

   return 0;
}


unsigned int zvcos (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = cos (ia [i*s_ia]);

   return 0;
}


unsigned int zvzero (float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = 0.0;

   return 0;
}


unsigned int zvmove (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = ia [i*s_ia];

   return 0;
}


unsigned int zvabs (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) {
      if (ia [i*s_ia] < 0.0) 
         oa [i*s_oa] = -ia [i*s_ia];
      else
         oa [i*s_oa] =  ia [i*s_ia];
   }

   return 0;
}


unsigned int zvneg (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) oa [i*s_oa] = - ia [i*s_ia];

   return 0;
}


unsigned int zvsign0 (float far *ia, int s_ia, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) {
      if (ia [i*s_ia] < 0.0)
         oa [i*s_oa] = -1.0;
      else
         oa [i*s_oa] =  1.0;
   }

   return 0;
}


unsigned int zvsign1 (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int s_oa, int n_e)
{
   int i;

   if (n_e < 0) return 1;

   for (i=0; i<n_e; ++i) {
      if (ia [i*s_ia] * ib [i*s_ib] < 0.0)
         oa [i*s_oa] = -ib [i*s_ib];
      else
         oa [i*s_oa] =  ib [i*s_ib];
   }

   return 0;
}


unsigned int zvdprod (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int n_e)
{
   int i;

   float sum;

   if (n_e < 0) return 1;

   *oa = 0.0;

   for (i=0; i<n_e; ++i)
      *oa += ia [i*s_ia] * ib [i*s_ib];

   return 0;
}

/*

Note this function is derived from a similar cubic spline function in
"Numerical Recipes in C".  That reference should be consulted for an
explanation for the algorithm.  However, the function served only as a
prototype.  Extensive modifications were made to fit it to my needs and
to speed it up.

*/


unsigned int zvsplin (float far *ia, int s_ia, float far *ib, int s_ib, float far *oa, int s_oa, int n, float yp1, float ypn, int opt, float far* w)
{
   int   i;
   float p, q;
   float *w1_ptr, *w2_ptr;

   if (n < 3) return 1;

   w1_ptr = w;
   w2_ptr = &w [n];

   if (opt & 0x00ff) {
      *w1_ptr = (0.5 / (ia [s_ia] - ia [0])) * ((ib [s_ib] - ib [0]) / (ia [s_ia] - ia [0]) - yp1);
      *w2_ptr = -0.5;
   } else {
      *w1_ptr = 0.0;
      *w2_ptr = 0.0;
   }

   for (i=1; i<n-1; i++) {
      p = (ia [i*s_ia] - ia [(i-1)*s_ia]) / (ia [(i+1)*s_ia] - ia [(i-1)*s_ia]);
      q = p * (*w2_ptr) + 2.0;

      (*++w2_ptr) = (p - 1.0) / q;
      (*++w1_ptr) = ((((ib [(i+1)*s_ib] - ib [i*s_ib]) / (ia [(i+1)*s_ia] - ia [i*s_ia]) - 
                             (ib [i*s_ib] - ib [(i-1)*s_ib]) / (ia [i*s_ia] - ia [(i-1)*s_ia]))
                     / (ia [(i+1)*s_ia] - ia [(i-1)*s_ia])) - p * (*w1_ptr)) / q;
   }

   if (opt & 0xff00) {
      oa [(n-1)*s_oa] = ((1.0 / (ia [(n-1)*s_ia] - ia [(n-2)*s_ia])) * 
                        (ypn - (ib [(n-1)*s_ib] - ib [(n-2)*s_ib]) / (ia [(n-1)*s_ia] - ia [(n-2)*s_ia]))
                        - (*w1_ptr)) / ((*w2_ptr) + 2.0);
   } else {
      oa [(n-1)*s_oa] = 0.0;
   }

   for (i=n-2; i>=0; i--) oa [i*s_oa] = (*w2_ptr--) * oa [(i+1)*s_oa] + (*w1_ptr--);

   return 0;
}


unsigned int zvspval0 (float far *ia, int s_ia, float far *ib, int s_ib, float far *ic, int s_ic, int n, float id, float far *oa)
{
   int      klo, khi, k;
   int      k1, k2;
   float    a, b, h;

   klo = 0;
   khi = n-1;

   if (n < 2) return 1;

   if (id < ia [klo*s_ia]) return 2;
   if (id > ia [khi*s_ia]) return 3;

   while (khi-klo > 1) {
      k = (khi + klo) >> 1;
      if (ia [k*s_ia] > id)
         khi = k;
      else
         klo = k;
   }

   h = ia [khi*s_ia] - ia [klo*s_ia];

   a = (ia [khi*s_ia] - id) / h;
   b = (id - ia [klo*s_ia]) / h;

   k1 = klo*s_ib;
   k2 = klo*s_ic;

   *oa = (a * ib [k1]) + (b * ib [k1+s_ib]) +
         ((a*a*a-a) * ic [k2] + (b*b*b-b) * ic [k2+s_ic]) * (h*h);
}


unsigned int zvspval1 (float far *ia, int s_ia, float far *ib, int s_ib, float far *ic, int s_ic, int n, float far* id, int s_id, float far *oa, int s_oa, int m)
{
   int      i, j, k, l;

   int      klo, khi, jlo;
   int      k1, k2, k3, k4;

   float    a, b, h;

   if (n < 2) return 1;
   if (m < 1) return 2;

   jlo = -1;
   klo = 0;
   khi = n-1;

   i = 0;
   j = (m-1) * s_id;
   k = 0;

   k1 = klo * s_ia;
   k2 = khi * s_ia;

   if (id [0] < ia [k1]) return 3;
   if (id [j] < ia [k1]) return 3;

   if (id [0] > ia [k2]) return 4;
   if (id [j] > ia [k2]) return 4;

   while (i <= j) {
      while (khi-klo > 1) {
         l = (khi + klo) >> 1;
         if (ia [l*s_ia] > id [i])
            khi = l;
         else
            klo = l;
      }

      if (jlo != klo) {
         jlo = klo;

         k1 = klo * s_ib;
         k2 = klo * s_ic;

         k3 = klo * s_ia;
         k4 = khi * s_ia;

         h = ia [k4] - ia [k3];
      }

      a = (ia [k4] - id [i]) / h;
      b = (id [i] - ia [k3]) / h;

      oa [k] = (a * ib [k1]) + (b * ib [k1+s_ib]) +
            ((a*a*a-a) * ic [k2] + (b*b*b-b) * ic [k2+s_ic]) * (h*h);

      i += s_id;
      k += s_oa;

      if (i <= j) {
         if (id [i] < id [i-s_id]) return 5;

         l = 1;
         while (ia [khi*s_ia] < id [i]) {
            klo = khi;
            khi += l;

            l *= 2;

            if (khi >= n) khi = n - 1;
         }
      } 
   }

   return 0;
}


unsigned int zvspval2 (float far *ia, int s_ia, float far *ib, int s_ib, float far *ic, int s_ic, int n, float first, float inc, float far *oa, int s_oa, int m)
{
   int      i, j, k, l;

   int      klo, khi, jlo;
   int      k1, k2, k3, k4;

   float    a, b, h;

   if (n < 2) return 1;
   if (m < 1) return 2;

   jlo = -1;
   klo = 0;
   khi = n-1;

   i = 0;
   j = 0;

   k1 = klo * s_ia;
   k2 = khi * s_ia;

   if (inc < 0.0) return 6;

   for (k=0; k<m; ++k) {
      oa [j] = first + k*inc;
      j += s_oa;
   }

   j -= s_oa;

   if (oa [0] < ia [k1]) return 3;
   if (oa [j] < ia [k1]) return 3;

   if (oa [0] > ia [k2]) return 4;
   if (oa [j] > ia [k2]) return 4;

   while (i <= j) {
      while (khi-klo > 1) {
         l = (khi + klo) >> 1;
         if (ia [l*s_ia] > oa [i])
            khi = l;
         else
            klo = l;
      }

      if (jlo != klo) {
         jlo = klo;

         k1 = klo * s_ib;
         k2 = klo * s_ic;

         k3 = klo * s_ia;
         k4 = khi * s_ia;

         h = ia [k4] - ia [k3];
      }

      a = (ia [k4] - oa [i]) / h;
      b = (oa [i] - ia [k3]) / h;

      oa [i] = (a * ib [k1]) + (b * ib [k1+s_ib]) +
            ((a*a*a-a) * ic [k2] + (b*b*b-b) * ic [k2+s_ic]) * (h*h);

      i += s_oa;

      if (i <= j) {
         l = 1;
         while (ia [khi*s_ia] < oa [i]) {
            klo = khi;
            khi += l;

            l *= 2;

            if (khi >= n) khi = n - 1;
         }
      } 
   }

   return 0;
}


unsigned int zbitrev (float far *ia, int s_ia, int n)
{
   int      i, j, k, l, nelem;

   float    temp;

   nelem = 0x0001 << n;

   for (i = 0; i < nelem; ++i) {
      k = 0;
      l = i;

      for (j = 0; j < n; ++j) {
         k = (k << 1);
         k |= (0x0001 & l);
         l = (l >> 1);
      }

      if (k > i) {
         temp          = ia [i * s_ia];
         ia [i * s_ia] = ia [k * s_ia];
         ia [k * s_ia] = temp;
      }
   }

   return 0;
}


unsigned int zbfly0 (float far *ia, int s_ia, float far *ib, int s_ib, int n, int opt)
{
   int      i, j;

   int      nsets, npair, ir, jr, ii, ji, distr, disti;

   float    twdlr [16] = {
      -1.0000000000000,
       0.0000000000000,
       0.7071067811865,
       0.9238795325113,
       0.9807852804032,
       0.9951847266722,
       0.9987954562052,
       0.9996988186962,
       0.9999247018391,
       0.9999811752826,
       0.9999952938096,
       0.9999988234517,
       0.9999997058629,
       0.9999999264657,
       0.9999999816164,
       0.9999999954041
   };

   float    twdlif [16] = {
       0.0000000000000,
      -1.0000000000000,
      -0.7071067811865,
      -0.3826834323651,
      -0.1950903220161,
      -0.0980171403296,
      -0.0490676743274,
      -0.0245412285229,
      -0.0122715382857,
      -0.0061358846492,
      -0.0030679567630,
      -0.0015339801863,
      -0.0007669903187,
      -0.0003834951876,
      -0.0001917475973,
      -0.0000958737991
   };

   float    twdlir [16] = {
       0.0000000000000,
       1.0000000000000,
       0.7071067811865,
       0.3826834323651,
       0.1950903220161,
       0.0980171403296,
       0.0490676743274,
       0.0245412285229,
       0.0122715382857,
       0.0061358846492,
       0.0030679567630,
       0.0015339801863,
       0.0007669903187,
       0.0003834951876,
       0.0001917475973,
       0.0000958737991
   };

   float    tempr, tempi, scale;
   float    wr, wi, *wpr, *wpi;

   nsets = 1;
   npair = 1;

   wpr = twdlr;
   wpi = (opt & 0x0001) ? twdlir : twdlif;

   for (i=1; i<n; ++i) {
      ++wpr;
      ++wpi;

      npair *= 2;
   }

   distr = s_ia * npair;
   disti = s_ib * npair;

   while (npair > 0) {
      wr = 1.0;
      wi = 0.0;

      for (i = 0; i < npair; ++i) {
         ir = i * s_ia;
         ii = i * s_ib;

         for (j = 0; j < nsets; ++j) {
            jr = ir + distr;
            ji = ii + disti;

            tempr = ia [ir] - ia [jr];
            tempi = ib [ii] - ib [ji];

            ia [ir] += ia [jr];
            ib [ii] += ib [ji];

            ia [jr] = (tempr * wr) - (tempi * wi);
            ib [ji] = (tempr * wi) + (tempi * wr);

            ir = jr + distr;
            ii = ji + disti;
         }

         tempr = (*wpr * wr) - (*wpi * wi);
         wi    = (*wpi * wr) + (*wpr * wi);
         wr    = tempr;
      }

      --wpr;
      --wpi;

      npair /= 2;
      nsets *= 2;

      distr /= 2;
      disti /= 2;
   }

   if (opt & 0x0002) {
      j = (0x0001) << n;
      scale = 1.0 / j;

      for (i=0; i<j; ++i) {
         ia [i * s_ia] *= scale;
         ib [i * s_ib] *= scale;
      }
   }

   return 0;
}


unsigned int zbfly1 (float far *ia, int s_ia, float far *ib, int s_ib, int n, int opt)
{
   int      i, j;

   int      nsets, npair, ir, jr, ii, ji, distr, disti;

   float    twdlr [16] = {
      -1.0000000000000,
       0.0000000000000,
       0.7071067811865,
       0.9238795325113,
       0.9807852804032,
       0.9951847266722,
       0.9987954562052,
       0.9996988186962,
       0.9999247018391,
       0.9999811752826,
       0.9999952938096,
       0.9999988234517,
       0.9999997058629,
       0.9999999264657,
       0.9999999816164,
       0.9999999954041
   };

   float    twdli [16] = {
       0.0000000000000,
      -1.0000000000000,
      -0.7071067811865,
      -0.3826834323651,
      -0.1950903220161,
      -0.0980171403296,
      -0.0490676743274,
      -0.0245412285229,
      -0.0122715382857,
      -0.0061358846492,
      -0.0030679567630,
      -0.0015339801863,
      -0.0007669903187,
      -0.0003834951876,
      -0.0001917475973,
      -0.0000958737991
   };

   float    tempr1, tempi1, tempr2, tempi2, a, b;
   float    wr, wi, *wpr, *wpi, wdr, wdi;

   wpr = twdlr;
   wpi = twdli;

   for (i=0; i<n; ++i) {
      ++wpr;
      ++wpi;
   }

   npair = 0x0001 << n;

   wr = wdr = *wpr;
   wi = wdi = *wpi;

   if (opt & 0x0001) {
      wr  *= -1.0;
      wdi *= -1.0;
   }

   ir = 0;
   ii = 0;

   jr = s_ia * npair;
   ji = s_ib * npair;

   while (ir != jr) {
      ir += s_ia;
      ii += s_ib;

      jr -= s_ia;
      ji -= s_ib;

      tempr1 = 0.5 * (ia [ir] + ia [jr]);
      tempi1 = 0.5 * (ib [ii] - ib [ji]);

      tempr2 = 0.5 * (ib [ii] + ib [ji]);
      tempi2 = 0.5 * (ia [jr] - ia [ir]);

      a = (wr * tempr2 - wi * tempi2);
      b = (wr * tempi2 + wi * tempr2);

      ia [ir] = tempr1 + a;
      ib [ii] = tempi1 + b;

      ia [jr] = tempr1 - a;
      ib [ji] = b - tempi1;

      a  = (wdr * wr) - (wdi * wi);
      wi = (wdi * wr) + (wdr * wi);
      wr = a;
   }

   a      =     ia [0];
   ia [0] = a + ib [0];
   ib [0] = a - ib [0];

   if (opt & 0x0001) {
      ia [0] *= 0.5;
      ib [0] *= 0.5;
   } 

   return 0;
}


unsigned int zfft0 (float far *ia, int s_ia, float *ib, int s_ib, int n, int o)
{
   unsigned int err;

   err = zbfly0 (ia, s_ia, ib, s_ib, n, o);
   if (err) return (err | 0x0100);
   
   if (o & 0x0100) return 0;

   err = zbitrev (ia, s_ia, n);
   if (err) return (err | 0x0200);

   err = zbitrev (ib, s_ib, n);
   if (err) return (err | 0x0300);

   return 0;
}


unsigned int zfft1 (float far *ia, int s_ia, float *ib, int s_ib, int n, int o)
{
   unsigned int err;

   if ((o & 0x0001) == 1) {
      err = zbfly1 (ia, s_ia, ib, s_ib, n, o);
      if (err) return (err | 0x0400);
   }

   err = zbfly0 (ia, s_ia, ib, s_ib, n, o);
   if (err) return (err | 0x0100);
   
   err = zbitrev (ia, s_ia, n);
   if (err) return (err | 0x0200);

   err = zbitrev (ib, s_ib, n);
   if (err) return (err | 0x0300);

   if ((o & 0x0001) == 0) {
      err = zbfly1 (ia, s_ia, ib, s_ib, n, o);
      if (err) return (err | 0x0400);
   }

   return 0;
}

