/*
 *    dnddll.c - Dialog editor / DLL interface for drag-and-drop edit control.
 */

#include <stdio.h>
#include <ctype.h>

#include <windows.h>
#include <custcntl.h>

#include <string.h>
#include <memory.h>
#include "dndedit.h"
#include "ccdlg.h"

static
BOOL    GetButton(HWND Dialog, WPARAM Control)
    {
    return (SendDlgItemMessage(Dialog, Control, BM_GETSTATE, 0, 0)
        & 0x001) != 0;
    }

static
void    SetData(HWND Dialog, LPARAM DataPointer)
    {
    SetProp(Dialog, "HI", (HANDLE)HIWORD(DataPointer));
    SetProp(Dialog, "LO", (HANDLE)LOWORD(DataPointer));
    }

static
CC_INIT_DIALOG  FAR *GetData(HWND Dialog)
    {
    return (CC_INIT_DIALOG FAR *)MAKELP(GetProp(Dialog,"HI"),
                                      GetProp(Dialog, "LO"));
    }

static
void    Pack(HWND Dialog)
    {
    DWORD   Bits;
    CC_INIT_DIALOG  FAR *InitData = GetData(Dialog);
    CTLSTYLE    FAR *StyleData = GlobalLock(InitData->StyleData);
    
    Bits    = StyleData->dwStyle;
    Bits    &= ~(WS_VISIBLE | WS_DISABLED | WS_BORDER | WS_TABSTOP
            | WS_GROUP | WS_VSCROLL | WS_HSCROLL | 0x0FFFFL);

    GetDlgItemText(Dialog, D_CAPTION, StyleData->szTitle, CTLTITLE);
    if(GetButton(Dialog, D_VISIBLE))
        Bits    |=  WS_VISIBLE;
    if(GetButton(Dialog, D_DISABLED))
        Bits    |=  WS_DISABLED;
    if(GetButton(Dialog, D_BORDER))
        Bits    |=  WS_BORDER;
    if(GetButton(Dialog, D_TAB_STOP))
        Bits    |=  WS_TABSTOP;
    if(GetButton(Dialog, D_GROUP))
        Bits    |=  WS_GROUP;
    if(GetButton(Dialog, D_SCROLL_VERT))
        Bits    |=  WS_VSCROLL;
    if(GetButton(Dialog, D_SCROLL_HORZ))
        Bits    |=  WS_HSCROLL;
    if(GetButton(Dialog, D_CENTER_JUST))
        Bits    |=  ES_CENTER;
    else if(GetButton(Dialog, D_RIGHT_JUST))
        Bits    |=  ES_RIGHT;
    if(GetButton(Dialog, D_UPPER_CASE))
        Bits    |= ES_UPPERCASE;
    else if(GetButton(Dialog, D_LOWER_CASE))
        Bits    |= ES_LOWERCASE;
    if(GetButton(Dialog, D_MULTIPLE_LINE))
        Bits    |= ES_MULTILINE;
    if(GetButton(Dialog, D_PASSWORD))
        Bits    |= ES_PASSWORD;
    if(GetButton(Dialog, D_CONVERT_OEM))
        Bits    |= ES_OEMCONVERT;
    if(GetButton(Dialog, D_KEEP_SELECTION))
        Bits    |= ES_NOHIDESEL;
    if(GetButton(Dialog, D_AUTO_VERT))
        Bits    |= ES_AUTOVSCROLL;
    if(GetButton(Dialog, D_AUTO_HORIZ))
        Bits    |= ES_AUTOHSCROLL;
    if(GetButton(Dialog, D_READ_ONLY))
        Bits    |= ES_READONLY;
    if(GetButton(Dialog, D_WANT_RETURN))
        Bits    |= ES_WANTRETURN;
    StyleData->dwStyle      = Bits;
    {
    char    Buffer[128];
    DWORD   Id;
    GetDlgItemText(Dialog, D_ID_STRING, Buffer, 127);
    Id  = InitData->StrToId(Buffer);
    StyleData->wId  = HIWORD(Id);
    }

    GlobalUnlock(InitData->StyleData);
    }

static
void    UnPack(HWND Dialog)
    {
    DWORD   Bits;
    CC_INIT_DIALOG  FAR *InitData = GetData(Dialog);
    CTLSTYLE    FAR *StyleData = GlobalLock(InitData->StyleData);

    Bits    = StyleData->dwStyle;
    {
    char    Buffer[128];
    if(InitData->IdToStr(StyleData->wId, Buffer, sizeof(Buffer)))
        SetDlgItemText(Dialog, D_ID_STRING, Buffer);
    if(Buffer[0] == '_' || isalpha(Buffer[0]))
        {
        sprintf(Buffer, "%d", StyleData->wId);
        SetDlgItemText(Dialog, D_ID_NUMBER, Buffer);
        }
    }
    SetDlgItemText(Dialog, D_CAPTION, StyleData->szTitle);
    SendDlgItemMessage(Dialog, D_CAPTION_TEXT, BM_SETCHECK, 1, 0);
    SendDlgItemMessage(Dialog, D_CAPTION_TEXT, WM_ENABLE, FALSE, 0);
    
    if(Bits & WS_VISIBLE)
        SendDlgItemMessage(Dialog, D_VISIBLE, BM_SETCHECK, 1, 0);
    if(Bits & WS_DISABLED)
        SendDlgItemMessage(Dialog, D_DISABLED, BM_SETCHECK, 1, 0);
    if(Bits & WS_BORDER)
        SendDlgItemMessage(Dialog, D_BORDER, BM_SETCHECK, 1, 0);
    if(Bits & WS_TABSTOP)
        SendDlgItemMessage(Dialog, D_TAB_STOP, BM_SETCHECK, 1, 0);
    if(Bits & WS_GROUP)
        SendDlgItemMessage(Dialog, D_GROUP, BM_SETCHECK, 1, 0);
    if(Bits & WS_VSCROLL)
        SendDlgItemMessage(Dialog, D_SCROLL_VERT, BM_SETCHECK, 1, 0);
    if(Bits & WS_HSCROLL)
        SendDlgItemMessage(Dialog, D_SCROLL_HORZ, BM_SETCHECK, 1, 0);
    if(Bits & ES_CENTER)
        SendDlgItemMessage(Dialog, D_CENTER_JUST, BM_SETCHECK, 1, 0);
    else if(Bits & ES_RIGHT)
        SendDlgItemMessage(Dialog, D_RIGHT_JUST, BM_SETCHECK, 1, 0);
    else
        SendDlgItemMessage(Dialog, D_LEFT_JUST, BM_SETCHECK, 1, 0);
    if(Bits & ES_UPPERCASE)
        SendDlgItemMessage(Dialog, D_UPPER_CASE, BM_SETCHECK, 1, 0);
    else if(Bits & ES_LOWERCASE)
        SendDlgItemMessage(Dialog, D_LOWER_CASE, BM_SETCHECK, 1, 0);
    else
        SendDlgItemMessage(Dialog, D_CASE_INSENSITIVE, BM_SETCHECK, 1, 0);
    if(Bits & ES_MULTILINE)
        SendDlgItemMessage(Dialog, D_MULTIPLE_LINE, BM_SETCHECK, 1, 0);
    else
        SendDlgItemMessage(Dialog, D_SINGLE_LINE, BM_SETCHECK, 1, 0);
    if(Bits & ES_PASSWORD)
        SendDlgItemMessage(Dialog, D_PASSWORD, BM_SETCHECK, 1, 0);
    if(Bits & ES_OEMCONVERT)
        SendDlgItemMessage(Dialog, D_CONVERT_OEM, BM_SETCHECK, 1, 0);
    if(Bits & ES_NOHIDESEL)
        SendDlgItemMessage(Dialog, D_KEEP_SELECTION, BM_SETCHECK, 1, 0);
    if(Bits & ES_AUTOVSCROLL)
        SendDlgItemMessage(Dialog, D_AUTO_VERT, BM_SETCHECK, 1, 0);
    if(Bits & ES_AUTOHSCROLL)
        SendDlgItemMessage(Dialog, D_AUTO_HORIZ, BM_SETCHECK, 1, 0);
    if(Bits & ES_READONLY)
        SendDlgItemMessage(Dialog, D_READ_ONLY, BM_SETCHECK, 1, 0);
    if(Bits & ES_WANTRETURN)
        SendDlgItemMessage(Dialog, D_WANT_RETURN, BM_SETCHECK, 1, 0);
    

    GlobalUnlock(InitData->StyleData);
    }



BOOL FAR PASCAL _export DndEditDialog(HWND Dialog, WORD Message, WORD wParam, LONG lParam)
    {
    switch(Message)
        {
        case    WM_INITDIALOG   :
            SetData(Dialog, lParam);
            UnPack(Dialog);
            return TRUE;
        case    WM_COMMAND      :
            switch(wParam)
                {
                case    IDOK    :
                    Pack(Dialog);
                    EndDialog(Dialog, TRUE);
                    break;
                case    IDCANCEL    :
                    EndDialog(Dialog, FALSE);
                    break;
                default :
                    return 0;
                }
            break;
        case    WM_DESTROY  :
            RemoveProp(Dialog, "HI");
            RemoveProp(Dialog, "LO");
        default:
            return FALSE;
        }
    return TRUE;
    }

