
unit DndEdit;
interface
{$IFDEF VER70}
uses Objects, OWindows, OMemory, ODialogs, WinProcs, WinTypes, Strings;
{$ELSE}
uses WObjects, WinProcs, WinTypes, Strings;
{$ENDIF}
const
    DRAG_INTERVAL = 100;
type
    TLRect    = record
        Left, Top, Right, Bottom
            : longint;
        end;
    TLPt    = record
        X, Y
            : longint;
        end;
    TDndEdit = object(TEdit)

        constructor Init(AParent : PWindowsObject;
            Id : integer; Title : PChar;
            x, y, w, h, TextLength : integer);
        procedure WMHScroll(var Message : TMessage);
            virtual wm_First + wm_HScroll;
        procedure WMVScroll(var Message : TMessage);
            virtual wm_First + wm_VScroll;
        procedure WMMouseMove(var Message:TMessage);
            virtual wm_First + wm_MouseMove;
        procedure WMSetCursor(var Message:TMessage);
            virtual wm_First + wm_SetCursor;
        procedure WMLButtonDown(var Message:TMessage);
            virtual wm_First + wm_LButtonDown;
        procedure WMLButtonUp(var Message:TMessage);
            virtual wm_First + wm_LButtonUp;
        procedure WMKeyDown(var Message:TMessage);
            virtual wm_First + wm_KeyDown;
        procedure EMLineScroll(var Message:TMessage);
            virtual wm_First + em_LineScroll;
        procedure EMSetSel(var Message:TMessage);
            virtual wm_First + em_SetSel;
        procedure WMLButtonDblClk(var Message:TMessage);
            virtual wm_First + wm_LButtonDblClk;
    private
        Grabbed, CaretIsHidden, CopyOnly
            : Boolean;
        SelectionStart, SelectionEnd
            : integer;
        InsertionPos
            : integer;
        CaretBlinkTime
            : word;
        FontHeight, FontWidth
            : integer;
        Drag, VScroll, HScroll
               : longint;
        DragAccelerate, DragSize
            : integer;
        HDrag, VDrag
            : integer;
        ScrollCorrection
            : longint;
        Cursor
            : HCursor;
        SelRect        : array [0..2] of TLRect;
        TextRect    : TRect;
        function  MemGetLine(LineNumber, LineLength : integer) : PChar;
        procedure MemFreeLine(LineMem : PChar; LineLength : integer);
        function  PlaceCaret(var CaretPos : TPoint) : boolean;
        procedure StartSelection;
        procedure ZeroSelection;
        procedure StopSelection;
        function  CharPosToX(DC : HDC; Pos : integer) : integer;
        function  InSelection(Point : TPoint) : boolean;
        procedure SetFontHeight;
        procedure SetSelectionRect;
        function  ClipScroll(Vertical, Horizontal : integer) : boolean;
        function  ShowCaret(ShowIt : boolean) : boolean;
    end;
    PDndEdit        = ^TDndEdit;


implementation


var
   Arrow, Cross, IBeam
          : HCursor;

{
SecretEditWord - extract a word from edit control's internal structure.

    This function depends upon the fact that the standard edit control stores
the offset of a data structure of internal information in the first two bytes
of extra window bytes.  The word at offset $12 in this structure happens to
contain the number of vertical lines of scrolling currently in effect and the
word at offset $3A happens to contain the number of horizontal characters
of scrolling currently in effect.
}
function SecretEditWord(HWindow : HWND; Offset : Word) : Word;
var
    DataSegment, DataOffset
        : Word;
    LongPointer
        : longint;                 
begin
    DataSegment := GetWindowWord(HWindow, GWW_HINSTANCE) OR $0001;
    DataOffset  := GetWindowWord(HWindow, 0);
    LongPointer := (longint(DataSegment) shl 16) OR (DataOffset + Offset);
    DataOffset  := (PInteger(LongPointer))^;
    SecretEditWord := DataOffset;
end;
 

function LPtInLRect(LRect : TLRect; LPt : TLPt) : boolean;
begin
    LPtInLRect    := FALSE;
    if LPt.Y < LRect.Top then
    else if LPt.Y > LRect.Bottom then
    else if LPt.X < LRect.Left then
    else if LPt.X > LRect.Right then
    else
        LPtInLRect    := TRUE;
end;

procedure    SwapInt(var A, B:integer);
var
    Temp    : integer;
begin
    Temp    := A;    A    := B;    B    := Temp;
end;

function Min(a,b:integer) : integer;
begin
    if a < b then
        Min := a
    else
        Min := b;
end;
function Max(a,b:integer) : integer;
begin
    if a > b then
        Max := a
    else
        Max := b;
end;

{ Init - Constructor for TDndEdit }

constructor TDndEdit.Init(AParent : PWindowsObject; Id : integer;
    Title : PChar; x, y, w, h, TextLength : integer);
begin
    TEdit.Init(AParent, Id, Title, x, y, w, h, TextLength, TRUE);
    Grabbed        := FALSE;
    CaretIsHidden  := FALSE;
    CopyOnly       := FALSE;
    Cursor         := 0;

    SetFontHeight;
    VScroll        := 0;
    HScroll        := 0;
    Drag           := 0;
    DragAccelerate := DRAG_INTERVAL;
    DragSize       := 1;

    ZeroSelection;
end;


function TDndEdit.ShowCaret(ShowIt : boolean) : boolean;
begin
    ShowCaret       := FALSE; { Assume nothing need be done }
    if ShowIt = CaretIsHidden then
        begin
        CaretIsHidden   := NOT CaretIsHidden;
        ShowCaret       := TRUE;
        if ShowIt = TRUE then
            WinProcs.ShowCaret(HWindow)
        else
            WinProcs.HideCaret(HWindow);
        end;
end;

function  TDndEdit.MemGetLine(LineNumber, LineLength : integer) : PChar;
var
   SizeP    : PInteger;
   Buffer   : PChar;
   Message  : TMessage;
begin
    { TPW's "GetLine" won't retrieve partial lines, so
      call em_GetLine the hard way.                      }
    GetMem(Buffer, LineLength+4+1);
    SizeP            := PInteger(Buffer);
    SizeP^           := LineLength;
    Message.Receiver := HWindow;
    Message.Message  := em_GetLine;
    Message.wParam   := LineNumber;
    Message.lParam   := longint(Buffer);
    DefWndProc(Message);
    Buffer[Message.Result]  := #0;
    MemGetLine       := Buffer;
end;

procedure TDndEdit.MemFreeLine(LineMem : PChar; LineLength : integer);
begin
    FreeMem(LineMem, LineLength+4+1);
end;

{ PlaceCaret - Calculate text caret position, return TRUE if dragging is called for. }

function    TDndEdit.PlaceCaret(var CaretPos : TPoint) : boolean;
var
    LineNumber, LineLength, CharPos, X, Y, MaxY, HalfWidth, LinesInWindow
        : integer;
    LineText
        : PChar;
    DC    : HDC;
    Coord, XCoord, MinXCoord
        : longint;
    CursorPos
        : TPoint;
begin
    GetCursorPos(CursorPos);
    ScreenToClient(HWindow, CursorPos);

    X            := integer(longint(CursorPos.X));
    Y            := integer(longint(CursorPos.Y));
    HDrag        := 0;
    VDrag        := 0;
    LinesInWindow:= (TextRect.bottom-TextRect.top) div FontHeight;
    { MaxY must only include integral number of lines }
    MaxY        := LinesInWindow * FontHeight + TextRect.top - 1;
    if X < TextRect.left then
        begin
        X        := TextRect.left;
        HDrag    := -1;
        end;
    if X > TextRect.right then
        begin
        X        := TextRect.right;
        HDrag    := 1;
        end;
    if Y < TextRect.top then
        begin
        Y        := TextRect.top;
        VDrag    := -1;
        end;
    if Y > MaxY then
        begin
        Y        := MaxY-2;
        VDrag    := 1;
        end;

    LineNumber   := (Y - TextRect.top + VScroll) div FontHeight;
    LineNumber   := Max(Min(LineNumber, GetNumLines-1), 0);
    Y            := (longint(LineNumber) * FontHeight) - VScroll + TextRect.top;
    DC           := GetDC(HWindow);
    InsertionPos := GetLineIndex(LineNumber);

    LineLength   := GetLineLength(LineNumber);
    LineText     := MemGetLine(LineNumber, LineLength);
    if LineText <> NIL then
        begin
        { A binary search might be better ... }
        CharPos    := 0;
        MinXCoord  := X - FontWidth;
        if MinXCoord < 0 then
            MinXCoord  := TextRect.Left;
        repeat
            Coord    := GetTextExtent(DC,LineText, CharPos);
            XCoord   := LoWord(Coord) + TextRect.left - HScroll;
            CharPos  := CharPos + 1;
        until (CharPos > StrLen(LineText)) OR (XCoord >= (MinXCoord));

        HalfWidth    := (TextRect.Right - TextRect.Left) div 2;
        if XCoord < (TextRect.Left + FontWidth div 2) then
            begin
            ScrollCorrection := (TextRect.Left - XCoord) + HalfWidth - FontWidth;
            HDrag            := -1;
            end
        else
            ScrollCorrection    := 0;

        if (ScrollCorrection = 0) AND (HDrag = 1) AND (XCoord < (TextRect.Right - HalfWidth)) then
            HDrag := 0;
        X    := Max(TextRect.Left,XCoord);
        InsertionPos    := InsertionPos + (CharPos-1);
        MemFreeLine(LineText, LineLength);
        end;
    ReleaseDC(HWindow, DC);
    CaretPos.X    := X;
    CaretPos.Y    := Y;

    PlaceCaret    := FALSE;
    if (HDrag <> 0) OR (VDrag <> 0) then
        PlaceCaret    := True;
end;

procedure    TDndEdit.ZeroSelection;
var
    i : integer;
begin
    for i := 0 to 2 do
        begin
        SelRect[i].top      := 0;
        SelRect[i].bottom   := 0;
        SelRect[i].left     := 0;
        SelRect[i].right    := 0;
        end;
end;

procedure    TDndEdit.StartSelection;
var
    StartPos, EndPos
        : integer;
begin
    ZeroSelection;
    GetSelection(StartPos, EndPos);
    SelectionStart    := StartPos;
    if StartPos = EndPos then
        ShowCaret(TRUE);
end;

procedure    TDndEdit.StopSelection;
var
    StartPos, EndPos
        : integer;
    CaretPos
        : TPoint;
    Message
        : TMessage;
begin
    GetSelection(StartPos, EndPos);
    Message.Message    := em_GetRect;
    Message.LParam    := LongInt(@TextRect);
    DefWndProc(Message);
    if SelectionStart = StartPos then
        SelectionEnd    := EndPos
    else
        SelectionEnd    := StartPos;
    SetSelectionRect;
    if StartPos <> EndPos then
        ShowCaret(FALSE);
    Grabbed    := FALSE;
end;

function    TDndEdit.InSelection(Point : TPoint) : boolean;
var
    i    : integer;
    LPt    : TLPt;
begin
    InSelection    := FALSE;
    if PtInRect(TextRect, Point) then
        begin
        LPt.X    := Point.x + HScroll;
        LPt.Y    := Point.y + VScroll;
        for i := 0 to 2 do
            if LPtInLRect(SelRect[i], LPt) then
                InSelection    := TRUE;
        end
end;


{
Description:
    Given a character position within an edit control, this function
returns the X coordinate of that character within the edit control window.
}
function    TDndEdit.CharPosToX(DC : HDC; Pos : integer) : integer;
var
    LineNumber, LineLength
        : integer;
    Buffer
        : PChar;
    SizeP    : PInteger;
    Coord      : longint;
    Message    : TMessage;
begin
    CharPosToX    := 0;
    LineNumber    := GetLineFromPos(Pos);
    LineLength    := Pos - GetLineIndex(LineNumber);

    Buffer        := MemGetLine(LineNumber, LineLength);

    if Buffer <> NIL then
        begin
        Buffer[LineLength]  := #0;
        Coord               := GetTextExtent(DC, Buffer, LineLength);
        CharPosToX          := LoWord(Coord);
        MemFreeLine(Buffer, LineLength);
        end
    else
        CharPosToX          := 0;
end;

procedure TDndEdit.SetFontHeight;
var
    Message
        : TMessage;
    Font
        : HFont;
    TM
        : TTextMetric;
    N    : array [0..1] of char;
    DC    : HDC;
begin
    Message.WParam    := 0;
    Message.LParam    := 0;
    Message.Receiver:= HWindow;
    Message.Message    := WM_GETFONT;
    DefWndProc(Message);
    Font         := HFont(Message.ResultLo);
    DC           := GetDC(HWindow);
    GetTextMetrics(DC, TM);
    N[0]         := 'n';
    N[1]         := #0;
    FontWidth    := LoWord(GetTextExtent(DC, @N, 1));
    ReleaseDC(HWindow, DC);
if TM.tmExternalLeading >0 then MessageBeep(1);
    FontHeight   := TM.tmHeight + TM.tmExternalLeading;
end;


procedure TDndEdit.SetSelectionRect;
var
    CaretPos
        : TPoint;
    StartLine, EndLine, CaretLine
        : integer;
    XStart, XEnd, XTemp
        : longint;
    i, X
        : integer;
    DC    : HDC;
    NLines
        : integer;

begin
    ZeroSelection;
    if SelectionStart <> SelectionEnd then
        begin
        GetCaretPos(CaretPos);
        CaretLine := (CaretPos.Y-TextRect.Top) div FontHeight;
        DC        := GetDC(HWindow);
        XStart    := CharPosToX(DC, SelectionStart) + TextRect.left;
        XEnd      := CharPosToX(DC, SelectionEnd) + TextRect.left;
        StartLine := GetLineFromPos(SelectionStart);
        EndLine   := GetLineFromPos(SelectionEnd);
        HScroll   := XEnd - CaretPos.X;
        VScroll   := (EndLine - CaretLine) * FontHeight;
        if SelectionEnd < SelectionStart then
            begin
            XTemp     := XStart;
            XStart    := XEnd;        XEnd    := XTemp;
            SwapInt(StartLine, EndLine);
            end;
        NLines               := 1 + EndLine - StartLine;
        SelRect[0].top       := StartLine * FontHeight + TextRect.top;
        SelRect[0].bottom    := SelRect[0].top + FontHeight;
        SelRect[0].left      := XStart;
        if NLines <= 1 then
            SelRect[0].right := XEnd
        else
            begin
            SelRect[0].right := MAXLONGINT;
            SelRect[2].left  := 0;
            SelRect[2].top   := FontHeight * EndLine + TextRect.top;
            SelRect[2].bottom:= SelRect[2].top + FontHeight;
            SelRect[2].right := XEnd;
            if NLines > 2 then
                begin
                SelRect[1].left    := 0;
                SelRect[1].right   := MAXLONGINT;
                SelRect[1].top     := SelRect[0].bottom;
                SelRect[1].bottom  := SelRect[2].top;
                end;
            end;

        ReleaseDC(HWindow, DC);
        end;

end;

procedure TDndEdit.WMMouseMove(var Message:TMessage);
var
    i    : integer;
    Msg
        : TMsg;
    NewTick
        : Longint;
    HiddenCaret, MoveCaret
        : Boolean;
    CaretPos
        : TPoint;
begin
    HiddenCaret    := FALSE;
    MoveCaret      := FALSE;
    if not Grabbed then
        begin
        DefWndProc(Message);
        Cursor     := 0;
        if InSelection(TPoint(Message.LParam)) then
            Cursor := Arrow;
        end
    else
        begin
        if Message.LParam <> -1 then { if "real" mouse move }
            begin
            MoveCaret    := TRUE;
            DefWndProc(Message);
            if PlaceCaret(CaretPos) then
                begin
                if Drag = 0 then
                    begin
                    DragAccelerate    := DRAG_INTERVAL;
                    DragSize          := 1;
                    Drag              := abs(GetTickCount)+1;
                    end
                else
                    begin
                    if DragSize > 1 then
                        DragSize := Min(1,DragSize div 3)
                    else
                        DragAccelerate    := min(DRAG_INTERVAL, DragAccelerate+10);
                    end
                end
            else
                   Drag    := 0;
            end;
        if (Drag <> 0)
             AND
            (NOT PeekMessage(Msg, HWindow, wm_LButtonUp, wm_LButtonUp, pm_NoRemove))
            AND (NOT PeekMessage(Msg, HWindow, wm_MouseMove, wm_MouseMove, pm_NoRemove)) then
            begin
            NewTick        := abs(GetTickCount);
            if abs(NewTick - Drag) > DragAccelerate then
                begin
                DragAccelerate    := DragAccelerate * 6 div 7;
                Drag              := NewTick+1; { add 1 so it can never be zero }
                if ScrollCorrection <> 0 then
                    HDrag         := -(ScrollCorrection div FontWidth);
                if DragAccelerate < 10 then
                    DragSize      := Min((TextRect.bottom-TextRect.top)div FontHeight, DragSize + 1);
                Scroll(HDrag, VDrag * DragSize);
                MoveCaret         := FALSE;
                end;
            PostMessage(HWindow, WM_MouseMove,
                    Message.WParam, -1);
            end;
        end;
    if MoveCaret then
        SetCaretPos(CaretPos.X, CaretPos.Y);
end;

procedure TDndEdit.WMSetCursor(var Message:TMessage);
begin
if Cursor <> 0 then
    SetCursor(Cursor)
else
    DefWndProc(Message);
end;

procedure TDndEdit.WMLButtonDown(var Message:TMessage);
var
    CaretPos    : TPoint;
begin
    if InSelection(TPoint(Message.LParam)) then { if grabbing selection }
        begin
        CopyOnly    := ((GetKeyState(VK_CONTROL) AND $8000) = $8000);
        SetCapture(HWindow);
        Cursor    := Cross;  { Not really required since mouse is captured }
        SetCursor(Cross);
        ShowCaret(FALSE);
        CreateCaret(HWindow, 1, 2*GetSystemMetrics(sm_CXBORDER), FontHeight);
        PlaceCaret(CaretPos);
        SetCaretPos(CaretPos.X, CaretPos.Y);
        CaretBlinkTime    := GetCaretBlinkTime;
        SetCaretBlinkTime($FFFF);
        { Explicit ShowCaret() required after CreateCaret() }
        ShowCaret(TRUE);
        Grabbed        := TRUE;
        end
    else
        begin
        DefWndProc(Message);
        StartSelection;
        end;
end;


procedure TDndEdit.WMLButtonUp(var Message:TMessage);
var
    SelectionSize, StartPos, EndPos
        : integer;
    Buffer
        : PChar;
begin
    Drag    := 0;
    if Grabbed then
        begin
        ShowCaret(FALSE);
        ReleaseCapture;
        CreateCaret(HWindow, 0, 2*GetSystemMetrics(sm_CXBORDER), FontHeight);
        SetCaretBlinkTime(CaretBlinkTime);
        ShowCaret(TRUE);
        Cursor    := 0;

        StartPos  := SelectionStart;
        EndPos    := SelectionEnd;
        if StartPos > EndPos then
           SwapInt(StartPos, EndPos);
        if (InsertionPos >= StartPos) AND (InsertionPos < EndPos) then
            begin
            Grabbed    := FALSE;
            SetSelection(InsertionPos, InsertionPos);
            end
        else
            begin
            SelectionSize    := abs(StartPos-EndPos);
            GetMem(Buffer, SelectionSize+1);
            GetSubText(Buffer, StartPos, EndPos);
            if CopyOnly = FALSE then
                begin
                DeleteSelection;
                if InsertionPos >= StartPos then
                    InsertionPos    := InsertionPos - SelectionSize;
                end;
            SetSelection(InsertionPos, InsertionPos);

            Insert(Buffer);

            Grabbed    := FALSE;
            SetSelection(InsertionPos, InsertionPos+SelectionSize);
            FreeMem(Buffer, SelectionSize+1);
            end;
        { Explicit ShowCaret() required after CreateCaret() }
        end
    else
        StopSelection;
    DefWndProc(Message);

end;

procedure    TDndEdit.WMHScroll(var Message : TMessage);

begin
    DefWndProc(Message);
    HScroll     := SecretEditWord(HWindow, $16);
end;

procedure    TDndEdit.WMVScroll(var Message : TMessage);

begin
    DefWndProc(Message);
    VScroll     := SecretEditWord(HWindow, $12) * FontHeight;
end;


procedure TDndEdit.WMKeyDown(var Message:TMessage);
var
    StartPos, EndPos
        : integer;
begin
    if Grabbed AND (Message.WParam = VK_CONTROL) then
        CopyOnly    := TRUE;

    if not Grabbed then
        begin
        GetSelection(StartPos, EndPos);
        if (StartPos <> EndPos) AND ((GetKeyState(VK_SHIFT) AND $8000) = 0) AND
            ((Message.WParam = VK_LEFT) OR (Message.WParam = VK_RIGHT)) then
            begin
            if Message.WParam = VK_LEFT then
                SetSelection(StartPos, StartPos)
            else if Message.WParam = VK_RIGHT then
                SetSelection(EndPos, EndPos)
            end
        else
            begin
            if StartPos = EndPos then
                StartSelection;
            DefWndProc(Message);
            GetSelection(StartPos, EndPos);
            if StartPos = EndPos then
                StartSelection
            else
                StopSelection;
            end;
        end;
end;

procedure TDndEdit.WMLButtonDblClk(var Message:TMessage);
begin
    DefWndProc(Message);
    StartSelection;
    StopSelection;

end;

procedure TDndEdit.EMSetSel(var Message:TMessage);
var
    StartPos, EndPos
        : integer;
begin

    DefWndProc(Message);
    StartSelection;
    StopSelection;
end;


procedure TDndEdit.EMLineScroll(var Message:TMessage);
var
    LineScroll, CharScroll
        : integer;
    Changed
        : boolean;
    CaretPos
        : TPoint;
begin

    if Grabbed then
        begin
        Changed    := FALSE;
        LineScroll := integer(Message.LParamLo);

        if LineScroll > 0 then
            begin
            LineScroll  := Max(0, Min(LineScroll, GetNumLines - (VScroll div FontHeight)
                        - ((TextRect.bottom-TextRect.top) div FontHeight div 2)));
            Message.LParamLo := LineScroll;                          
            end;


        CharScroll := integer(Message.LParamHi);
        Changed    := ClipScroll(LineScroll*FontHeight,
                                CharScroll*FontWidth);
        if Changed then
            begin
            ShowCaret(FALSE);
            DefWndProc(Message);
            PlaceCaret(CaretPos);
            SetCaretPos(CaretPos.X, CaretPos.Y);
            ShowCaret(TRUE);
            end
        else
            begin
            DragAccelerate  := DRAG_INTERVAL;
            DragSize        := 1;
            end;
        end
    else
        DefWndProc(Message);
end;

function  TDndEdit.ClipScroll(Vertical, Horizontal : integer) : boolean;
    function  RangeClip(Point, Low, High : longint) : longint;
    begin
        if Point < Low then
            RangeClip    := Low
        else if Point > High then
            RangeClip    := High
        else
            RangeClip    := Point;
    end;

var
    NewVScroll, NewHScroll, MaxVScroll, MaxHScroll
        : longint;
    MaxVScrollLines
        : integer;                
begin
    ClipScroll       := FALSE;    { Assume no change in scroll position }

    MaxVScroll := (GetNumLines-1) * FontHeight;
    MaxHScroll       := SecretEditWord(HWindow, $3A) - 1;

    NewVScroll       := RangeClip(VScroll+Vertical, 0, MaxVScroll);
    NewHScroll       := RangeClip(HScroll+Horizontal, 0, MaxHScroll);

    if (NewVScroll <> VScroll) OR (NewHScroll <> HScroll) then
        ClipScroll    := TRUE;

    HScroll        := NewHScroll;
    VScroll        := NewVScroll;
end;


begin
    Arrow          := LoadCursor(0, idc_Arrow);
    Cross          := LoadCursor(0, idc_UpArrow);
    IBeam          := LoadCursor(0, idc_IBeam);
end.
