/**************************************************

  Test rig for MessageBoxEx function

**************************************************/

#include <windows.h>
#include "msgboxex.h"
#include "testrc.h"

HINSTANCE hInst;		/* Application instance handle */
int wType;				/* Type mask for basic styles */
int wExType;			/* Type mask for extended styles */
int idmIconType;		/* Selected icon type */
int idmButtonType;	/* Selected button type */
HBRUSH hbrDlg;			/* Background color for dialog */
HMENU hMenu;			/* Handle of menu */
FARPROC fpMsgHook;	/* MessageBoxEx hook procedure */
FARPROC lpfMsgBoxHook;	/* MessageBox procedure addr */

char * szShort =
		"This is a short message.";
char * szWrapped =
		"This is a much longer message that will be "
		"wrapped over two lines.";
char * szMulti =
		"This message has multiple lines.\nEach line is "
		"terminated by a newline character.\n\nBlank "
		"lines are possible too.";

char * szApp = "MessageBoxEx Test Rig";

LONG FAR PASCAL TestRigProc( HWND, UINT,
							WPARAM, LPARAM );
LONG FAR PASCAL MsgBoxHook( HWND, UINT,
							WPARAM, LPARAM );

void DoCommand( HWND hwnd, int id );

/***************************************************/
/* WinMain														*/
/***************************************************/

int PASCAL WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPSTR lpszCmdLine,
	int nCmdShow )
{
	WNDCLASS wc;
	HWND hwnd;
	MSG msg;

	hInst  				= hInstance;
	idmIconType 		= IDM_NOICON;
	wType 				= MB_OK;
	idmButtonType 		= IDM_OK;
	wExType 				= 0;
	fpMsgHook 			= NULL;

	wc.style				= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= TestRigProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInst;
	wc.hIcon				= NULL;
	wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground	= (HBRUSH)( COLOR_APPWORKSPACE + 1 );
	wc.lpszMenuName	= NULL;
	wc.lpszClassName	= "testrig";
	if( !RegisterClass( &wc ) )
		return( FALSE );

	hMenu = LoadMenu( hInst, RIGMENU );

	hbrDlg = CreateSolidBrush(
					GetSysColor( COLOR_BTNFACE ) );

	hwnd = CreateWindow( "testrig", szApp,
		WS_OVERLAPPEDWINDOW, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT,
		CW_USEDEFAULT, NULL, hMenu, hInst, NULL );
	ShowWindow( hwnd, nCmdShow );
	UpdateWindow( hwnd );

	lpfMsgBoxHook = MakeProcInstance(
			(FARPROC)MsgBoxHook, hInst );

	CheckMenuItem( hMenu, idmIconType,
		MF_BYCOMMAND|MF_CHECKED );
	CheckMenuItem( hMenu, idmButtonType,
		MF_BYCOMMAND|MF_CHECKED );

	while( GetMessage( &msg, NULL, 0, 0 ) ) {
		TranslateMessage( &msg );
		DispatchMessage( &msg );
		}
	return( (int)msg.wParam );
}

/***************************************************/
/* TestRigProc													*/
/***************************************************/

LONG FAR PASCAL TestRigProc(
	HWND hwnd,
	UINT message,
	WPARAM wParam,
	LPARAM lParam )
{
	switch( message )
		{
		case WM_CLOSE:
			DestroyWindow( hwnd );
			PostQuitMessage( 0 );
			break;

		case WM_COMMAND:
			DoCommand( hwnd, wParam );
			break;
		}
	return( DefWindowProc( hwnd, message, wParam, lParam ) );
}

/***************************************************/
/* DoCommand													*/
/***************************************************/

void DoCommand( HWND hwnd, int id )
{
	switch( id )
		{
		case IDM_OK:
		case MB_OKCANCEL:
		case MB_YESNO:
		case MB_YESNOCANCEL:
		case MB_ABORTRETRYIGNORE:
		case MB_RETRYCANCEL:
			wType = (wType & ~MB_TYPEMASK) |
						( id == IDM_OK ? MB_OK : id );
			CheckMenuItem( hMenu, idmButtonType,
						MF_BYCOMMAND|MF_UNCHECKED );
			CheckMenuItem( hMenu, id,
						MF_BYCOMMAND|MF_CHECKED );
			idmButtonType = id;
			break;

		case IDM_NOICON:
			id = 0;

		case MB_ICONEXCLAMATION:
		case MB_ICONHAND:
		case MB_ICONQUESTION:
		case MB_ICONASTERISK:
			wType = (wType & ~MB_ICONMASK) | id;
			CheckMenuItem( hMenu, idmIconType,
					MF_BYCOMMAND|MF_UNCHECKED );
			CheckMenuItem( hMenu,
					id, MF_BYCOMMAND|MF_CHECKED );
			idmIconType = id;
			break;

		case IDM_SYSMODAL:
			wType ^= MB_SYSTEMMODAL;
			CheckMenuItem( hMenu, id, MF_BYCOMMAND|
				( (wType & MB_SYSTEMMODAL) ?
						MF_CHECKED : MF_UNCHECKED ) );
			break;

		case IDM_EX_HELP:
			if( wExType & MB_EX_HELP )
				{
				wExType &= ~MB_EX_HELP;
				fpMsgHook = NULL;
				CheckMenuItem( hMenu, id,
						MF_BYCOMMAND|MF_UNCHECKED );
				}
			else
				{
				wExType |= MB_EX_HELP;
				fpMsgHook = lpfMsgBoxHook;
				CheckMenuItem( hMenu, id,
						MF_BYCOMMAND|MF_CHECKED );
				}
			break;

		case IDM_SHORT:
			MessageBoxEx( hwnd, szShort, szApp, wType,
							  wExType, (WNDPROC)fpMsgHook );
			break;

		case IDM_WRAPPED:
			MessageBoxEx( hwnd, szWrapped, szApp, wType,
							  wExType, (WNDPROC)fpMsgHook );
			break;

		case IDM_MULTI:
			MessageBoxEx( hwnd, szMulti, szApp, wType,
							  wExType, (WNDPROC)fpMsgHook );
			break;

		case IDM_EXIT:
			SendMessage( hwnd, WM_CLOSE, 0, 0L );
			break;
		}
}

/***************************************************/
/* MsgBoxHook													*/
/***************************************************/

LONG FAR PASCAL MsgBoxHook(
	HWND hwnd,
	UINT message,
	WPARAM wParam,
	LPARAM lParam )
{
	switch( message )
		{
		case WM_COMMAND:
			if( wParam == IDEXHELP )
				WinHelp( hwnd, "WINHELP.HLP",
							HELP_INDEX, 0L );
			break;

		case WM_CTLCOLOR: {
			LOGBRUSH logBrush;

			switch( HIWORD( lParam ) )
				{
				case CTLCOLOR_LISTBOX:
				case CTLCOLOR_EDIT:
				case CTLCOLOR_MSGBOX:
					return( (BOOL)NULL );

				default:
					GetObject( hbrDlg, sizeof( LOGBRUSH ),
								  (LPSTR)&logBrush );
					SetBkColor( (HDC)wParam, logBrush.lbColor );
					return( (BOOL)hbrDlg );
				}
			}
		}
	return( 0L );
}

