#include "biglb.h"
#ifdef __BORLANDC__
#pragma hdrstop
#endif


LRESULT CALLBACK _export BigLBWndFn(HWND hWnd,
         UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;  /* note : no static data */
  HWND hChild;
  TEXTMETRIC tm;
  RECT clRect;
  LPMEASUREITEMSTRUCT mis;
  LPCOMPAREITEMSTRUCT cis;
/*  LPDELETEITEMSTRUCT delis; */
  LPDRAWITEMSTRUCT dis;
  LPCREATESTRUCT crs;
  HFONT hProp,hOldFont;
  FARPROC lpfnOldproc;
  char wrk[16];
  BOOL handled = FALSE;

/* deliver all listbox messages to ownerdraw control... */
  if (GetWindow(hWnd,GW_CHILD) != NULL)
    if (msg >= LB_ADDSTRING && msg <= LAST_LB_MSG)
      return SendMessage(GetWindow(hWnd,GW_CHILD), msg,
                                        wParam,lParam);
  switch (msg)
  {
    case WM_CREATE : /* copy the styles and sizes */
        crs = (LPCREATESTRUCT) lParam;
        /* to set a different font, pass a pointer
           to a font handle in CreateWindow() */
        if (crs->lpCreateParams != NULL)
        {
          hProp = *((HFONT FAR *)crs->lpCreateParams);
          LoadString(ModuleInstance,IDS_CTL_PROP,wrk,16);
          SetProp(hWnd,wrk,hProp);
        }
        else
          hProp = NULL;
         /* parent of OWNERDRAW listbox (not invisible, but
           behind the listbox without a border)         */
        SetWindowLong(hWnd,GWL_STYLE,WS_CHILD);
        hChild = CreateWindow("LISTBOX", crs->lpszName,
              (crs->style | LBS_OWNERDRAWFIXED), 
              crs->x,crs->x,crs->cx,crs->cy,hWnd,crs->hMenu,
              crs->hInstance, crs->lpCreateParams);
        if (hProp != NULL)
        {
          /* change the font in the child */    
          SendMessage(hChild,WM_SETFONT,(WPARAM)hProp,NULL); 
          /* clean up the property list */   
          LoadString(ModuleInstance,IDS_CTL_PROP,wrk,16);
          RemoveProp(hWnd,wrk);
        }
        if (hChild == NULL)
          return -1L;
        /* change LISTBOX's default wndproc
         * (save the old function pointer)  */
        lpfnOldproc = (FARPROC) SetWindowLong(hChild,
                      GWL_WNDPROC, (LONG)BigLB_SubProc);
        /*  save the old address of the LISTBOX's wndproc
         *  in window extra bytes of the BIGLB control...
         *  rest of the extra bytes initialized to zero */
        SetWindowLong(hWnd, GWL_OLDPROC, (LONG)lpfnOldproc);
        handled = TRUE;
        break;
    case WM_DESTROY: /* clean up */
        hChild = GetWindow(hWnd,GW_CHILD);
        SendMessage(hChild,LB_RESETCONTENT,NULL,NULL);
        break;
/*  for OWNERDRAW listboxes, one must handle WM_MEASUREITEM
 *  for creation (and drawing if style is
 *  LBS_OWNERDRAWVARIABLE) WM_COMPAREITEM for LB_ADDSTRING
 *  in a sorted listbox WM_DRAWITEM for painting
 *  WM_DELETEITEM when deleting a string. note: wParam means
 *  something different under Windows 3.0 and Windows 3.1
 *  so don't look at it when servicing these messages!!! */
    case WM_MEASUREITEM:
    /*  if wParam == -1, the system is requesting the
     *  dimensions of an edit control in an owner draw
     *  control. if wParam == 0, the measure message is
     *  being sent by a menu otherwise, wParam is the ID of
     *  the control that sent the WM_MEASURE item message.*/
        mis = (LPMEASUREITEMSTRUCT) lParam;
        LoadString(ModuleInstance,IDS_CTL_PROP,wrk,16); 
        hProp = (HFONT) GetProp(hWnd,wrk);
        hDC = GetDC(hWnd);
        if (hProp != NULL)
          hOldFont = (HFONT)SelectObject(hDC,hProp);
        GetTextMetrics(hDC,&tm);
        if (hProp != NULL)
          SelectObject(hDC,hOldFont);
        ReleaseDC(hWnd,hDC);
        /*  give height of listbox line (of display font) */
        mis->itemHeight = tm.tmHeight;
        mis->CtlType = ODT_LISTBOX;
        mis->CtlID = GetWindowWord(hWnd,GWW_ID);
        return (TRUE);
    case WM_COMPAREITEM :
        cis = (LPCOMPAREITEMSTRUCT) lParam;
        return lstrcmp((LPSTR) cis->itemData1,
                 (LPSTR) cis->itemData2);
/*
 * This message is ignored because of the way
 * that global memory management works
 *
 *  case WM_DELETEITEM:
 *      delis = (LPDELETEITEMSTRUCT) lParam;
 *      farfree((LPSTR)delis->itemData);
 *      return (TRUE);
 */
    case WM_DRAWITEM :
        dis = (LPDRAWITEMSTRUCT) lParam;
        SaveDC(dis->hDC);
        /* dis->itemData points to the string */
        if (dis->itemAction & ODA_DRAWENTIRE)
        {   
          /* check for control enabled-ness */
          if (!IsWindowEnabled(hWnd))
            SetTextColor(dis->hDC,
              GetSysColor(COLOR_GRAYTEXT));
          /*  fill the rectangle with the background color
           *  (erases the old focus rectangle...)       */
          ExtTextOut(dis->hDC, dis->rcItem.left,
                dis->rcItem.top, ETO_OPAQUE, &dis->rcItem,
                     NULL, 0, NULL);
          /* if you don't need tab expansion, just draw the
           * text with ExtTextOut                       */
          DrawText(dis->hDC,(LPSTR)dis->itemData,
                   lstrlen((LPSTR)dis->itemData),
                   &dis->rcItem, (DT_EXPANDTABS|DT_LEFT|
                    DT_NOCLIP|DT_SINGLELINE|DT_VCENTER));
          if (dis->itemState & ODS_SELECTED)
            InvertRect(dis->hDC,&dis->rcItem);
          if (dis->itemState & ODS_FOCUS)
            DrawFocusRect(dis->hDC,&dis->rcItem);
          handled = TRUE;
        }
        /* if selecting, invert that line...
         * (invert the old, and invert the new...)  */
        if (dis->itemAction & ODA_SELECT)
        {
          InvertRect(dis->hDC,&dis->rcItem);
          handled = TRUE;
        }
        /* dis->itemID < 0 means the listbox is empty */
        if (dis->itemAction & ODA_FOCUS &&
                (int) dis->itemID >= 0)
        {
          DrawFocusRect(dis->hDC,&dis->rcItem);
          handled = TRUE;
        }
        RestoreDC(dis->hDC,-1);
        return (handled);
    case WM_COMMAND :
        /* echo notification messages to parent window */
        return (SendMessage(GetParent(hWnd), WM_COMMAND,
                GetWindowWord(hWnd,GWW_ID),
                  MAKELONG(hWnd, HIWORD(lParam))));
    case WM_SIZE:  /* resize the listbox */
        GetClientRect(hWnd,&clRect);
        MoveWindow(GetWindow(hWnd,GW_CHILD), clRect.left,
          clRect.top, LOWORD(lParam), HIWORD(lParam), TRUE);
        break;
    case WM_SETFONT:
        handled = TRUE;
        break;
    case WM_SETFOCUS:
        /* pass the focus along to the child LISTBOX */
        SetFocus(GetWindow(hWnd,GW_CHILD));
        handled = TRUE;
        break;
    case WM_SETREDRAW:
        /* pass the redraw state on to the child LISTBOX */
        return (SendMessage(GetWindow(hWnd,GW_CHILD),
                  msg,wParam,lParam));
    case WM_ENABLE :
        /* ask for redraw when the enabled state changes. */
    case WM_PAINT:
       /* fixes a re-draw error with Windows 3.0 */
        InvalidateRect(GetWindow(hWnd,GW_CHILD),NULL,TRUE);
        break;
  }
  if (handled)  
    return 0L;  /* message handled */
 /* else, do the default processing... */
 return DefWindowProc(hWnd,msg,wParam,lParam);
}


