/*   Title:  DEMOSTEP.C                              *
 *  Author:  John Reynolds                           *
 * Purpose:  Exercise the Stepper Motor Data         *
 *           Acquisition System.                     *
 * Copyright 1991 Magnetic Technologies Corporation  *
 *           Rochester, New York  14625              */

#include <conio.h>
#include <math.h>
#include "stepisr.h"
#include "printdio.h"
#include "ioutils.h"

unsigned steps_moved;
long int steps;
float velocity, accel, value;
char cmd;
int clk_hertz;
float getvalue(char param[10]);

void main()
{
  int i, LPTport;

  steps = 400;      /* default values  */
  velocity = 60;    /* for 400 steps   */
  accel = 180;      /* in 10 seconds   */
  clk_hertz = 128;
  sound_on = TRUE;

  /* Select parallel port */
  clrscr();
  puts("\nAvalable LPT ports:\n");
  for (i=0; i<3; i++)
   printf("LPT%d= %04x\n",i+1, peek(0x00,0x0408 +i*2));

  printf("\nUse which LPT port (1-3): ");
  scanf("%d",&LPTport);

  if (!use_port_LPT(LPTport))
  {
    printf("\nLPT%d port unavailable !!\n\n",
            LPTport);
    exit();
  } /* if bad port */

  init_io(LPTport);

  while (TRUE)
  {
   clrscr();
   printf("<C>lock Frequency = %d Hertz\n", clk_hertz);
   printf("<T>oggle Sound [currently %s]\n\n",
           sound_on ? "ON": "OFF");
   printf("<S>teps = %ld\n", steps);
   printf("<V>elocity = %1.4f\n", velocity);
   printf("<A>ccel = %1.4f\n\n",  accel);
   printf("Change which C,T,S,V,A, <R>un motor, "
          "or <Q>uit ? ");

   cmd = toupper(getch());
   switch(cmd)
   {
     case 'C':
       printf("\n\nSelect clock speed: [1] 128 Hz, (2)"
              " 1024 Hz, (3) 4096 Hz >> ");
       scanf("%d",&clk_hertz);
       switch(clk_hertz)
       {
         case 2:  clk_hertz = 1024;
                  break;
         case 3:  clk_hertz = 4096;
                  break;
         default: clk_hertz = 128;
                  break;
       } /* switch */
       break; /* case C */
     case 'T':
        sound_on = !sound_on;
        break; /* case T */
     case 'S':
       steps = (int) getvalue("Steps");
       break; /* case S */
     case 'V':
       velocity = getvalue("Velocity");
       break; /* case V */
     case 'A':
       accel = getvalue("Accel");
       break; /* case A */
     case 'Q':
       exit();
       break; /* case Q */
     case 'R':
       printf("\n\nExecuting Motor Move. . .");
       steps_moved =
       move_stepper(steps, velocity, accel,
                    clk_hertz);

       if (keybreak) puts("KEYBREAK STOP!");
       else if (*eot_stop_ptr)
              puts("END OF TRAVEL STOP!");

       printf("\n%d software steps moved.\n",
               steps_moved);
       printf("Hit any key to continue. . .");
       getch();
       break; /* case R */
   } /* switch */
  } /* while cmd */
} /* main() */

/*--------------------------*/
float getvalue(char param[10])
{
  char value[7];

  printf("\n\nInput new value for %s >> ",param);
  scanf("%s",&value);
  return(atof(value));
} /* getvalue() */
