/*   Title:  PRINTDIO.C                              *
 *  Author:  John Reynolds                           *
 * Purpose:  Provide Digital I/O to outside world    *
 *           via parallel port.                      *
 * Copyright 1991 Magnetic Technologies Corporation  *
 *           Rochester, New York  14625              */

#include "printdio.h"
#include <dos.h>

static unsigned int active_port;

/*----------------------------------*/
int use_port_LPT(unsigned char number)
{
  /* bios memory info: 408h = address of LPT1 port
                       40Ah = address of LPT2 port
                       40Ch = address of LPT3 port */
  if (number > 3) return 0;
  active_port = peek(0x00, 0x0408 + 2*(number - 1));
  return active_port;
} /* use_port_LPT() */

/*-----------------*/
void clear_all_bits()
{
  if (active_port)
    outport(active_port, 0x00); /* all pins = 0 VDC */
} /* clear_all_bits() */

/*---------------*/
void set_all_bits()
{
  if (active_port)
   outport(active_port, 0xFF); /* all pins = 5 VDC */
} /* set_all_bits */

/*------------------------------*/
void set_bit(unsigned char bit0_7)
{
  if (active_port)
    set_masked_bits(0x01 << bit0_7);
} /* turn_on_bit() */

/*--------------------------------*/
void clear_bit(unsigned char bit0_7)
{
  if (active_port)
    clear_masked_bits(0x01 << bit0_7);
} /* turn_off_bit() */

/*---------------------------------------*/
void set_bit_pattern(unsigned char pattern)
{
  if (active_port)
    outportb(active_port, pattern);
} /* set_bit_pattern() */

/*------------------------------------*/
void set_masked_bits(unsigned char mask)
/* bits = 1 in mask will be set,
   others left unchanged        */
{
  if (active_port)
    outportb(active_port, mask | inportb(active_port));
} /* set_masked_bits() */

/*--------------------------------------*/
void clear_masked_bits(unsigned char mask)
/* bits = 1 in mask will be cleared,
   others left unchanged            */
{
  if (active_port)
    outportb(active_port,
       (mask ^ 0xff) & inportb(active_port));
} /* clear_masked_bits() */

/*----------------------------*/
unsigned char read_output_byte()
/* returns current output settings */
{
  if (active_port)
    return inportb(active_port);
  else return 0;
} /* read_outputs() */

/*-----------------------------------------------*/
unsigned char read_output_bit(unsigned char bit0_7)
/* return 0 if bit is low, 1 if high */
{
  if (active_port)
    return(inportb(active_port) & (0x01 << bit0_7)
            ? 1:0);
  else return 0;
} /* read_output_bit() */

/*----------------------*/
unsigned char input_byte()
/* read printer ports and twiddle the bits */
{
  unsigned char inbyte;

  if (active_port)
  {
    /* Read STATUS port, keep bits 7,6,5,4,3 */
      inbyte = inportb(active_port + 1) & 0xf8;
    /* Read CONTROL port, keep bits 2,1,0 */
      inbyte |= inportb(active_port + 2) & 0x07;
    /* convert active low inputs to active high */
      inbyte ^= 0x83;
    return(inbyte); /* return input byte */
  } /* if active_port */
  else return 0;
} /* input_byte() */

/*-----------------------------------------*/
unsigned char input_bit(unsigned char bit0_7)
/* return 0 if bit is low, 1 if high */
{
  if (active_port)
    return( input_byte() & (0x01 << bit0_7) ? 1:0);
  else return 0;
} /* input_bit() */
