/*   Title:  RTCLOCK.C                               *
 *  Author:  John Reynolds                           *
 * Purpose:  Routines to tap the Real Time Clock     *
 *           periodic interrupt which can be set     *
 *           to run at 128, 1024, or 4096 Hertz      *
 *    Note:  EnableRtcInts(), DisableRtcInts(), and  *
 *           RtcHook() were originally coded in      *
 *           Pascal and were Copyright 1990 by       *
 *           Kenneth Roach and published in          *
 *           Dr. Dobb's Journal, June 1991, page 88. *
 *           Modified by John Reynolds to allow      *
 *           selection of three clock frequencies    *
 *           (128 Hz, 1024 Hz, and 4096 Hz).         *
 * Copyright 1991 Magnetic Technologies Corporation  *
 *           Rochester, New York  14625              */

#include <dos.h>
#include "rtclock.h"

#define   CMOSFLAG   0x70
#define   CMOSDATA   0x71
#define   STATUSA    0x0A
#define   RTC_VEC    0x70
#define   IMR2       0xA1
#define   CMD1       0x20
#define   CMD2       0xA0
#define   EOI        0x20
#define   RTC_MASK   0xFE
#define   STATUSB    0x0B
#define   STATUSC    0x0C
#define   RTC_FLAG   0x40

int RtcCounter;   /* total number of Rtc ISR calls */
unsigned char OldMaskA, OldMaskB;

void interrupt (*OldRtcVec)(void);
void (*NewRtcVec)(void);

/*------------------------*/
void interrupt RtcHook(void)
{
   disable(); /* asm cli */
   outportb(CMOSFLAG,STATUSC); /* Is this interrupt */
   if (inportb(CMOSDATA) & 0x40)  /*   for us?     */
   {
     NewRtcVec();
     /* signal end of interrupt to 8259s */
     outportb(CMD1, EOI);  /* primary 8259 */
     outportb(CMD2, EOI);  /* secondary 8259 */
   } /* if interrupt for us */
   else OldRtcVec();
   enable(); /* asm sti */
 } /* interrupt Rtc_hook() */

/*------------------------------*/
void EnableRtcInts(int ClockSpeed)
{
  /* change interrupt vector */
    OldRtcVec = getvect(RTC_VEC);
    setvect(RTC_VEC, RtcHook);
  /* enable clock interrupt */
    outportb(IMR2, inportb(IMR2) & RTC_MASK);
    outportb(CMOSFLAG, STATUSA);
  /* select clock frequency */
    OldMaskA = inportb(CMOSDATA);
    outportb(CMOSFLAG, STATUSA);
    outportb(CMOSDATA, OldMaskA & (ClockSpeed | 0xf0));
  /* enable periodic interrupts */
    outportb(CMOSFLAG, STATUSB);
    OldMaskB = inportb(CMOSDATA);
    outportb(CMOSFLAG, STATUSB);
    outportb(CMOSDATA, OldMaskB | RTC_FLAG);
} /* EnableRtcInts() */

/*---------------------*/
void DisableRtcInts(void)
{
  outportb(CMOSFLAG, STATUSB);
  outportb(CMOSDATA, OldMaskB);
  outportb(CMOSFLAG, STATUSA);
  outportb(CMOSDATA, OldMaskA);
  outportb(IMR2, inportb(IMR2) & !RTC_MASK);
  setvect(RTC_VEC, OldRtcVec);
} /* DisableRtcInts() */
