// Listing 1 - HARDINT.C
// HARDINT.C written by Guy Eddon, 1992
// Monitors hardware interrupts in real time
// Microsoft C 6.0
// cl /Gs hardint.c /link /NOE

#include <dos.h> 
#include <conio.h>

#define TICK 5

void (interrupt far *old_int8)(void);
void far unsg_to_str(unsigned, char far *, int);
void far moutchar(int, int, char, unsigned char);
void far mxyputs(int, int, char far *, unsigned char);
void far box(int, int, int, int);
void set_vid_mem(void);
char cputype(void);

#define MK_FP(seg,ofs) ((void far *) \
        (((unsigned long)(seg) << 16) | (unsigned)(ofs)))
#define NORM_VID 0x07
#define NORMI_VID 0x0F

unsigned int status, bit, p8259, port[2] = { 0x21, 0xA1 };
char col, buffer[3], tick = 0, far *vid_mem, cpu;

void interrupt far irq(void)
{
   if(++tick >= TICK)
      {
      tick = 0;
      mxyputs(71, 1, "IRQ STAT", NORMI_VID);
      box(70, 0, 79, cpu < 2 ? 11 : 19);
      moutchar(70, 2, (char)199, NORM_VID);
      for(col = 71; col < 79; col++)
         moutchar(col, 2, (char)196, NORM_VID);
      moutchar(79, 2, (char)182, NORM_VID);
      for(col = 3, p8259 = 0; p8259 < (cpu < 2 ? 1 : 2); p8259++)
         {
         status = inp(port[p8259]);
         for(bit = 0; bit <= 7; bit++)
            {
            unsg_to_str(bit + p8259 * 8, buffer, 2);
            mxyputs(71, col, buffer, NORMI_VID);
            if((status >> bit) & 1)
               mxyputs(73, col++, "  FREE", NORM_VID);
            else
               mxyputs(73, col++, "  USED", NORMI_VID);
            }
         }
      }
   _chain_intr(old_int8);
}

main()
{
   set_vid_mem();
   cpu = cputype();
   _disable();
   old_int8 = _dos_getvect(8);
   _dos_setvect(8, irq);
   _enable();
   _dos_keep(0, 185);
}

// Set proper address of video RAM
void set_vid_mem(void)
{
   union REGS r;
   int vmode;
   r.h.ah = 15;
   vmode = int86(0x10, &r, &r) & 255;
   vid_mem = (char far *)MK_FP(vmode == 7 ? 0xB000 : 0xB800, 0000);
}

// Output a string at specified X,Y coordinates
void far mxyputs(int x, int y, char far *str, unsigned char attr)
{
   while(*str)
      moutchar(x++, y, *str++, attr);
}

// Output a character at specified X,Y coordinates
void far moutchar(int x, int y, char ch, unsigned char attr)
{
   char far *v = vid_mem;
   v += y * 160 + x * 2;
   *v++ = ch, *v = attr;
}

void far box(int x1, int y1, int x2, int y2)
{
   register int i;
   char far *v, far *t;
   t = v = vid_mem;
   for(i = y1 + 1; i < y2; i++)
      {
      v += i * 160 + x1 * 2;
      *v++ = 186;
      *v = NORM_VID;
      v = t;
      v += i * 160 + x2 * 2;
      *v++ = 186;
      *v = NORM_VID;
      v = t;
      }
   for(i = x1 + 1; i < x2; i++)
      {
      v += y1 * 160 + i * 2;
      *v++ = 205;
      *v = NORM_VID;
      v = t;
      v += y2 * 160 + i * 2;
      *v++ = 205;
      *v = NORM_VID;
      v = t;
      }
   moutchar(x1, y1, (char)201, NORM_VID);
   moutchar(x1, y2, (char)200, NORM_VID);
   moutchar(x2, y1, (char)187, NORM_VID);
   moutchar(x2, y2, (char)188, NORM_VID);
}

void far unsg_to_str(unsigned i, char far f[], int l)
{
   int k = l - 1, h;
   for(h = 0; h < l; h++)
      {
      f[k--] = i % 10 + 0x30;
      i /= 10;
      }
   f[l] = 0;
}

// Determines the cpu type
char cputype(void)
{
   _asm
      {
      pushf
      mov   ax,0
      push  ax
      popf
      pushf
      pop   AX
      and   AH,0F0h
      cmp   AH,0F0h
      jne   L2
      mov   ax,0
      push  SP
      pop   BX
      cmp   BX,SP
      jne   L1
      jmp   L3
L2:   mov   AX,0F000h
      push  AX
      popf
      pushf
      pop   AX
      and   AH,0F0h
      mov   AX,2
      jz    L1
L3:   inc   AX
L1:   popf
      }
}

_setenvp()
{
}

_setargv()
{
}
