#include <windows.h>
#include <stdio.h>


#define TIME_STUFF(Stuff, Title)\
    StartTime   = GetTickCount(); \
    for(i = 0; i < ITERATIONS; ++i) \
        (Stuff); \
    StopTime    = GetTickCount(); \
    sprintf(Buffer, "%ld " Title " per millisecond\n", \
        ITERATIONS/(StopTime-StartTime-LoopTime)); \
    MessageBox(Window, Buffer, Title, MB_OK);

#define    ITERATIONS    (1000000L)

void    BenchMark(HWND Window, ATOM Atom)
    {
    unsigned long    i, StartTime, StopTime, LoopTime;
    char    Buffer[256];
    char    *NonAtom    = "NonAtom Property";

    SetProp(Window, NonAtom, 0);
    SetProp(Window, (LPCSTR)Atom, 0);

    StartTime    = GetTickCount();
    for(i = 0; i < ITERATIONS; ++i)
        ;
    StopTime    = GetTickCount();
    LoopTime    = StopTime-StartTime;
    sprintf(Buffer,  "%ld loop iterations per millisecond\n",
        ITERATIONS/LoopTime);
    MessageBox(Window, Buffer, "Loop Time", MB_OK);

    TIME_STUFF(GetProp(Window, NonAtom), "GetProp()s");
    TIME_STUFF(GetProp(Window, (LPCSTR)Atom), "Atomic GetProp()s");
    TIME_STUFF(GetWindowWord(Window, 0),"GetWindowWord()'s");
    TIME_STUFF(SendMessage(Window, WM_USER, 0, 0L),"SendMessage()s");
    PostMessage(Window, WM_USER+1, 0, 0L);
    RemoveProp(Window, NonAtom);
    }


LRESULT CALLBACK    WindProc(HWND, UINT, WPARAM, LPARAM);

int PASCAL WinMain(HINSTANCE Instance,
    HINSTANCE Previous, LPSTR CmdLine, int Command)
    {
    static  char AppName[] = "bench";
    HWND        MainWindow;
    WNDCLASS    WindowClass;
    MSG         Message;

    if(!Previous)
        {
        WindowClass.style           = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
        WindowClass.lpfnWndProc     = WindProc;
        WindowClass.cbClsExtra      = 0;
        WindowClass.cbWndExtra      = 0;
        WindowClass.hInstance       = Instance;
        WindowClass.hIcon           = LoadIcon(NULL, IDI_APPLICATION);
        WindowClass.hCursor         = LoadCursor(NULL, IDC_ARROW);
        WindowClass.hbrBackground   = GetStockObject(WHITE_BRUSH);
        WindowClass.lpszMenuName    = NULL;
        WindowClass.lpszClassName   = AppName;
        RegisterClass(&WindowClass);
        }
    MainWindow  = CreateWindow(AppName, AppName,
            WS_OVERLAPPEDWINDOW,
            CW_USEDEFAULT, CW_USEDEFAULT,
            150, 150,
            NULL, NULL, Instance, NULL);
    ShowWindow(MainWindow, Command);
    UpdateWindow(MainWindow);
    while(GetMessage(&Message, NULL, 0, 0))
        {
        TranslateMessage(&Message);
        DispatchMessage(&Message);
        }
    return Message.wParam;
    }

long FAR PASCAL WindProc(HWND Window,
        UINT MsgNum, WPARAM WordParm, LPARAM LongParm)
    {
    switch(MsgNum)
        {
        case    WM_USER :
            return 0;
        case    WM_USER + 1 :
            {
            static unsigned long    StartTime, StopTime, LoopTime;
            char    Buffer[256];

            if(LongParm == 0)
                StartTime   = GetTickCount();
            if(LongParm < ITERATIONS)
                PostMessage(Window, WM_USER+1, 0, LongParm+1);
            else
                {
                StopTime    = GetTickCount();
                sprintf(Buffer, "%ld PostMessage()s per millisecond\n",
                    ITERATIONS/(StopTime-StartTime-LoopTime));
                MessageBox(Window, Buffer, "PostMessage()s", MB_OK);
                }
            return 0;
            }
        case    WM_DESTROY:
            PostQuitMessage(0);
            return 0;
        case    WM_LBUTTONDOWN:
            {
            ATOM    Atom;
            Atom    = GlobalAddAtom("AtomName");
            if(Atom)
                {
                BenchMark(Window, Atom);
                GlobalDeleteAtom(Atom);
                }
            else
                MessageBeep(0);
            }
        default:
            return DefWindowProc(Window, MsgNum, WordParm, LongParm);
        }
    }

