#include <owl.h>

//  Turn on C++ conventions in the header file
#define __COMPILER 3  
#include <tbx.h>

//
//  A window class
//
class TBoxWin : public TWindow
{
public:
	TBoxWin(PTWindowsObject pParent, LPSTR psTitle)
		:TWindow(pParent, psTitle) { }

	//
	//	These are static because they're callback functions.
    //
	static BOOL PASCAL
		DefToolboxProc(PTBXOBJECT pTbx, int nButtonID);
	static BOOL PASCAL
		FlatButtonProc(PTBXOBJECT pTbx, int nButtonID);

protected:
    //  Button ids:
	enum { eButSmile = 100, eButFlat = 101, eButFrown = 102 };

    //  Our toolbox:
    PTBXOBJECT pTbx;

    //  Called after successful window creation.
	virtual void SetupWindow();

	//  Called when a WM_CLOSE message is received.
    virtual void WMClose(RTMessage msg) = [WM_FIRST + WM_CLOSE];
};

//
//  An application class
//
class TBoxApp : public TApplication
{
public:
	TBoxApp(LPSTR psName, HINSTANCE hInst,
	        HINSTANCE hPrevInst, LPSTR psCmdLine, int nCmdShow)
		:TApplication(psName, hInst, hPrevInst,
					  psCmdLine, nCmdShow) { }

  	virtual void InitMainWindow()
		{ MainWindow = new TBoxWin(NULL, "Toolboxes!"); }
};

//
//	OWL function called once a TBoxWin window has been created.
//  Create the toolbox, a child window.
//
void TBoxWin::SetupWindow()
{
	//  Base class processing first.
	TWindow::SetupWindow();

    HINSTANCE hInst = GetModule()->hInstance;

	//  Create the toolbox
    pTbx =
	TbxCreate(hInst, 				// application instance
              HWindow,          	// parent window
              "A Toolbox!",	  		// caption
			  (PFN)DefToolboxProc,  // default event function
              NULL,             	// private data
			  3,                	// number of columns
              TBX_CAPTION);     	// style

    //  Set button size
	TbxButtonSize(pTbx, 40, 40);

    //  No space between buttons
	TbxFreeArea(pTbx, 0);

    //
	//  Add some buttons.
    //  Only the 2nd one has its own event function;
    //  the others will use the toolbox's default.
    //
	TbxCreateButton(pTbx,				// the toolbox		
					eButSmile,          // button id
					(PFN)NULL,      	// event function
					TBX_BUT_SBITMAP,    // named bitmap style
					(ULONG)"bmpSmile"); // bitmap name

	TbxCreateButton(pTbx,				
					eButFlat,           
					(PFN)FlatButtonProc,  	// special! 
					TBX_BUT_SBITMAP,    
					(ULONG)"bmpFlat");  

	TbxCreateButton(pTbx,				
					eButFrown,          
					(PFN)NULL,      	
					TBX_BUT_SBITMAP,    
					(ULONG)"bmpFrown");

	//  Move the toolbox to (100,100) screen coordinates.
    TbxMove(pTbx, 100, 100);

    //  Let it know we're done with construction.
    TbxComplete(pTbx);

	//  Show it onscreen.
    TbxShow(pTbx, TRUE );
}

//
//	The default toolbox event function.
//  Called for the smile and frown buttons.
//
BOOL PASCAL
TBoxWin::DefToolboxProc(PTBXOBJECT pTbx, int nButtonID)
{
	switch(nButtonID) {
		case eButSmile:
			MessageBox(NULL, "Smile!", "Button", MB_OK);
        	break;
		case eButFrown:
			MessageBox(NULL, "Frown!", "Button", MB_OK);
			break;
	}

    return TRUE;
}

//
//	The flat button's event notification function.
//
BOOL PASCAL
TBoxWin::FlatButtonProc(PTBXOBJECT pTbx, int nButtonID)
{
	MessageBox(NULL, "Blah!", "Button", MB_OK);
}

//
//	Before the window closes, destroy the toolbox.
//
void TBoxWin::WMClose(RTMessage msg)
{
    //  Toolbox destruction.
	TbxDestroy(pTbx);
	pTbx = NULL;      			// safety!

	//	Base class processing.
	TWindow::WMClose(msg);
}

//
//	Create and run the application
//
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrevInst,
				   LPSTR lpCmdLine, int nCmdShow)
{
	TBoxApp BoxApp("BoxApp", hInst, hPrevInst,
	               lpCmdLine, nCmdShow);
  	BoxApp.Run();
  	return BoxApp.Status;
}

