
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=Begin Listing 01-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/*****************************************************/
/* dragdemo.c                                        */
/* -- Demonstrate outline dragging.                  */
/*****************************************************/

#include <windows.h>

char    szApp[]    = "DragDemo";
HBRUSH  hbrsDither; /* Dithered gray brush. */

int     rgwBits[] = /* Bits for dithered grah brush. */
    { 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55 };

LONG CALLBACK LMainWndProc(HWND, UINT, WPARAM, LPARAM);
void          PaintRect(HDC, RECT *);

int PASCAL
WinMain(HINSTANCE hinsThis, HINSTANCE hinsPrev,
  LPSTR lszCommand, int wShowWindow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    HWND    hwnd;   /* Main window. */
    HBITMAP hbmp;   /* Bitmap for patterned brush. */
    int     wVal    = 0;    /* Return value. */

    if (!hinsPrev)  /* Regsiter a class first time. */
        {
        WNDCLASS    wcs;

        wcs.style = CS_HREDRAW | CS_VREDRAW;
        wcs.lpfnWndProc = LMainWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra = 0;
        wcs.hInstance = hinsThis;
        wcs.hIcon = LoadIcon(hinsThis, szApp);
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = szApp;
        wcs.lpszClassName = szApp;
        if (!RegisterClass(&wcs))
            return 0;
        }

    /* Create the patterened brush. */
    if (hbmp = CreateBitmap(8, 8, 1, 1, rgwBits))
        {
        hbrsDither = CreatePatternBrush(hbmp);
        DeleteObject(hbmp);
        }
    if (!hbrsDither)
        return 0;

    /* Create and show the main window. */
    if (hwnd = CreateWindow(szApp, szApp,
      WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, wShowWindow,
      CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL,
      hinsThis, NULL))
        {
        MSG msg;    /* For message pump. */

        ShowWindow(hwnd, wShowWindow);
        UpdateWindow(hwnd);
        while (GetMessage(&msg, NULL, NULL, NULL))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }
        wVal = msg.wParam;
        }

    DeleteObject(hbrsDither);
    return wVal;
    }

LRESULT CALLBACK
LMainWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Main window procedure.                         */
/*****************************************************/
    {
    static  RECT    rect;   /* Gray frame to drag. */

    switch (wm)
        {
    default:
        break;

    case WM_SIZE:
        rect.left = LOWORD(lParam) / 4;
        rect.top = HIWORD(lParam) / 4;
        rect.right = 3 * rect.left;
        rect.bottom = 3 * rect.top;
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    case WM_PAINT:
        {
        PAINTSTRUCT wps;

        if (BeginPaint(hwnd, &wps))
            {
            PaintRect(wps.hdc, &rect);
            EndPaint(hwnd, &wps);
            }
        }
        return TRUE;

    case WM_LBUTTONDOWN:
        SetCapture(hwnd);
        /* Fall through. */

    case WM_MOUSEMOVE:
        {
        HDC hdc;

        if (GetCapture() == hwnd && (hdc = GetDC(hwnd)))
            {
            int dx, dy;

            PaintRect(hdc, &rect);
            dx = rect.right - rect.left;
            dy = rect.bottom - rect.top;
            rect.left = LOWORD(lParam) - dx / 2;
            rect.top = HIWORD(lParam) - dy / 2;
            rect.right = rect.left + dx;
            rect.bottom = rect.top + dy;
            PaintRect(hdc, &rect);
            ReleaseDC(hwnd, hdc);
            }
        }
        break;

    case WM_LBUTTONUP:
        ReleaseCapture();
        break;
        }   /* End switch wm. */

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

void
PaintRect(HDC hdc, RECT * prect)
/*****************************************************/
/* -- Paint a rectangle on the screen.               */
/* -- hdc    : Display context to paint onto.        */
/* -- prect  : Outline of rectangle to paint.        */
/*****************************************************/
    {
    int     dxBorder, dyBorder; /* Border thickness. */
    int     dxRect, dyRect;     /* Frame dimensions. */
    HBRUSH  hbrsSav;            /* Restore at exit. */

    UnrealizeObject(hbrsDither);    /* Reset origin */
    if (!(hbrsSav = SelectObject(hdc, hbrsDither)))
        return;                 /* Error. */

    /* Compute dimesions. */
    dxRect = prect->right - prect->left;
    dyRect = prect->bottom - prect->top;
    dxBorder = GetSystemMetrics(SM_CXFRAME);
    dyBorder = GetSystemMetrics(SM_CYFRAME);

    /* Top. */
    PatBlt(hdc, prect->left, prect->top,
      dxRect, dyBorder, PATINVERT);

    /* Bottom. */
    PatBlt(hdc, prect->left, prect->bottom - dyBorder,
      dxRect, dyBorder, PATINVERT);

    /* Left. */
    PatBlt(hdc, prect->left, prect->top + dyBorder,
      dxBorder, dyRect - 2 * dxBorder, PATINVERT);

    /* Right. */
    PatBlt(hdc, prect->right - dxBorder,
      prect->top + dyBorder, dxBorder,
      dyRect - 2 * dxBorder, PATINVERT);

    SelectObject(hdc, hbrsSav); /* Restore DC. */
    }


