/*
 *  DEMO.C
 *  Written by Fred Smith
 *  Developed with MSC 5.1
 *
 *  Tested under Borland C++ 3.1 (by LZ)
 *  Compile with -DTEST compiler switch.
 *  Borland C++:
 *      bcc -DTEST demo.c
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <process.h>
#include <fcntl.h>
#include <io.h>

/*
 * -----------------------  do_cmd()  ----------------------------
 *
 * Calls spawnlp() to execute the command which is passed in 'string'.
 * 
 * Checks the specified stream to see if it has been redirected
 * and if necessary will un-redirect it before the child program is
 * executed, then re-redirect it after the child returns.
 */

FILE * do_cmd(FILE *stream, char * string)
{
    char shell_name [128 + 1];
    char * envptr;
    char get_switch_char (void);
    int old, redirected = 0;
    int new;
    
    /* find out which command processor to use */
    envptr = getenv ("COMSPEC");
    if (envptr != (char *) 0)
        strcpy (shell_name, getenv ("COMSPEC"));
    else
        strcpy (shell_name, "command.com");

    /* if input is redirected, restore stdin to the console */
    if (stream == stdin)
    {
        old = dup (fileno (stdin));
        new = open ("CON", O_RDONLY);
        dup2 (new, fileno (stdin));
        redirected = 1;
    } 

    /* fire off the child process */
    if (strlen(string) != 0)   /* do single command            */
    {
        spawnlp (P_WAIT, shell_name, shell_name,
            "/c", string, (char *)NULL);
    }
    else                    /* give me a command prompt     */
    {
        spawnlp (P_WAIT, shell_name, shell_name, (char *)NULL);
    }


    /* if needed, reinstate the original redirection */
    if (redirected)
    {
        dup2 (old, fileno (stdin));
        close (new);
    }
    return (stream);
}

#ifdef TEST
main ()
{
    if (do_cmd(stdin, "junk.exe JUNK.FIL") != stdin)
        fprintf (stderr, "Redirection failed!\n");

    return (0);
}
#endif /* TEST */
