;------------------------------------------------------
;   LISTING 2
;
;   Module:     SLDD.ASM
;   Summary:    Self-loading device driver
;   Assembler:  TASM 2.0
;   Author:     T.W. Nelson
;   Version:    1.00
;   Date:       29-Jan-1992
;   Notes:
;
;   This module is "start-up" code and must
;   appear first in the link order.
;
;   Source code Copyright (c) 1992 T.W. Nelson. May
;   be used only with appropriate acknowledgement of
;   copyright.
;------------------------------------------------------

INCLUDE sldd.inc
ASSUME  cs:_TEXT,ds:DGROUP,es:NOTHING,ss:NOTHING
STACKSIZ    equ     256     ;driver stack size, words

_DATA   segment
Rhdr        dd  ?               ;-> request header
Caller_ss   dw  ?               ;caller's stack vitals
Caller_sp   dw  ?
__psp       dw  ?               ;program seg prefix
public __psp
_DATA   ends

STACK   segment
            dw  STACKSIZ dup(-1)  ;interrupt stack
stack_top   dw  ?

public stack_top
STACK   ends

_TEXT   segment

;------------------------------------------------------
; Resident driver header block.  IMPORTANT: Header
; block must start at offset 0 in the _TEXT segment,
; otherwise offsets in 'drv_strat' and 'drv_intr' must
; be corrected by subtracting (offset _TEXT:drv_header)
; from them.
;
drv_header  label byte
drv_next    dd  -1              ;-> ground (NULL)
drv_attr    dw  8000h           ;driver attributes
drv_strat   dw  _strategy       ;offset strategy()
drv_intr    dw  _interrupt      ;offset interrupt()
drv_name    db  "TEMPLATE",0    ;device name, 8 bytes

public drv_header, drv_next, drv_attr, drv_strat
public drv_intr, drv_name
extrn   exec_command:proc

;------------------------------------------------------
public _strategy

_strategy       proc    far
        push    ax
        push    ds
        mov     ax,seg DGROUP
        mov     ds,ax
        mov     wptr [Rhdr+2],es
        mov     wptr [Rhdr],bx
        pop     ds
        pop     ax
        ret
_strategy   endp

;------------------------------------------------------
public _interrupt

_interrupt      proc    far
        push    ax          ;Save caller's CPU state
        push    bx          ;and stack context ....
        push    cx
        push    dx
        push    di
        push    si
        push    bp
        push    ds
        push    es
        pushf
        mov     ax,seg DGROUP       ;local data seg
        mov     ds,ax
        mov     Caller_ss,ss
        mov     Caller_sp,sp
        cli
        mov     ss,ax
        mov     sp,offset DGROUP:stack_top
        sti
        mov     bp,sp               ;set C stack frame
        les     di,[Rhdr]           ;request header
        call    exec_command
        cli                         ;restore CPU state
        mov     ss,Caller_ss        ;and caller's stack
        mov     sp,Caller_sp
        sti
        popf
        pop     es
        pop     ds
        pop     bp
        pop     si
        pop     di
        pop     dx
        pop     cx
        pop     bx
        pop     ax
        ret
_interrupt  endp

_TEXT   ends

;------------------------------------------------------
; Driver load and initialization code.  The entire
; _INIT segment is jettisoned after initialization if
; COMEXEC is not defined.
;
_INIT   segment
ASSUME  cs:_INIT,ds:_INIT,es:NOTHING,ss:NOTHING

dosMajor    dw  0       ;DOS version #s
dosMinor    dw  0       ;
DosDataSeg  dw  0       ;DOS data segment
ofsLoL      dw  0       ;offset to DOS LoL
pBlkDevCnt  dd  0       ;-> Block device count in LoL
LastDrive   dw  0       ;LASTDRIVE value in LoL
ofsNULdev   dw  0       ;offset of NUL device header
pCDS        dd  0       ;-> Current Directory Structure
sizeofCDS   dw  0       ;sizeof a CDS[] element
pspSeg      dw  0       ;PSP segment
NULname     db  "NUL     ",0    ;NUL device name
cmdPkt      RH_HDR  <>  ;driver command packet
            dw  STACKSIZ dup(-1)  ;setup stack
setup_stack dw  ?

;Version-specific offsets for DOS data (CVT, etc.) ...
DOS30   db      10h,1Bh,17h,28h,51h
DOS31   db      20h,21h,16h,22h,51h
DOS4    db      20h,21h,16h,22h,58h     ;DOS 4+

;byte offsets into initializer tables .....
E_pBlkDevCnt  equ   0
E_LastDrive   equ   1
E_pCDS        equ   2
E_ofsNULdev   equ   3
E_sizeofCDS   equ   4

dos_ver     db  "You need at least DOS ver 3",10,13,0
bad_dos     db  "Unknown DOS version",10,13,0
no_nul      db  "Can't find NUL device header",10,13,0
is_loaded   db  " device is already loaded",10,13,0
is_file     db  " device is a disk file",10,13,0
bad_init    db  "Driver initialization failed",10,13,0
unloaded    db  "Driver template unloaded",10,13,0
CDSfull     db  "Out of CDS entries",10,13,0
no_block    db  "Can't install block device",10,13,0

; Current Directory Structure .......
CDS struc
path    db  43h dup(?)  ;current path
flags   dw  ?           ;= 0 if entry available
pDPB    dd  ?           ;-> disk param block
sclust  dw  ?           ;starting cluster
ffff    dd  ?           ;always -1 (?)
slash   dw  ?           ;ofs '\' in current path
qmark1  db  ?           ;unknown byte  (DOS 4+)
ifs     dd  ?           ;-> ?          (DOS 4+)
qmark2  dw  ?           ;unknown byte  (DOS 4+)
CDS ends

IFDEF   COMEXEC     ;load command shell to simulate
                    ;residency
comSpec         db      "c:\command.com",0
exec_params     equ     $   ;parameter block for
environ_seg     dw      ?   ;DOS EXEC function ....
command_ofs     dw      ?
command_seg     dw      ?
fcb1_ofs        dw      ?
fcb1_seg        dw      ?
fcb2_ofs        dw      ?
fcb2_seg        dw      ?
save_sp         dw      ?

ENDIF   ;COMEXEC

;------------------------------------------------------
; entry:    al == exit code
; exit:     n/a
;
exit2dos:
    mov     ah,4ch          ;DOS exit
    int     21h

;------------------------------------------------------
; entry:    ds:bx - asciiz message string
; exit:     n/a
;
err_exit:
    call    putstr
    mov     al,3
    jmp     exit2dos

;------------------------------------------------------
; entry:    ds:bx - asciiz message string
; exit:     n/a
;
putstr  proc    near
    push    ax
    push    bx
putstr1:
    mov     al,[bx]
    or      al,al           ;finished?
    jz      _px             ;yes
    mov     ah,0eh          ;BIOS write tty
    push    bx              ;save it
    xor     bx,bx           ;page 0
    int     10h
    pop     bx              ;restore bx
    inc     bx              ;increment pointer
    jmp     putstr1         ;next byte
_px:
    pop     bx
    pop     ax
    ret
putstr  endp

;------------------------------------------------------
; entry:    n/a
; exit:     ax = index of 1st avail CDS entry, aborts
;           program if no free entries exist
; note:     pCDS, sizeofCDS must be known before call
;
new_drive   proc    near
        push    es
        push    bx
        les     bx,pCDS         ;point to table head
        mov     ax,0            ;index of 1st CDS entry
n1:     cmp     es:[bx].flags,0 ;entry available?
        je      nex             ;yes, quit
        add     bx,sizeofCDS    ;else, -> next entry
        inc     ax              ;next index
n5:     cmp     ax,LastDrive    ;end of loop?
        jl      n1              ;no, continue
        mov     bx,offset CDSfull ;yes, no entries free
        jmp     err_exit          ;abort the program
nex:    pop     bx
        pop     es
        ret
new_drive   endp

;------------------------------------------------------
start_up:

    mov     ax,cs               ;Recover segment
    mov     ds,ax               ;registers and PSP seg
    mov     pspSeg,es
    cli
    mov     ss,ax
    mov     sp,offset setup_stack
    sti
    mov     ax,seg DGROUP
    mov     ds,ax
    ASSUME  ds:DGROUP           ;get -> PSP for
    mov     __psp,es            ;resident portion
    push    cs
    pop     ds
    ASSUME  ds:_INIT

; ----- Check and save DOS version --------------------
    mov     ax,3000h            ;DOS get version
    int     21h
    cmp     al,3                ;at least ver 3?
    jge     saveDOSver          ;yes, bypass
    mov     bx,offset dos_ver   ;no ....
    jmp     err_exit            ;quit
saveDOSver:
    xor     bx,bx
    mov     bl,al
    mov     dosMajor,bx
    mov     bl,ah
    mov     dosMinor,bx

; ----- Grab data from CVT (List of Lists) ------------
    mov     ah,52h
    int     21h                 ;es:bx -> LoL
    cmp     dosMajor,3
    ja      dos_4x
    cmp     dosMinor,0
    ja      dos_31
    mov     di,offset DOS30
    jmp     short go_CVT
dos_31:
    mov     di,offset DOS31
    jmp     short go_CVT
dos_4x:
    mov     di,offset DOS4
    cmp     dosMajor,5
    jbe     go_CVT
    mov     bx,offset bad_dos
    jmp     err_exit
go_CVT:
    mov     DosDataSeg,es
    mov     ofsLoL,bx
    xor     ax,ax
    mov     al,[di].E_pBlkDevCnt
    add     ax,bx
    mov     wptr pBlkDevCnt[2],es
    mov     wptr pBlkDevCnt[0],ax
    xor     ax,ax
    mov     al,[di].E_LastDrive
    mov     si,ax
    mov     al,bptr es:[bx+si]
    mov     LastDrive,ax
    xor     ax,ax
    mov     al,[di].E_pCDS
    mov     si,ax
    mov     ax,wptr es:[bx+si]          ;-> ofsCDS
    mov     wptr pCDS[0],ax
    mov     ax,wptr es:[bx+si+2]        ;-> segCDS
    mov     wptr pCDS[2],ax
    xor     ax,ax
    mov     al,[di].E_ofsNULdev
    add     ax,bx
    mov     ofsNULdev,ax
    xor     ax,ax
    mov     al,[di].E_sizeofCDS
    mov     sizeofCDS,ax

; ----- Verify that NUL device is present -------------
    mov     si,offset NULname   ;ds:si
    mov     di,ofsNULdev        ;es:di -> device hdr
    add     di,d_name           ;-> device name
    mov     cx,8                ;compare count
    repz    cmpsb               ;get 1st non-match
    jz      chk_load            ;NUL device present
    mov     bx,offset no_nul    ;can't find NUL
    jmp     err_exit

; ----- Check driver load status ----------------------
chk_load:
    mov     ax,seg _TEXT
    mov     ds,ax
    mov     dx,offset _TEXT:drv_name ;ds:dx = our name
    mov     ax,3d00h            ;DOS Open File, rdonly
    int     21h
    jc      init_driver         ;OK, driver not loaded
    mov     bx,dx               ;not OK,
    call    putstr              ;print device name
    push    cs
    pop     ds                  ;fix ds
    mov     bx,ax               ;bx = open handle
    mov     ax,4400h            ;DOS IOCTL,
    int     21h                 ;get device info
    test    dx,0080h            ;is it a file?
    jnz     already_loaded      ;no, driver installed
    mov     bx,offset is_file   ;yes, file w/ same name
    jmp     err_exit            ;quit
already_loaded:
    mov     bx,offset is_loaded ;and loaded msg
    jmp     err_exit            ;quit

; ----- Build cmd packet and initialize driver --------
init_driver:
    push    cs
    pop     ds                  ;fix ds
    mov     cmdPkt.rh_cmd,0     ;driver INIT command
    mov     cmdPkt.rh_len,size RH_HDR
    mov     cmdPkt.rh_unit,0
    mov     es,pspSeg
    mov     di,81h              ;es:di -> PSP cmd line
d1: cmp     bptr es:[di],' '
    jne     d2                  ;skip leading spaces
    inc     di
    jmp     d1
d2: mov     cmdPkt.rh_bufofs,di
    mov     cmdPkt.rh_bufseg,es
    call    new_drive           ;don't know yet if
    mov     cmdPkt.rh_drive,al  ;it's a block device
    push    cs
    pop     es
    mov     bx,offset cmdPkt        ;es:bx -> cmd pkt
    call    far ptr _strategy
    call    far ptr _interrupt
    test    cmdPkt.rh_status,8000h  ;error?
    jz      init_block              ;no, continue
    mov     bx,offset bad_init      ;yes, bail out
    jmp     err_exit
init_block:
    include initblk.asm         ;init block device

; ----- Patch driver into existing chain --------------
    push    ds
    mov     es,DosDataSeg
    mov     bx,ofsNULdev
    mov     ax,seg _TEXT
    mov     ds,ax
    ASSUME  ds:_TEXT
    mov     dx,wptr es:[bx+2]   ;dx:ax == next driver
    mov     ax,wptr es:[bx+0]   ;after NUL device.
    mov     wptr drv_next[0],ax ;make ours point to
    mov     wptr drv_next[2],dx ;what NUL pointed to.
    mov     wptr es:[bx+0],offset _TEXT:drv_header
    mov     wptr es:[bx+2],ds   ;point NUL to us
    pop     ds
    ASSUME  ds:_INIT

IFNDEF  COMEXEC                 ;release environment
    mov     es,pspSeg           ;get environ seg
    mov     es,es:[2ch]         ;in PSP block
    mov     ah,49h              ;DOS free memory
    int     21h
ENDIF

; ----- Compute resident program size -----------------
    mov     dx,cmdPkt.rh_brkofs
    mov     cx,4
    shr     dx,cl
    inc     dx
    add     dx,cmdPkt.rh_brkseg
    sub     dx,pspSeg           ;#resident paragraphs

IFDEF   COMEXEC
; ----- Load Command.com to simulate residency --------
    mov     es,pspSeg           ;beg segment address
    mov     bx,dx               ;total resident size
    mov     ax,4a00h            ;DOS SETBLOCK
    int     21h                 ;return unused memory
    mov     di,0080h            ;es:di -> command line
    mov     bptr es:[di],0      ;cmd length = zero
    mov     dx,offset comSpec
    mov     cx,es:[2ch]         ;grab environment seg
    mov     environ_seg,cx
    mov     command_ofs,di
    mov     command_seg,es
    mov     fcb1_ofs,5ch
    mov     fcb1_seg,es
    mov     fcb2_ofs,6ch
    mov     fcb2_seg,es
    mov     ax,4b00h            ;DOS EXEC program
    mov     save_sp,sp
    push    cs
    pop     es
    mov     bx,offset exec_params
    int     21h                 ;load COMMAND.COM
    mov     ax,cs               ;restore regs
    mov     es,ax
    mov     ds,ax
    cli
    mov     ss,ax
    mov     sp,save_sp
    sti

; ----- Remove driver from device chain ---------------
    push    ds
    mov     es,DosDataSeg       ;point to NUL device
    mov     bx,ofsNULdev
    mov     ax,seg _TEXT
    mov     ds,ax
    ASSUME  ds:_TEXT
    mov     ax,wptr drv_next[0] ;dx:ax = what
    mov     dx,wptr drv_next[2] ;we point to
    mov     wptr es:[bx+0],ax   ;make NUL device point
    mov     wptr es:[bx+2],dx   ;to it instead
    pop     ds
    ASSUME  ds:_INIT
    mov     bx,offset unloaded
    call    putstr              ;print unloaded msg
    xor     ax,ax               ;DOS exit code
    jmp     exit2dos            ;quit, back to DOS
driver_end  label   byte        ;end of COMEXEC image
public driver_end

ELSE                            ;true residency
    mov     ax,3100h            ;DOS TSR
    int     21h
ENDIF   ;COMEXEC

_INIT       ends
            end     start_up
;--- End of File --------------------------------------
