/* BTN.EXE: 3-D Buttons Demonstration Program

   Copyright (c) 1992 Springtime Software
   All Rights Reserved

   Uses owner-draw 3-D buttons with special text in their captions
   describing 3-D text and graphics effects.

   User Pushbutton caption fields:

	@f ............................ 3-D Frame (must come first)
	@b<name> ...................... Draw Named Bitmap With Key Bitmap
									(see below)
	@t<text string> ............... Draw Text String
	@<modifiers>t<text string> .... Draw Text String

   The modifiers are:
		2 ............. 2-D (Floating) Text or Graphic
		e ............. Engraved
		r ............. Raised (Embossed)
		u ............. Up (At the Top)
		d ............. Down (At the Bottom)

   Each bitmap has a corresponding "key" bitmap called <name>_3d,
   where <name> is the name of the bitmap.  In this key bitmap colors
   indicate the 3-D effects, as follows:

   Key Colors:

		Light Gray ........ Background that moves with button
		Red ............... Flat areas that move
		Green ............. Engraved areas that move
		Blue .............. Raised (Embossed) areas that move

		Gray .............. Background that "floats" above button
		Dark Red .......... Flat areas that float
		Dark Green ........ Engraved areas that float
		Dark Blue ......... Raised areas that float

        White ............. Transparent (clear) areas
*/

#include <windows.h>
#include "btn_fncs.h"

/* Copied from BTN.RC: */
#define IDPIC2	15
#define IDNOPE	14
#define IDENABLE	13
#define IDDISABLE	12
#define IDPIC1	11

/* Standard declarations */
#define EQ ==
#define NE !=
#define NUL '\0'
#define STRCMP lstrcmp

/* Macros */

/* Dialog procedure header */
#define		DLG(name)\
	FARPROC				name##_dlgi;\
	BOOL CALLBACK		name##_dlg

/* Call a dialog box */
#define		CALL_DLG(name, parent)\
	{\
	name##_dlgi =\
		MakeProcInstance((FARPROC)name##_dlg, hInstance);\
	if (DialogBox(hInstance, #name, parent, (DLGPROC)name##_dlgi) EQ -1)\
		fatal("could not start dialog box %s", (long)#name);\
	FreeProcInstance(name##_dlgi);\
	}

char	app_name[] = "BTN";

/* Functions */

/* Fatal error message box */
CATCHBUF		catchbuf;
void	fatal(LPSTR msg, long info)
	{
	char		text[512];
	LPSTR		t;

	t = text;
	t += wsprintf(t, "Sorry, ");
	t += wsprintf(t, msg, info);
	t += wsprintf(t, ".  This is an internal error in %s.",
		(LPSTR)app_name);
	MessageBox(NULL, text, "Internal Error", MB_ICONSTOP | MB_OK);
	Throw(catchbuf, 1); /* Quit this application */
	} /* fatal */

/* Handle dialog messages for the main modal dialog box "demo"
 * The three special 3-D calls are marked with asterisk comments */
DLG(demo)(HWND dlg, UINT message, UINT wParam, LONG lParam)
	{
	switch (message)
		{
		case WM_INITDIALOG:
			scan_3d_buttons(dlg, 0); /* * */
			return TRUE;
		case WM_DRAWITEM:
			paint_3d_button(lParam); /* * */
			return TRUE;
		case WM_COMMAND:
			switch (wParam)
				{
				case IDDISABLE:
					EnableWindow(GetDlgItem(dlg, IDPIC1), FALSE);
					EnableWindow(GetDlgItem(dlg, IDPIC2), FALSE);
					return TRUE;
				case IDENABLE:
					EnableWindow(GetDlgItem(dlg, IDPIC1), TRUE);
					EnableWindow(GetDlgItem(dlg, IDPIC2), TRUE);
					return TRUE;
				case IDOK:
					/* Fall through */
				case IDCANCEL:
					EndDialog(dlg, 0);
					return TRUE;
				} /* Switch on command */
			break; /* WM_COMMAND default */
		case WM_DESTROY:
			scan_3d_buttons(dlg, 1); /* * */
			break;
		} /* Switch on message */
	return FALSE; /* Default: Message needs to be handled */
	} /* test_proc */

/* Main function */
/* The two special 3-D calls are marked with an asterisk comment */
#ifdef __BORLANDC__
    #pragma argsused
#endif
int pascal	WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpszCmdLine, int nCmdShow)
	{
	/* Set up for handling any program-detected error (in "fatal") */
	if (!Catch(catchbuf))
		{
		/* First time through (not Catching a Throw) */
		init_3d(app_name, hInstance, fatal); /* * */

		/* Start the main dialog */
		CALL_DLG(demo, NULL);
		} /* Portion of program in which "fatal" can be called */

	/* Do any program cleanup here */
	end_3d(); /* * */

	return 0;
	} /* WinMain */
