/* password.c - Maintains a password per program manager group.

   To build: "cc -wd -DSTRICT password.c toolhelp.lib"
   If your compiler did not install a "toolhelp.lib", use
   "implib" to create it yourself from "toolhelp.dll".
 */

#include <windows.h>
#include <windowsx.h>
#include <toolhelp.h>
#include "password.h"

char        szClass[] = "PMGroup";
char        szOpen[]  = "Password";
WNDPROC     lpfnOrig;
UINT        wmOpen;
HINSTANCE   hinsLib;

/* Prototypes. */
BOOL    FTestPassword(LPSTR, LPSTR);

/* Callbacks. */
BOOL CALLBACK __export FChildEnumProc(HWND, LPARAM);
BOOL CALLBACK __export
    FDlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK __export FTopEnumProc(HWND, LPARAM);
LRESULT CALLBACK __export
    LwWndProc(HWND, UINT, WPARAM, LPARAM);

int CALLBACK LibMain(HINSTANCE hins, WORD ds,
  WORD cbHeap, LPSTR lsz)
/*****************************************************/
/* -- Initialization entry point.                    */
/* -- Install the progman window hook.               */
/*****************************************************/
    {
    TASKENTRY   tsk;
    BOOL        f;
    HWND        hwnd;

    /* Find progman's instance handle. */
    tsk.dwSize = sizeof tsk;
    for (f = TaskFirst(&tsk); f; f = TaskNext(&tsk))
        if (lstrcmpi(tsk.szModule, "PROGMAN") == 0)
            break;
    if (!f)
        return FALSE;

    if ((wmOpen = RegisterWindowMessage(szOpen)) == 0)
        return FALSE;

    hinsLib = hins;
    hwnd = NULL;
    EnumTaskWindows(tsk.hTask, FTopEnumProc,
      (LPARAM)(HWND far *)&hwnd);
    if (hwnd == NULL)
        {
        WNDCLASS    wcs;

        GetClassInfo(tsk.hInst, szClass, &wcs);
        UnregisterClass(szClass, tsk.hInst);
        wcs.hInstance = tsk.hInst;
        wcs.lpszClassName = szClass;
        lpfnOrig = wcs.lpfnWndProc;
        wcs.lpfnWndProc = LwWndProc;
        return RegisterClass(&wcs);
        }
    SetClassLong(hwnd, GCL_WNDPROC, (LONG)LwWndProc);
    return TRUE;
    }

BOOL CALLBACK __export
FTopEnumProc(HWND hwnd, LPARAM lParam)
/*****************************************************/
/* -- EnumTaskWindows() callback.                    */
/*****************************************************/
    {
    if (GetWindow(hwnd, GW_CHILD) == NULL)
        return TRUE;    /* No kids. */

    return EnumChildWindows(hwnd, FChildEnumProc,
      lParam);
    }

BOOL CALLBACK __export
FChildEnumProc(HWND hwnd, LPARAM lParam)
/*****************************************************/
/* -- EnumChildWindows() callback.                   */
/* -- Look for a PMGroup window.                     */
/*****************************************************/
    {
    char    szBuf[10];

    GetClassName(hwnd, szBuf, sizeof szBuf);
    if (lstrcmpi(szBuf, szClass) == 0)
        {
        *(HWND far *)lParam = hwnd;
        lpfnOrig = SubclassWindow(hwnd, LwWndProc);
        SendMessage(hwnd, WM_SYSCOMMAND, SC_ICON, 0);
        }
    return TRUE;
    }

LRESULT CALLBACK __export LwWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam)
/*****************************************************/
/* -- PMGroup subclass proc.                         */
/*****************************************************/
    {
    static HWND hwndChecked;

    if (wm == wmOpen)
        {
        if (DialogBoxParam(hinsLib,
          MAKEINTRESOURCE(dlgGetPass),
          GetActiveWindow(), FDlgProc,
          (LPARAM)(UINT)hwnd) == IDOK)
            {
            hwndChecked = hwnd;
            SendMessage(hwnd, WM_SYSCOMMAND,
              SC_RESTORE, lParam);
            hwndChecked = NULL;
            }
        return 0;
        }

    switch (wm)
        {
    default:
        break;

    case WM_QUERYOPEN:
        if (hwndChecked == hwnd)
            return CallWindowProc(lpfnOrig, hwnd, wm,
              wParam, lParam);
        PostMessage(hwnd, wmOpen, 0, GetMessagePos());
        return 0;

    case WM_SHOWWINDOW:
        if (wParam != 0)
            SendMessage(hwnd, WM_SYSCOMMAND, SC_ICON,
              0);   /* Don't allow it to show. */
        return 0;
        }   /* End switch wm. */

    return CallWindowProc(lpfnOrig, hwnd, wm, wParam,
      lParam);
    }

BOOL CALLBACK __export FDlgProc(HWND hwnd, UINT wm,
  WPARAM wParam, LPARAM lParam)
/*****************************************************/
/* -- Password dialog procecudure.                   */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        break;

    case WM_INITDIALOG:
        {
        char    sz[100];

        /* Get name of group file. */
        GetWindowText((HWND)lParam, sz, sizeof sz);
        SetDlgItemText(hwnd, didGroup, (LPSTR)sz);
        }
        return FALSE;

    case WM_COMMAND:
        switch (wParam)
            {
        default:
            break;

        case IDOK:
            {
            char    szPassword[100], szGroup[100];

            GetDlgItemText(hwnd, didPassword,
              szPassword, sizeof szPassword);
            GetDlgItemText(hwnd, didGroup,
              szGroup, sizeof szGroup);
            if (!FTestPassword(szPassword,
              szGroup))
                {
                MessageBox(hwnd,
                  "You entered an invalid password",
                  "Invalid Password", MB_OK);
                return TRUE;
                }
            }   /* Fall through. */

        case IDCANCEL:
            EndDialog(hwnd, wParam);
            return TRUE;
            }   /* End switch wParam. */
        break;  /* End case WM_COMMAND. */
        }       /* End switch wm. */

    return FALSE;
    }

BOOL FTestPassword(LPSTR lpszPassword, LPSTR lpszGroup)
/*****************************************************/
/* -- Validate a password.                           */
/*****************************************************/
    {
    return lstrcmpi(lpszPassword, lpszGroup) == 0;
    }
