/*****************************************************/
/* symbase.c                                         */
/* -- Framework to support Windows function hooks.   */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include <toolhelp.h>
#include "symbase.h"
#include "symhook.h"
#include "symdll.h"

/* Private prototypes. */
BOOL CALLBACK _export FNotifyProc(WORD, NFYLOADSEG FAR *);
int  CALLBACK LibMain(HINSTANCE, WORD, WORD, LPSTR);
int  CALLBACK _export WEP(int);

/* Set or remove all hooks. */
#define SetAllHooks(fHook)                            \
    {                                                 \
    HKS *   phks;                                     \
    for (phks = rghks; phks->lpfnOrig; phks++)        \
        HookHks(phks, (fHook));                       \
    }

HTASK   htskReg;        /* NotifyRegister() task. */
LPCDS   lrgcds;         /* Client array. */
int     ccds;           /* Size of array. */
int     ccdsClient;     /* Client task count. */
#define dcds   10       /* Allocation increment. */

int CALLBACK
LibMain(HINSTANCE hinsThis, WORD ds, WORD cbHeap,
  LPSTR lsz)
/*****************************************************/
/* -- Entry point during initialization.             */
/*****************************************************/
    {
    HKS *   phks;

    /* Save entry code of functions to hook. */
    for (phks = rghks; phks->lpfnOrig; phks++)
        MemoryRead(SELECTOROF(phks->lpfnOrig),
            OFFSETOF(phks->lpfnOrig), phks->rgbCode,
          sizeof phks->rgbCode);

    return 1;
    }

int CALLBACK _export WEP(int wExitCode)
/*****************************************************/
/* -- Exit proc.                                     */
/* -- No need to free global memory, Windows will.   */
/*****************************************************/
    {
    SetAllHooks(FALSE);
    if (htskReg)
        NotifyUnRegister(htskReg);
    return 1;
    }

BOOL WINAPI _export FStartFontMix(HFONT hfnt)
/*****************************************************/
/* -- Called by a client when it wants mixed font.   */
/*****************************************************/
    {
    int     icds, icdsT;
    HTASK   htsk    = GetCurrentTask();

    /* Look for an empty slot in the task array and */
    /* make sure task is not already known. */
    for (icds = ccds, icdsT = 0; icdsT < ccds; icdsT++)
        {
        if (!lrgcds[icdsT].htsk && icds == ccds)
            icds = icdsT;
        if (lrgcds[icdsT].htsk == htsk)
            return FALSE;
        }

    /* If we didn't find one, grow the array. */
    if (icds == ccds)
        {
        LPCDS   lrgcdsNew;
        int     cb   = (ccds + dcds) * sizeof(CDS);

        if (!(lrgcdsNew = lrgcds ? GlobalReAllocPtr(
            lrgcds, cb, GHND | GMEM_SHARE) :
          GlobalAllocPtr(GHND | GMEM_SHARE, cb)))
            return FALSE;   /* Out of memory. */

        lrgcds = lrgcdsNew;
        ccds += dcds;
        }

    /* If this is the first app, install the hooks */
    /* and toolhelp callback. */
    if (!ccdsClient)
        {
        SetAllHooks(TRUE);
        if (!NotifyRegister(htskReg = htsk,
          (LPFNNOTIFYCALLBACK)FNotifyProc, NF_NORMAL))
            return FALSE;   /* Give up. */
        }

    ccdsClient++;
    lrgcds[icds].htsk = htsk;
    lrgcds[icds].hfnt = hfnt;
    return TRUE;
    }

void WINAPI _export StopFontMix(void)
/*****************************************************/
/* -- Called by a client when it no longer wants     */
/*    mixed font support.                            */
/*****************************************************/
    {
    LPCDS   lpcds, lpcdsLim;
    HTASK   htsk;

    if (!(lpcds = LpcdsGetCur()))
        return;         /* Unknown task. */

    htsk = lpcds->htsk; /* Remove from list. */
    lpcds->htsk = NULL;

    /* Remove toolhelp callback if it was installed */
    /* with this task. */
    if (htskReg == htsk)
        {
        NotifyUnRegister(htskReg);
        htskReg = NULL;
        }

    /* If there is another client, reinstall. */
    if (--ccdsClient)
        {
        for (lpcds = lrgcds, lpcdsLim = lpcds + ccds;
          lpcds < lpcdsLim; lpcds++)
            if (lpcds->htsk)
                {
                NotifyRegister(htskReg = lpcds->htsk,
                  (LPFNNOTIFYCALLBACK)FNotifyProc,
                  NF_NORMAL);
                break;
                }
        }
    else
        {
        SetAllHooks(FALSE);
        }
    }

LPCDS LpcdsGetCur(void)
/*****************************************************/
/* -- Return a pointer to the current task's entry   */
/*    in the client task list.  Return NULL if task  */
/*    is unknown.                                    */
/*****************************************************/
    {
    HTASK   htsk    = GetCurrentTask();
    LPCDS   lpcds, lpcdsLim;

    if (!lrgcds)
        return NULL;

    for (lpcds = lrgcds, lpcdsLim = lpcds + ccds;
      lpcds < lpcdsLim; lpcds++)
        if (lpcds->htsk == htsk)
            return lpcds;

    return NULL;
    }

BOOL CALLBACK _export FNotifyProc(WORD nfy,
                                     NFYLOADSEG FAR * lpnls)
/*****************************************************/
/* -- Toolhelp callback.                             */
/* -- If the segment containing a hooked routine is  */
/*    being reloaded, reinstall the jump.            */
/*****************************************************/
    {
    switch (nfy)
        {
    default:
        break;

    case NFY_LOADSEG:
        {
        HKS *   phks;

        for (phks = rghks; phks->lpfnOrig; phks++)
            if (lpnls->wSelector ==
              SELECTOROF(phks->lpfnOrig))
                HookHks(phks, TRUE);
        }
        break;

    case NFY_EXITTASK:
        if (LpcdsGetCur())
            StopFontMix();
        break;
        }

    return FALSE;
    }
