// ---------------------------------------------------------
//  search.cpp
// ---------------------------------------------------------

#include <windows.h>
#include <string.h>
#include <fstream.h>
#include "zyhapi.h"

//
//  Open the given file, get the filename, and send info
//  to the output text file.  Called by WinMain(), below.
//
void ProcessDoc(long lZYIndex, int nFileId, ofstream &ofs)
{
    //  Open the given file.
    ZYopenDoc(lZYIndex, nFileId);

    // Get the number of hits, filename, and path.
    char sInfo[HAPI_INFO_LENGTH+1];
    ZYgetResult(lZYIndex,              // index handle
                nFileId,               // file id
                FILENAME | PATHNAME,   // info requested
                sInfo,                 // put info here
                sizeof(sInfo));        // max info chars

    ofs << "Doc #" << nFileId << " " << sInfo << endl;

    // The current line number (first line in file == 0).
    long lLine = (-1);

    //
    //  Seek to each line containing a hit, get the line #,
    //  and write it to the output stream.
    //
    while(1) {
        long lResultLine;
        int nResult =
            ZYseekDoc(lZYIndex,         // index handle
                      nFileId,          // file id
                      ZY_SEEK_FORWARD,  // find next hit
                      lLine,            // current line
                      &lResultLine);    // resulting line
            lLine = lResultLine;

        if(nResult != 0)
            break;

        ofs << "    line #" << lLine << endl;
    }

    // Close the file.
    ZYcloseDoc(lZYIndex, nFileId);
}

int PASCAL WinMain(HINSTANCE, HINSTANCE, LPSTR psCmd, int)
{
    //  The paths for the control files and the index.
    char *psControlPath = "d:\\zydev\\control";
    char *psIndexPath   = "d:\\mp\\zy\\index";

    //  Start the Toolkit.
    ToolkitOptions tkOptions;
    memset(&tkOptions, 0, sizeof(tkOptions));
    tkOptions.nResourceLevel = ZY_MEDIUM;
    ZYstart(psControlPath, 
            tkOptions);

    //  Open the index in SEARCH mode and get its handle. 
    long lZYIndex;
    ZYopenIndex(psIndexPath,        // path to index 
                ZY_SEARCH,          // search mode
                &lZYIndex);         // put handle here

    //
    //  Execute a search for the expression passed to
    //  WinMain() on the command line.
    //
    int nFiles;
    long lHits;
    ZYsearchIndex(lZYIndex,     // index handle
                  psCmd,        // search request
                  &nFiles,      // put # files here
                  &lHits);      // put # hits here

    //
    //  Create the file for text output and process each
    //  file that contained a hit.
    //
    ofstream ofs("words.txt");
    ofs << "Results for \""<< psCmd << "\" :" << endl;

    for(int nFileId=0; nFileId<nFiles; nFileId++)
        ProcessDoc(lZYIndex, nFileId, ofs);

    // Close the index and shut down the toolkit.
    ZYcloseIndex(lZYIndex);
    ZYfinish();
    return 0;
}


