/*****************************************************/
/* dlgicon.c                                         */
/* -- Demonstrate per-dialog icons.                  */
/* -- To build: "cc -DSTRICT dlgicon.c dlgicon.rc"   */
/*****************************************************/
#include <windows.h>
#include <windowsx.h>
#include <toolhelp.h>
#include <string.h>

typedef struct
    {
    DWORD           ulwStyle;
    BYTE            citm;
    unsigned short  x, y, dx, dy;
    char            szMenuName[1];
    } DLT;          /* DiaLog Template. */

BOOL CALLBACK __export FDlgProc(HWND hwnd, UINT wm,
    WPARAM wParam, LPARAM lParam);
HGLOBAL HgblSetDlgIcon(LPCSTR lpszDlt, char * pszClass,
  HICON hicn, HINSTANCE hins);

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lpszCmd, int wShow)
    {
    HGLOBAL     hgbl;
    HICON       hicn;
    char        szClass[MAX_CLASSNAME];

    szClass[0] = 0;
    if ((hicn = LoadIcon(NULL, IDI_QUESTION)) != NULL)
        {
        if ((hgbl = HgblSetDlgIcon("MyDialog", szClass,
          hicn, hins)) != NULL)
            {
            DialogBoxIndirect(hins, hgbl, NULL,
              FDlgProc);
            FreeResource(hgbl);
            }
        if (szClass[0] != 0)
            UnregisterClass(szClass, hins);
        }

    return 0;
    }

HGLOBAL
HgblSetDlgIcon(LPCSTR lpszDlt, char * pszClass,
  HICON hicn, HINSTANCE hins)
/*****************************************************/
/* -- Create a new private dialog class with given   */
/*    icon.  Load the dialog template, and change    */
/*    its class to the new name.  Return the         */
/*    modified dialog template and new class name.   */
/*****************************************************/
    {
    WNDCLASS    wcs;    /* Cloned class. */
    HRSRC       hrsrc;  /* Dialog template resource. */
    HGLOBAL     hgblNew; /* New dialog template. */
    HGLOBAL     hgbl        = NULL; /* Old template. */
    int         fOk         = FALSE;
    DLT far *   lpdlt;  /* New template pointer. */
    char far *  lpszClass; /* Unique class name. */
    UINT        bClass; /* Offset to class name. */
    int         cbClassOld, cbClassNew, cbOld;
    static long cact;   /* Number of times called. */

    /* Get the dialog class and clone it. */
    GetClassInfo(NULL, MAKEINTRESOURCE(32770), &wcs);
    wsprintf(pszClass, "MyDialog%lx", ++cact);
	wcs.lpszClassName = pszClass;
    wcs.lpszMenuName = NULL;
    wcs.hIcon = hicn;
    wcs.hInstance = hins;

    /* Register the cloned class. */
    if (!RegisterClass(&wcs))
        {
        pszClass[0] = 0;
        return NULL;
        }

    /* Get the dialog template. */
    if ((hrsrc = FindResource(hins, lpszDlt,
      RT_DIALOG)) == NULL ||
      (hgbl = LoadResource(hins, hrsrc)) == NULL ||
      (lpdlt = LockResource(hgbl)) == NULL)
        goto LCleanup;

    /* Find class name member. */
    bClass = (lpdlt->szMenuName[0] == 0xff ?
        3 : lstrlen(lpdlt->szMenuName) + 1);
    lpszClass = lpdlt->szMenuName + bClass;

    /* Resize dialog template resource if necessary */
    /* to hold new class name. */
    cbClassOld = lstrlen(lpszClass) + 1;
    cbClassNew = lstrlen(pszClass) + 1;
    cbOld = (int)GlobalSize(hgbl);
    if (cbClassNew > cbClassOld)
        {
        UnlockResource(hgbl);
        lpdlt = NULL;
        if ((hgblNew = GlobalReAlloc(hgbl, cbOld +
            cbClassNew - cbClassOld, GMEM_MOVEABLE)) ==
          NULL)
            goto LCleanup;
        hgbl = hgblNew;
        if ((lpdlt = LockResource(hgbl)) == NULL)
            goto LCleanup;
        lpszClass = lpdlt->szMenuName + bClass;
        }

    /* Make room for new class name. */
    _fmemmove(lpszClass + cbClassNew,
      lpszClass + cbClassOld, cbOld -
      (lpszClass - (char far *)lpdlt) - cbClassOld);

    /* Copy new class name into dialog template */
    /* resource. */
    lstrcpy(lpszClass, pszClass);
    fOk = TRUE;

LCleanup:
    if (lpdlt != NULL)
        UnlockResource(hgbl);
    if (!fOk && hgbl != NULL)
        {
        FreeResource(hgbl);
        hgbl = NULL;
        }

    return hgbl;
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK __export
FDlgProc(HWND hwnd, UINT wm, WPARAM wParam, LPARAM lParam)
    {
    switch (wm)
        {
    default:
        break;

    case WM_INITDIALOG:
        return TRUE;

    case WM_COMMAND:
        if (wParam == IDOK || wParam == IDCANCEL)
            {
            EndDialog(hwnd, TRUE);
            return TRUE;
            }
        break;  /* End case WM_COMMAND. */
        }       /* End switch wm. */

    return FALSE;
    }
