/*****************************************************/
/* editval.c                                         */
/* -- Sample edit field validation dialog.           */
/* -- To build: "cc -DSTRICT editval.c editval.rc"   */
/*****************************************************/
#include <windows.h>
#include "editval.h"

#define cw  2   /* Number of edit fields. */

typedef struct
    {
    UINT    rgw[cw];
    } EFV;  /* Edit field values. */

struct  /* Edit Field Limits. */
    {
    UINT    wMin, wMax;
    } rgefl[] =
    {
        { 1000, 2000 },
        { 2000, 3000 }
    };

BOOL CALLBACK __export FDlgProc(HWND hwnd, UINT wm,
    WPARAM wParam, LPARAM lParam);
BOOL FParseEditsEfv(EFV * pefv, HWND hwnd);

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lpszCmd, int wShow)
    {
    EFV     efv;    /* Initial and final values. */
    char    sz[128];

    efv.rgw[0] = efv.rgw[1] = 0;
    DialogBoxParam(hins,
      MAKEINTRESOURCE(dlgEditVal), NULL, FDlgProc,
      (LONG)(EFV far *)&efv);
    wsprintf(sz, "Edit 1 = %d, Edit 2 = %d",
      efv.rgw[0], efv.rgw[1]);
    MessageBox(NULL, sz, "Final values", MB_OK);
    return 0;
    }

BOOL CALLBACK __export
FDlgProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Dialog procedure.                              */
/*****************************************************/
    {
    static EFV far *    lpefv;

    switch (wm)
        {
    default:
        break;

    case WM_INITDIALOG:
        {
        int iw;

        /* Remember where to return values. */
        lpefv = (EFV far *)lParam;

        /* Initialize edit fields. */
        for (iw = 0; iw < cw; iw++)
            SetDlgItemInt(hwnd, didEdit1 + iw,
              lpefv->rgw[iw], FALSE);
        }
        return TRUE;

    case WM_COMMAND:
        switch (wParam)
            {
        default:
            break;

        case IDOK:  /* Parse edit fields. */
            {
            EFV efv;

            if (FParseEditsEfv(&efv, hwnd))
                {
                *lpefv = efv;
                EndDialog(hwnd, TRUE);
                return TRUE;
                }
            }
            break;

        case IDCANCEL:
            EndDialog(hwnd, FALSE);
            return TRUE;
            }               /* End switch wParam. */
        break;              /* End case WM_COMMAND. */
        }                   /* End switch wm. */

    return FALSE;
    }

BOOL
FParseEditsEfv(EFV * pefv, HWND hwnd)
/*****************************************************/
/* -- Parse the dialog's edits and return in *pefv.  */
/* -- pefv : Return parsed edit field values here.   */
/* -- hwnd : Dialog window.                          */
/*****************************************************/
    {
    int iw;

    for (iw = 0; iw < cw; iw++)
        {
        BOOL    fOk;
        UINT    did = iw + didEdit1;

        pefv->rgw[iw] =
          GetDlgItemInt(hwnd, did, &fOk, FALSE);
        if (!fOk || pefv->rgw[iw] < rgefl[iw].wMin ||
          pefv->rgw[iw] > rgefl[iw].wMax)
            {
            char    sz[128];

            /* Invalid field, inform user, and put */
            /* focus back on field. */
            GetDlgItemText(hwnd, did, sz, sizeof sz);
            wsprintf(sz + lstrlen(sz), " is invalid. "
              "Please enter a number from %d to %d",
              rgefl[iw].wMin, rgefl[iw].wMax);
            MessageBox(hwnd, sz, "Sample Dialog",
              MB_OK);
            SetFocus(GetDlgItem(hwnd, did));
            return FALSE;
            }
        }
    return TRUE;    /* All fields are valid. */
    }
