/*****************************************************/
/* mousedmo.c                                        */
/* -- Program demonstrates enabling the mouse when   */
/*    no mouse is physically present.                */
/* -- To build: "cc -DSTRICT mousedmo.c"             */
/*****************************************************/

#include <windows.h>

LRESULT CALLBACK __export LwWndProc(HWND hwnd, UINT wm,
  WPARAM wParam, LPARAM lParam);

char    szMouseClass[]  = "MouseClass";
int     dxScreen, dyScreen;
int     dxMouse, dyMouse;

#define wGrid   20

int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lpszCmd, int wShow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    MSG     msg;
    HWND    hwnd;

    if (hinsPrev == NULL)
        {
        WNDCLASS    wcs;

        wcs.style = 0;
        wcs.lpfnWndProc = LwWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra = 0;
        wcs.hInstance = hins;
        wcs.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = NULL;
        wcs.lpszClassName = szMouseClass;
        if (!RegisterClass(&wcs))
            return FALSE;
        }

    msg.wParam = 0;
    dxScreen = GetSystemMetrics(SM_CXSCREEN);
    dyScreen = GetSystemMetrics(SM_CYSCREEN);
    dxMouse = dxScreen / wGrid;
    dyMouse = dyScreen / wGrid;
    if (!GetSystemMetrics(SM_MOUSEPRESENT))
        ShowCursor(TRUE);
    if ((hwnd = CreateWindow(szMouseClass,
      "Mouse Demo", WS_OVERLAPPEDWINDOW, CW_USEDEFAULT,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      NULL, NULL, hins, NULL)) != NULL)
        {
        ShowWindow(hwnd, wShow);
        while (GetMessage(&msg, NULL, 0, 0))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }
        }

    if (!GetSystemMetrics(SM_MOUSEPRESENT))
        while (ShowCursor(FALSE) >= 0)
            ;

    return msg.wParam;
    }

LRESULT CALLBACK __export
LwWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Main window procedure.                         */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    case WM_USER:
        ShowCursor(TRUE);
        break;

    case WM_KEYDOWN:
        {
        POINT   pt;

        GetCursorPos(&pt);
        switch (wParam)
            {
        default:
            goto LCallDef;

        case VK_LEFT:
            pt.x -= dxMouse;
            break;

        case VK_RIGHT:
            pt.x += dxMouse;
            break;

        case VK_UP:
            pt.y -= dyMouse;
            break;

        case VK_DOWN:
            pt.y += dyMouse;
            break;
            }       // End switch wParam.

        pt.x = (pt.x + dxScreen) % dxScreen;
        pt.y = (pt.y + dyScreen) % dyScreen;
        SetCursorPos(pt.x, pt.y);
        }
        return 0;   // End case WM_KEYDOWN.
        }           // End switch wm.

LCallDef:
    return DefWindowProc(hwnd, wm, wParam, lParam);
    }
