// wuiobjdb.c - implement WUIMAN object database.

#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <windows.h>

#include "wuistd.h"
#include "wuiobjdb.h"
#include "wuierror.h"
#include "wuimanui.h"

// Hmm, these should probably be in wuiobjdb.h
static  const   TWuiName    M_DELETE("m_Delete");
static  const   TWuiName    M_ISA("m_IsA");
static  const   TWuiName    M_RENAME("m_Rename");

////////////////////////////////////////////////////////////
// TWuiObjects - Provide arrays of AWuiObject pointers


// TWuiObjects - initialize object with desired array size.

TWuiObjects::TWuiObjects(int Size)
    :   TWuiGenericList(Size)
    {   MEMBERASSERT();
    }

// (private)
// Find - Return position of named child

int     TWuiObjects::Find(const TWuiName Which)
    {   MEMBERASSERT();
    for(int iChild = 0; iChild < NObjects(); ++iChild)
        if(Get(iChild)->Name() == Which)
            return iChild;
    return -1;
    }

void   *TWuiObjects::Insert(AWuiObject *NewObject, const TWuiName To)
    {   MEMBERASSERT();
    ASSERT(NewObject != 0);
    return Insert(NewObject, Find(To));
    }

void   *TWuiObjects::Insert(AWuiObject *NewObject, int Position)
    {   MEMBERASSERT();
    ASSERT(NewObject != 0);
    return TWuiGenericList::Insert((void *)NewObject, Position);
    }

AWuiObject  *TWuiObjects::Get(const TWuiName Which)
    {   MEMBERASSERT();
    int     Position = Find(Which);
    if(Position >= 0)
        return Get(Position);
    else
        return 0;
    }

AWuiObject *TWuiObjects::Remove(const TWuiName Which)
    {   MEMBERASSERT();
    int     Position = Find(Which);
    if(Position >= 0)
        return Remove(Position);
    else
        return 0;
    }

// AWuiObject - supply minimal object functions

AWuiObject::AWuiObject(const TWuiName ObjectName,
                                      const char */*Value*/)
    :   Name_(ObjectName)
    {   MEMBERASSERT();
    }

// Object destructor
AWuiObject::~AWuiObject()
    {   MEMBERASSERT();
    }

// Get - default get attribute handler
//      NOTE: the default is to forward the message, or to return
//            FALSE if this object is the destination
long    AWuiObject::Get(const char *Path,
                      TWuiArgs *Args, AWuiObject *Attribute,
                             char *Buffer, size_t MaxLength)
    {   MEMBERASSERT();
    ASSERT(Path != NULL);
    long    Result  = TRUE; // assume success
    if(*Path)   // if message is for a child
        {
        TWuiName ChildName(Path);
        // advance Path to child name
        Path    += strlen(Path)+1;
        TWuiObjects *MyChildren = Children();
        AWuiObject *Child  = NULL;
        if(MyChildren)
            Child   =MyChildren->Get(ChildName);
        if(Child)
            Result = Child->Get(Path, Args, Attribute,
                                         Buffer, MaxLength);
        else    // else, no child by that name!
            Result  = WUIMAN_ERR_BAD_PATH;
        }
    else        // else message is for me
        Result  = WUIMAN_ERR_BAD_ATTR;
    return Result;
    }

// Set - default Set attribute handler
//      NOTE: the default is to forward the message, or to return
//            FALSE if this object is the destination
long    AWuiObject::Set(const char *Path,
                      TWuiArgs *Args, AWuiObject *Attribute,
                                               char *Buffer)
    {   MEMBERASSERT();
    ASSERT(Path != NULL);
    long    Result  = TRUE; // assume success
    if(*Path)   // if message is for a child
        {
        TWuiName ChildName(Path);
        // advance Path to child name
        Path    += strlen(Path)+1;
        TWuiObjects *MyChildren = Children();
        AWuiObject *Child  = NULL;
        if(MyChildren)
            Child   =MyChildren->Get(ChildName);
        if(Child)
            {
            Result = Child->Set(Path, Args, Attribute,
                                                    Buffer);
            // if error or grandchild, just pass back result
            if(Result < WUIMAN_ERRORS || *Path)
                ;
            // else, no error and message was for child
            else if(Args->AttributeName() == M_DELETE)
                {
                // Since tree has no upward pointers,
                // parent of target node must do deleting
                delete MyChildren->Remove(ChildName);
                }
            }
        else    // else, no child by that name!
            Result  = WUIMAN_ERR_BAD_PATH;
        }
    else        // else message is for me
        {
        if(Args->AttributeName() == M_DELETE)
            ;   // parent will delete
        else if(Args->AttributeName() == M_ISA)
            return FALSE; // we ain't IsA nothin'!
        else if(Args->AttributeName() == M_RENAME)
            Name_   = Buffer;
        else
            Result  = WUIMAN_ERR_BAD_ATTR;
        }
    return Result;
    }

TWuiObjects *AWuiObject::Children()
    {   MEMBERASSERT();
    // default is to not support children
    return NULL;
    }

TWuiObjects *AWuiObject::Attributes()
    {   MEMBERASSERT();
    // default is to not support attributes
    return NULL;
    }

