/* ----------------------------------------------------
 *  LISTING 2
 *
 *  drv.h
 *
 *  Source code Copyright (c) 1993 T.W. Nelson. May
 *  be used only with appropriate acknowledgement of
 *  copyright.
 * ------------------------------------------------- */

#ifndef __DRV_H
#define __DRV_H

typedef int flag_t;

//Driver function type.  All driver functions must
//load register DS on entry, in order to address local
//data.  Calls to driver functions are always 'far'
//calls.  Driver module must also be able to address
//stack-based variables, since module's SS:SP always
//points to client's stack......
//
#define DRVFUNC flag_t _loadds far

// This MUST be set up by driver modules as a
// function export and jump table.  The driver
// initialization procedure must also be listed as
// the 1st function in the table. Table must also
// be terminated by a NULL member....
//
struct expTab {
    char *name;         //driver function name
    DRVFUNC (*pfun)();  //-> driver function
    };

#define MOD_EXT     ".DRV"  //.ext for all drivers
#define MAX_NAME    15

// This tracks loaded driver extension modules.
// Requested module size and pointer to function
// export table MUST be returned by user-
// supplied initializer....
//
struct ModuleDescriptor    {
    unsigned mseg;          //module segment selector
    unsigned modsize;       //module size, pgraphs
    struct expTab *fTab;   //-> func export table
    DRVFUNC (*exit_f)(void); //-> on-exit function
    void *pblock;           //-> malloc'ed block
    void *args;           //-> module setup argument(s)
    char *id;             //-> module id string
    int v_major;            //module version numbers
    int v_minor;
    char mname[MAX_NAME];   //module name, less .ext
    };

DRVFUNC driver_init(struct ModuleDescriptor *);

// Driver extension module manager prototypes
// and return values .....
flag_t DrvLoadModule ( char *modname,
                        unsigned *handle,
                        void *args   );
flag_t DrvGetProcAddr( unsigned handle,
                        char *procname,
                        DRVFUNC (**ppfn)());
flag_t DrvGetModID( unsigned handle,
                     int *major,
                     int *minor,
                     char *idstr    );
flag_t DrvFreeModule( unsigned handle );
flag_t DrvGetModName( unsigned handle,
                       char **name     );
flag_t DrvGetModHandle( char *mname,
                         unsigned *handle );

#define TRUE             0  //OK
#define ERR_MOD_LIMIT   -1  //Too many modules loaded
#define ERR_MOD_ACCESS  -2  //Can't find driver module
#define ERR_NO_MEM      -3  //Out of memory for load
#define ERR_BAD_LOAD    -4  //DOS Exec overlay error
#define ERR_INIT_FAIL   -5  //module init failure
#define ERR_BAD_HANDLE  -6  //Bad handle passed
#define ERR_BAD_LINK    -7  //Can't find function
#define ERR_MOD_NAME    -8  //Can't match module name

#endif   // __DRV_H

/* ----- End of file ------------------------------- */
