/*****************************************************/
/* sysicon.c                                         */
/* -- Display a custom system menu icon.             */
/* -- To compile:                                    */
/*    cc -DSTRICT sysicon.c toolhelp.lib             */
/*****************************************************/
#include <windows.h>
#include <toolhelp.h>

/* Class name and bitmap resource name. */
char      szName[] = "SysIcon";

/* Location of USER's memory DC. */
HDC far * lphdc;

/* Custom system bitmap bar. */
HBITMAP   hbmpMySysBar;

/* Offset of USER memory DC inside USER DGROUP. */ 
#define bhdcRetail  0x0120
#define bhdcDebug   0x0162

LRESULT CALLBACK __export
  LwWndProc(HWND hwnd, UINT wm, WPARAM wParam,
    LPARAM lParam);

HBITMAP HbmpPatchBmp(HBITMAP hbmp, HDC hdcMem);

int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
                                 LPSTR lpsz, int wShow)
    {
    HWND            hwnd;
    MSG             msg;
    SYSHEAPINFO     shi;
    WINDEBUGINFO    wdi;
    HBITMAP         hbmpMySys;
    UINT            bhdc;

    if (hinsPrev == NULL)
        {
        WNDCLASS    wcs;

        wcs.style = 0;
        wcs.lpfnWndProc = LwWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra = 0;
        wcs.hInstance = hins;
        wcs.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = NULL;
        wcs.lpszClassName = szName;
        if (!RegisterClass(&wcs))
            return 0;
        }

    msg.wParam = 0;

    /* Get pointer to USER's memory DC. */
    shi.dwSize = sizeof shi;
    SystemHeapInfo(&shi);
    bhdc = GetWinDebugInfo(&wdi, 0) ?
      bhdcDebug : bhdcRetail;
    lphdc = (HDC far *)MAKELP(shi.hUserSegment, bhdc);

    /* Make a custom copy of the system bitmap bar. */
    if ((hbmpMySys = LoadBitmap(hins, szName)) == NULL)
        return 0;
    if ((hbmpMySysBar = HbmpPatchBmp(hbmpMySys,
      *lphdc)) == NULL)
        goto LCleanup;

    /* Create a window to show off the custom system */
    /* menu icon. */
    if ((hwnd = CreateWindow(szName,
      "Custom System Icon Demo", WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      CW_USEDEFAULT, NULL, NULL, hins, NULL)) != NULL)
        {
        ShowWindow(hwnd, wShow);
        while (GetMessage(&msg, NULL, 0, 0))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }
        }

LCleanup:
    DeleteObject(hbmpMySys);
    if (hbmpMySysBar != NULL)
        DeleteObject(hbmpMySysBar);
    return msg.wParam;
    }

LRESULT CALLBACK __export
LwWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Main window procedure.                         */
/*****************************************************/
    {
    LRESULT lwVal;

    switch (wm)
        {
    default:
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    case WM_NCPAINT:
    case WM_NCACTIVATE:
        {
        HBITMAP hbmpSav;

        /* Temporarilly replace the system bitmap */
        /* bar in USER's memory DC with our own, let */
        /* DefWindowProc() do its thing, and replace */
        /* the original bitmap. */
        hbmpSav = SelectObject(*lphdc, hbmpMySysBar);
        lwVal =
          DefWindowProc(hwnd, wm, wParam, lParam);
        SelectObject(*lphdc, hbmpSav);
        }
        return lwVal;
        }

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

HBITMAP
HbmpPatchBmp(HBITMAP hbmp, HDC hdcMem)
/*****************************************************/
/* -- Create a system bitmap bar clone with the      */
/*    first bitmap replaced with the given one.      */
/* -- hbmp   : Custom system menu icon bitmap.       */
/* -- hdcMem : Memory DC containing system bitmap    */
/*             bar.                                  */
/*****************************************************/
    {
    BITMAP  bmpSysBar, bmp;
    HBITMAP hbmpSysBar, hbmpNew;
    HDC     hdcMemT;

    /* Get dimensions of system bitmap bar and */
    /* custom system menu icon bitmap to substitute */
    /* at start. */
    if ((hbmpSysBar = SelectObject(hdcMem, hbmp)) ==
      NULL)
        return NULL;
    GetObject(hbmpSysBar, sizeof bmpSysBar,
      &bmpSysBar);
    GetObject(hbmp, sizeof bmp, &bmp);
    SelectObject(hdcMem, hbmpSysBar);

    /* Create custom version of system bitmap bar */
    /* and fill with bits from custom system menu */
    /* icon bitmap and remainder of original system */
    /* bitmap bar. */
    if ((hbmpNew = CreateBitmap(bmpSysBar.bmWidth,
      bmpSysBar.bmHeight, bmpSysBar.bmPlanes,
      bmpSysBar.bmBitsPixel, NULL)) == NULL)
        return NULL;
    if ((hdcMemT = CreateCompatibleDC(hdcMem)) != NULL)
        {
        HBITMAP hbmpSav, hbmpSavT;

        /* Copy right part of original system */
        /* bitmap bar. */
        hbmpSavT = SelectObject(hdcMemT, hbmpNew);
        BitBlt(hdcMemT, bmp.bmWidth, 0,
          bmpSysBar.bmWidth - bmp.bmWidth,
          bmpSysBar.bmHeight, hdcMem, bmp.bmWidth, 0,
          SRCCOPY);

        /* Copy custom system menu icon bitmap. */
        hbmpSav = SelectObject(hdcMem, hbmp);
        StretchBlt(hdcMemT, 0, 0, bmpSysBar.bmHeight,
          bmpSysBar.bmHeight, hdcMem, 0, 0,
          bmp.bmWidth, bmp.bmHeight, SRCCOPY);
        if (hbmpSav != NULL)
            SelectObject(hdcMem, hbmpSav);
        if (hbmpSavT != NULL)
            SelectObject(hdcMemT, hbmpSavT);
        DeleteDC(hdcMemT);
        }
    else
        {
        DeleteObject(hbmpNew);
        hbmpNew = NULL;
        }

    return hbmpNew;
    }
