/*
 *  callback.c - Demonstrate C++ member callbacks.
 *
 *  This file simply supplies the user interface. It uses a dialog
 *  box as the main window, which greatly simplifies things.
 */

#include <stdio.h>
#include "callback.h"
#include "ctl3d.h"      // for Microsoft's new 3-D control look
#include "timer.h"

class   MyTimer : public Timer
    {
public:
    MyTimer(HWND Window)
        : Window_(Window), Count(0)
        {}
    void CALLBACK TimerProc(HWND,UINT,UINT,DWORD);
private:
    HWND    Window_;
    int     Count;
    };

void CALLBACK MyTimer::TimerProc(HWND /*Window*/,
         UINT /*Message*/, UINT /*TimerID*/, DWORD /*Time*/)
    {
    char    Buffer[64];
    ++Count;
    sprintf(Buffer, "%d", Count);
    SetWindowText(Window_, Buffer);
    }

BOOL CALLBACK _export DlgProc(HWND, UINT, WPARAM, LPARAM);

HINSTANCE   ThisInstance;   // ubiquitous instance handle.

MyTimer *ATimer;

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE Me, HINSTANCE Previous,
		     LPSTR lpszCmdLine, int nCmdShow)
    {
    ThisInstance    = Me;
    Ctl3dRegister(ThisInstance);
    Ctl3dAutoSubclass(ThisInstance);
    DialogIcon(LoadIcon(ThisInstance, "CallBack_Icon"),
                    Me, "CallBack", NULL, (DLGPROC)DlgProc);
    if(ATimer)
        delete ATimer;
    Ctl3dUnregister(ThisInstance);
    return 0;
    }


#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK _export DlgProc(HWND Dialog, UINT Message,
                             WPARAM Param1, LPARAM Param2)
    {
    if(Message == WM_INITDIALOG)
        {
        Ctl3dSubclassDlgEx(Dialog, CTL3D_ALL);
        ATimer  = new MyTimer(
                       GetDlgItem(Dialog, ID_TIMER_STATIC));
        return TRUE;
        }
    else if(Message == WM_COMMAND)
        if(Param1 == IDOK || Param1 == IDCANCEL) {
            EndDialog(Dialog, 0);
            return TRUE;
            }
        else if(Param1 == ID_START_BUTTON)
            ATimer->Start(1000);
        else if(Param1 == ID_STOP_BUTTON)
            ATimer->Stop();
    return FALSE;
    }


