// ****************************************************************************
// File:    dbbitmap.cpp
//
// Date:    October 22, 1994
//
// Author:  Greg Johnson
//
// Compiler:    Microsoft Visual C++ version 1.5, Microsoft Foundation Class
//              Library (MFC), version 2.5
//
// Purpose: This file defines the fdBitmap class, which is derived from the
//          MFC CBitmap class to provide additional information about a bitmap
//          and a storage/retrieval mechanism for utilizing a database table
//          for persistence of the bitmap.
//
// ****************************************************************************

#include "stdafx.h"
#include "bmp.h"

#ifndef _DbBmpRec_
#include "dbbmprec.h"
#endif

#ifndef _DbBitmap_
#include "dbbitmap.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

// ***** ctor *****
dbBitmap::dbBitmap(void)
{
}

// ***** dtor *****
dbBitmap::~dbBitmap(void)
{
}



// ***** store() method - stores the bitmap into the *****
// ***** database thru the dbBmpRecordset class.     *****
// ***** NOTE: this store method will replace the    *****
// ***** bitmap if a record already exists with the  *****
// ***** same bitmap identifier.                     *****
// ***** Returns a non-zero value if the store       *****
// ***** operation fails.                            *****

int dbBitmap::store(CDatabase* pDB)
{
    int ret= -1;
    BOOL upd= FALSE;
    
    TRY
        {
        CString filter("ID= %d");
        dbBmpRecordset rs(pDB);
        
        // ***** open the recordset and query for the current *****
        // ***** bitmap id.  If found it is an update, else   *****
        // ***** add a new record.                            *****
        
        sprintf(rs.m_strFilter.GetBufferSetLength(255),
                filter.GetBuffer(filter.GetLength()),iBmpID);
        rs.m_strFilter.ReleaseBuffer(-1);
        
        rs.Open();
        rs.MoveFirst();
        
        if(!rs.IsBOF() && !rs.IsEOF())
            {
            rs.Edit();
            upd= TRUE;
            }
        else
            rs.AddNew();
        
        // ***** if new record, need the identifier *****    
        if(!upd)
            rs.qryID()= iBmpID;    
        
        // ***** save the description *****
        rs.qryDescr()= szDescr;
        
        // ***** save the bitmap *****
        rs.setBitmap(this);
        
        // ***** put in the database *****
        rs.Update();
        
        // ***** now clean up *****
        rs.Close();
        ret= 0;
        }
    CATCH(CDBException,e)
        {
        // ***** OH-OH! return error code from ODBC *****
        ret= e->m_nRetCode;
        #ifdef _DEBUG
        OutputDebugString(e->m_strError);
        OutputDebugString(e->m_strStateNativeOrigin);
        #endif
        }
    END_CATCH
    
    
    return(ret);
}



// ***** fetch() method - retrieves a bitmap from the *****
// ***** database. Returns a non-zero value if the    *****
// ***** operation fails or the bitmap cannot be found*****
// ***** with the ID that is passed in.               *****

int dbBitmap::fetch(int id,CDatabase* pDB)
{
    int ret= FALSE;
    CString filter("ID=%d");
    iBmpID= id;
    
    TRY
        {
        dbBmpRecordset rs(pDB);
        
        // ***** open the recordset and query for the         *****
        // ***** bitmap id.  If found, retrieve it and put    *****
        // ***** the contents into this bitmap object.        *****
        
        sprintf(rs.m_strFilter.GetBufferSetLength(255),
                filter.GetBuffer(filter.GetLength()),id);
        rs.m_strFilter.ReleaseBuffer(-1);
        
        ret= rs.Open();
        rs.MoveFirst();
        
        if(!rs.IsBOF() && !rs.IsEOF())
            {
            szDescr= rs.qryDescr();
            rs.qryBitmap(this);
            }
        else
            ret= -1;
            
        rs.Close();
        }
    CATCH(CDBException,e)
        {
        ret= FALSE;
        #ifdef _DEBUG
        OutputDebugString(e->m_strError);
        OutputDebugString(e->m_strStateNativeOrigin);
        #endif
        }
    END_CATCH
    
    return(ret);
}



// ***** ReadBmpFile() method - Loads a DIB from a file *****
// ***** and attaches the bitmap to this bitmap object. *****

BOOL dbBitmap::ReadBmpFile(CString& file)
{
    BOOL ret= FALSE;
    
    HBITMAP hBmp= ::ReadBmpFile(file.GetBuffer(file.GetLength()));

    if(hBmp)
        {
        ret= Attach(hBmp);
        }
    
    return(ret);
}



        