// ****************************************************************************
// File:    dbbmprec.cpp
//
// Date:    October 22, 1994
//
// Author:  Greg Johnson
//
// Compiler:    Microsoft Visual C++ version 1.5, Microsoft Foundation Class
//              Library (MFC), version 2.5
//
// Purpose: This file defines the dbBmpRecordset class, which is used as an
//          interface to the Bitmaps database table for storing and retrieving
//          Windows bitmaps and an associated identifier and description.
//
//          The dbBmpRecordset class is derived from the CRecordset MFC class
//          for specifics to the Bitmaps table in the BMP.MDB database, which
//          is a Microsoft Access version 1.1 format.
// 
// ****************************************************************************


#include "stdafx.h"
#include "db_bmp.h"


#ifndef _DbBitmap_
#include "dbbitmap.h"
#endif


#ifndef _DbBmpRec_
#include "dbbmprec.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////////////////////////
// dbBmpRecordset

IMPLEMENT_DYNAMIC(dbBmpRecordset, CRecordset)

dbBmpRecordset::dbBmpRecordset(CDatabase* pdb)
    : CRecordset(pdb)
{
    //{{AFX_FIELD_INIT(dbBmpRecordset)
    m_ID = 0;
    m_Descr = "";
    m_nFields = 3;
    //}}AFX_FIELD_INIT
}


dbBmpRecordset::~dbBmpRecordset(void)
{
}


BOOL dbBmpRecordset::qryBitmap(dbBitmap* pBitmap)
{
    BOOL ret= FALSE;
    BYTE* bufr= NULL;
    BYTE* tmp= NULL;
    BITMAP bm;
    CDC dc;
    int sz= 0;
    
    if(pBitmap)
        {
        if(pBitmap->m_hObject)
            ::DeleteObject(pBitmap->Detach());
        
        if(!IsBOF() && !IsEOF())
            {
            dc.CreateDC("DISPLAY",NULL,NULL,NULL);
            bufr= (BYTE*)GlobalLock(m_Bmp.m_hData);
            tmp= bufr + sizeof(BITMAP);
            memcpy(&bm,bufr,sizeof(BITMAP));
            sz= ((bm.bmWidthBytes * bm.bmHeight) * bm.bmPlanes);
            pBitmap->CreateCompatibleBitmap(&dc,bm.bmWidth,bm.bmHeight);
            pBitmap->SetBitmapBits(sz,tmp);
            GlobalUnlock(m_Bmp.m_hData);
            ret= TRUE;
            }
        }    
    
    return(ret);
}    


BOOL dbBmpRecordset::setBitmap(dbBitmap* pBitmap)
{
    BOOL ret= FALSE;
    BYTE* bufr= NULL;       // buffer for the bitmap struct and the bits
    BYTE* tmp= NULL;        // pointer to the start of the bits
    BITMAP bm;
    int sz= NULL;
    
    if(IsOpen())
        {
        if(pBitmap && pBitmap->m_hObject)
            {
            // ***** get bitmap info *****
            pBitmap->GetObject(sizeof(BITMAP),&bm);
            
            // ***** calculate buffer needed for the bitmap bits *****
            sz= ((bm.bmWidthBytes * bm.bmHeight) * bm.bmPlanes);
            
            // ***** allocate the buffer for the bitmap struct AND the bits *****
            bufr= new BYTE[sz + sizeof(BITMAP) + 1];
            memset(bufr,'\0',sz + sizeof(BITMAP) + 1);
            
            // ***** locate start of bits after the bitmap struct *****
            tmp= bufr + sizeof(BITMAP);
            
            // ***** copy the bitmap struct *****
            memcpy(bufr,&bm,sizeof(BITMAP));
            
            // ***** copy the bitmap bits *****
            pBitmap->GetBitmapBits(sz,tmp);
            
            // ***** get rid of previous bitmap, if needed *****
            if(m_Bmp.m_hData)
                GlobalFree(m_Bmp.m_hData);
                
            // ***** allocate the bitmap space *****
            m_Bmp.m_hData= GlobalAlloc(GHND,sz+sizeof(BITMAP)+1);
            tmp= (BYTE*)GlobalLock(m_Bmp.m_hData);
            
            // ***** copy the bitmap into the long binary field *****
            memcpy(tmp,bufr,sz + sizeof(BITMAP));
            GlobalUnlock(m_Bmp.m_hData);
            
            // ***** now, remember how big the bitmap is ******
            m_Bmp.m_dwDataLength= sz + sizeof(BITMAP);
            
            // ***** required to manually indacte this field needs *****
            // ***** to be stored in the database since it changed *****
            SetFieldDirty(&(m_Bmp));
            SetFieldNull(&(m_Bmp),FALSE);
            
            if(bufr)
                delete[] bufr;
                
            ret= TRUE;
            }

        }
    
    return(ret);
}    



CString dbBmpRecordset::GetDefaultConnect()
{
    return "ODBC;DSN=BitmapDB;";
}

CString dbBmpRecordset::GetDefaultSQL()
{
    return "BITMAPS";
}

void dbBmpRecordset::DoFieldExchange(CFieldExchange* pFX)
{
    //{{AFX_FIELD_MAP(dbBmpRecordset)
    pFX->SetFieldType(CFieldExchange::outputColumn);
    RFX_Int(pFX, "ID", m_ID);
    RFX_Text(pFX, "Descr", m_Descr);
    RFX_LongBinary(pFX, "Bmp", m_Bmp);
    //}}AFX_FIELD_MAP
}
