#define STRICT
#include <windows.h>
#include <commdlg.h>
#include <string.h>

#include "win95api.h"
#include "win95.rh"

HINSTANCE hInst;    // app's HINSTANCE

#if defined(_MSC_VER) && defined(_WIN32)
#define _export
#endif

#ifdef __BORLANDC__
#   pragma argsused
#endif
BOOL CALLBACK _export DlgProc( HWND hwnd, UINT msg,
    WPARAM wParam, LPARAM lParam )
// dialog box proc for demo dialog box
{
   switch(msg) {
      case WM_INITDIALOG:
         return TRUE;

      case WM_COMMAND:
         switch(LOWORD(wParam)) {
            case IDOK:
            case IDCANCEL:
               EndDialog(hwnd,TRUE);
               return TRUE;
         }
         break;
   }
   return FALSE;
}


void OpenFileDlg( HWND hwnd )
{
  // display commdlg open file dialog
  static char szFileName[256];
  static OPENFILENAME ofn;
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = hwnd;
  ofn.lpstrFile = szFileName;
  ofn.nMaxFile = sizeof(szFileName);
  ofn.lpstrFilter = "All Files (*.*)\000*.*\000";
  ofn.Flags = OFN_HIDEREADONLY | OFN_LONGNAMES;
  if( GetOpenFileName(&ofn) ) {
     // open file with _lopen, which handles long filenames
     HFILE h = _lopen(szFileName,OF_READ);
     if(h!=HFILE_ERROR) {
        long size = _llseek(h,0,2); // get size
        char buf[512];
        wsprintf(buf,"%s is %ld bytes long",szFileName,size);
        MessageBox(hwnd,buf,"File Size",MB_ICONINFORMATION
                    |MB_OK);
        _lclose(h);
     }
  }
}


LRESULT CALLBACK _export MainWndProc( HWND hwnd, UINT msg,
   WPARAM wParam, LPARAM lParam )
{
   switch(msg) {
      case WM_CREATE: {
         // load popup menu
         HMENU hMenu = LoadMenu(hInst,MAKEINTRESOURCE
                                (MENU_POPUP));
         hMenu = GetSubMenu(hMenu,0);
         SetMenuDefaultItem(hMenu,CM_POPUPITEM1,
                            MF_BYCOMMAND);
         SetProp(hwnd,"popup",hMenu);  
         // intialize radio menu item
         CheckMenuRadioItem(GetMenu(hwnd),CM_ITEM1,
                            CM_ITEM4,CM_ITEM1,MF_BYCOMMAND);
         return 0;
      }
      case WM_COMMAND:
         switch(LOWORD(wParam)) {
            case CM_FILEOPEN:
               OpenFileDlg(hwnd);
               return 0;
            case CM_DIALOG:
               DialogBox(hInst,MAKEINTRESOURCE(DLG_BOX),
                         hwnd,DlgProc);
               return 0;
            case CM_FILEEXIT:
               SendMessage(hwnd,WM_CLOSE,0,0);
               return 0;
            case CM_ITEM1:
            case CM_ITEM2:
            case CM_ITEM3:
            case CM_ITEM4:
               CheckMenuRadioItem(GetMenu(hwnd),CM_ITEM1,
                              CM_ITEM4,wParam,MF_BYCOMMAND);
               return 0;
            case CM_ITEM5: {
               // toggle checked menu item
               HMENU hmenu = GetMenu(hwnd);
               if( GetMenuState(hmenu,CM_ITEM5,MF_BYCOMMAND)
                                                 &MF_CHECKED)
                  CheckMenuItem(hmenu,CM_ITEM5,MF_UNCHECKED
                                |MF_BYCOMMAND);
               else
                  CheckMenuItem(hmenu,CM_ITEM5,MF_CHECKED
                                |MF_BYCOMMAND);
               return 0;
            }
            case CM_POPUPITEM1:
            case CM_POPUPITEM2: {
               char buf[256];
               wsprintf(buf,"Popup Item %s",
                   LOWORD(wParam)==CM_POPUPITEM1?"1":"2");
               MessageBox(hwnd,buf,"Popup",
                          MB_ICONINFORMATION|MB_OK);
               return 0;
            }
         }
         break;
      case WM_RBUTTONDBLCLK:
      case WM_LBUTTONDBLCLK:
         // emulate default menu item for double click
         PostMessage(hwnd,WM_COMMAND,CM_POPUPITEM1,0);
         return 0;
      case WM_RBUTTONDOWN: {
         // display context menu
         POINT p;
         p.x = LOWORD(lParam);
         p.y = HIWORD(lParam);
         ClientToScreen(hwnd,&p);
         TrackPopupMenu((HMENU)GetProp(hwnd,"popup"),
            TPM_LEFTALIGN|TPM_RIGHTBUTTON,
            p.x,p.y,0,hwnd,NULL);
         return 0;
      }
      case WM_SIZE:
      case WM_MOVE: {
         // display scroll bars proportianal to size of
         // window on desktop
         RECT r;
         SCROLLINFO vsi, hsi;
         hsi.cbSize = vsi.cbSize = sizeof(vsi);
         hsi.fMask = vsi.fMask = SIF_RANGE | SIF_POS
                        | SIF_PAGE | SIF_DISABLENOSCROLL;
         hsi.nMin = vsi.nMin = 0;
         vsi.nMax = GetSystemMetrics(SM_CYSCREEN);
         hsi.nMax = GetSystemMetrics(SM_CXSCREEN);
         GetWindowRect(hwnd,&r);
         vsi.nPos = r.top;
         hsi.nPos = r.left;
         vsi.nPage = r.bottom-r.top;
         hsi.nPage = r.right - r.left;
         SetScrollInfo(hwnd,SB_VERT,&vsi,TRUE);
         SetScrollInfo(hwnd,SB_HORZ,&hsi,TRUE);
         break;
      }
      case WM_DESTROY:
         RemoveProp(hwnd,"popup");
         PostQuitMessage(0);
         return 0;
   }
   return DefWindowProc(hwnd,msg,wParam,lParam);
}

int MessagePump(void)
{
   MSG msg;
   while( GetMessage(&msg,NULL,0,0) ) {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return msg.wParam;
}

#ifdef __BORLANDC__
#   pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInst,
                    LPSTR lpstrCmdLine, int nCmdShow )
{
   char szClassName[] = "Windows 95 API Test "
      #ifdef WIN32
      "(32)";
      #else
      "(16)";
      #endif
   HWND hwnd;
   hInst = hInstance;
   if(!hPrevInst) {
      WNDCLASS wc;
      memset(&wc,0,sizeof(wc));
      wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
      wc.style         = CS_DBLCLKS;
      wc.hCursor       = LoadCursor(NULL,IDC_ARROW);
      wc.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
      wc.lpfnWndProc   = MainWndProc;
      wc.lpszClassName = szClassName;
      wc.hInstance     = hInst;
      wc.lpszMenuName  = MAKEINTRESOURCE(MENU_MAIN);
      RegisterClass(&wc);
   }
   hwnd = CreateWindow(szClassName,szClassName,
               WS_OVERLAPPEDWINDOW|WS_VSCROLL|WS_HSCROLL,
               CW_USEDEFAULT,CW_USEDEFAULT,250,250,
               NULL,NULL,hInst,NULL);
   if(hwnd) {
      ShowWindow(hwnd,nCmdShow);
      UpdateWindow(hwnd);
      return MessagePump();
   }
   return -1;
}

