#define STRICT
#include <windows.h>
#include "win95api.h"
#include <limits.h>

#if WINVER<0x0400 //don't compile if targeted for Windows 95

#ifdef WIN32
#define USER() GetModuleHandle("USER32")
#else
#define USER() GetModuleHandle("USER")
#endif

BOOL WINAPI SetScrollInfo(HWND hwnd, int flags,
                          LPCSCROLLINFO lpsi, BOOL redraw)
{
   typedef BOOL (WINAPI *FN)(HWND,int,LPCSCROLLINFO,BOOL);
   static FN func;
   static BOOL loaded;
   if(!loaded && !func) {
      func = (FN)GetProcAddress(USER(),"SetScrollInfo");
      loaded = TRUE;
   }
   if(func)
      return func(hwnd,flags,lpsi,redraw);
   else {
      if(lpsi->fMask&SIF_RANGE) {
         if(  lpsi->nMin > INT_MAX ||
              lpsi->nMax > INT_MAX ||
              lpsi->nMin < -INT_MAX ||
              lpsi->nMin < -INT_MAX )
            return FALSE;
         SetScrollRange(hwnd,flags,lpsi->nMin,
                        lpsi->nMax,redraw);
      }
      if(lpsi->fMask&SIF_POS) {
         if( lpsi->nPos < -INT_MAX || lpsi->nPos > -INT_MAX )
            return FALSE;
         SetScrollPos(hwnd,flags,lpsi->nPos,redraw);
      }
      return TRUE;
   }
}

BOOL WINAPI GetScrollInfo(HWND hwnd,int flags,
                          LPSCROLLINFO lpsi)
{
   typedef BOOL (WINAPI *FN)(HWND,int,LPSCROLLINFO);
   static FN func;
   static BOOL loaded;
   if(!loaded && !func) {
      func = (FN)GetProcAddress(USER(),"GetScrollInfo");
      loaded = TRUE;
   }
   if(func)
      return func(hwnd,flags,lpsi);
   else {
      if(lpsi->fMask & SIF_RANGE ) {
         lpsi->nMin = 0;
         lpsi->nMax = 0;
         GetScrollRange(hwnd,flags,(int FAR *)&lpsi->nMin,
                                (int FAR *)&lpsi->nMax);
      }
      if(lpsi->fMask & SIF_POS )
         lpsi->nPos = GetScrollPos(hwnd,flags);
      if(lpsi->fMask & SIF_PAGE )
         lpsi->nPage = 1;
      if(lpsi->fMask & SIF_TRACKPOS )
         return FALSE;  // can't get this
      return TRUE;
   }
}

BOOL WINAPI SetMenuDefaultItem( HMENU hmenu, UINT id,
                                UINT fbypos )
{
   typedef BOOL (WINAPI *FN)(HMENU,UINT,UINT);
   static FN func;
   static BOOL loaded;
   if(!loaded && !func) {
      func = (FN)GetProcAddress(USER(),"SetMenuDefaultItem");
      loaded = TRUE;
   }
   if(func)
      return func(hmenu,id,fbypos);
   else
      return FALSE;
}

UINT WINAPI GetMenuDefaultItem( HMENU hmenu, UINT fByPos,
                                UINT fFlags )
{
   typedef BOOL (WINAPI *FN)(HMENU,UINT,UINT);
   static FN func;
   static BOOL loaded;
   if(!loaded && !func) {
      func = (FN)GetProcAddress(USER(),"GetMenuDefaultItem");
      loaded = TRUE;
   }
   if(func)
      return func(hmenu,fByPos,fFlags);
   else
      return FALSE;
}

BOOL WINAPI CheckMenuRadioItem( HMENU hmenu, UINT idFirst,
                    UINT idLast, UINT idCheck, UINT fByPos )
{
   typedef BOOL (WINAPI *FN)(HMENU,UINT,UINT,UINT,UINT);
   static FN func;
   static BOOL loaded;
   if(!loaded && !func) {
      func = (FN)GetProcAddress(USER(),"CheckMenuRadioItem");
      loaded = TRUE;
   }
   if(func)
      return func(hmenu,idFirst,idLast,idCheck,fByPos);
   else {
      // do it the old-fashioned way for 3.x
      UINT i;
      for(i=idFirst;i<=idLast;i++)
         if(i==idCheck)
            CheckMenuItem(hmenu,i,fByPos|MF_CHECKED);
         else
            CheckMenuItem(hmenu,i,fByPos|MF_UNCHECKED);
      return TRUE;
   }
}

#endif
