// VIDSETUP.C - complete code for VidSetup helper program

#include <windows.h>
#include <ver.h>

long FAR PASCAL _export WndProc(HWND, WORD, WORD, LONG);
#define MAX_BUF	260	// file pathname buffer size
#define TIME_OUT	5000	// timeout after 5000 milliseconds
#define ID_STARTTEST	100	// WM_COMMAND message id

PSTR szAppName = "VIDSETUP";	// window class name
PSTR szCaption = "Video Setup";		// message box caption
HINSTANCE hInst;
PSTR szTestFile;				// test file pointer

PSTR szSetupPath = "VFW\\SETUP";	// path off module directory for VFW setup progrem
PSTR szDefMsg = "Default message";			// various messages
PSTR szAskMsg = "Ask if want to install message";
PSTR szCloseMsg = "Close without install message";
PSTR szErrMsg = "Error during setup message";

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
	LPSTR lpszCmdLine, int nCmdShow) {
	HWND		hWnd;
	MSG		msg;
	WNDCLASS	wndclass;
					// if no test file in command line, message and end
	if(!lpszCmdLine || !lpszCmdLine[0]) {
		MessageBox(NULL,szDefMsg,szCaption,MB_OK|MB_ICONEXCLAMATION);
		return 0;
	}
	if(!hPrevInstance) {
		wndclass.style		= CS_HREDRAW | CS_VREDRAW ;
		wndclass.lpfnWndProc	= (WNDPROC)WndProc ;
		wndclass.cbClsExtra	= 0 ;
		wndclass.cbWndExtra	= 0 ;
		wndclass.hInstance	= hInstance ;
		wndclass.hIcon		= NULL ;
		wndclass.hCursor	= LoadCursor(NULL, IDC_ARROW) ;
		wndclass.hbrBackground	= COLOR_WINDOW + 1 ;
		wndclass.lpszMenuName	= NULL ;
		wndclass.lpszClassName	= szAppName ;
		RegisterClass(&wndclass) ;
	}				// put globals in DS
	hInst = hInstance;
	szTestFile = (PSTR)LocalAlloc(LPTR,lstrlen(lpszCmdLine)+1);
	if(szTestFile) lstrcpy(szTestFile,lpszCmdLine);
					// create window but don't show
	hWnd = CreateWindow(szAppName,NULL,WS_OVERLAPPED,
		0,CW_USEDEFAULT,0,CW_USEDEFAULT,
		HWND_DESKTOP,(HMENU)NULL,hInst,NULL);
	while(GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}
// return TRUE if version info for lpFileName is returned in lpdwVersion
static BOOL GetFileVersion(LPCSTR lpFileName, LPDWORD lpdwVersion) {
	DWORD dwhVersion, dwBufSize;
	BOOL bRet = FALSE;
						// get size of info
	dwBufSize = GetFileVersionInfoSize(lpFileName,&dwhVersion);
	if(dwBufSize != NULL) {			// if info available, alloc buffer, lock and copy
	     HGLOBAL hg;
	     LPSTR lp;
	     if(hg = GlobalAlloc(GHND,dwBufSize)) {
		if(lp = (LPSTR)GlobalLock(hg))
		{
		     if(GetFileVersionInfo(lpFileName,dwhVersion,dwBufSize,lp)) {
			VS_FIXEDFILEINFO FAR* lpVer;
			WORD wSize;		// get pointer to structure
			if(VerQueryValue(lp,"\\",&lpVer,&wSize)) {
			     if(wSize != 0) {	// if successful, copy and set flag
				*lpdwVersion = lpVer->dwProductVersionMS;
				bRet = TRUE; 
			     }
			}
		     }
		     GlobalUnlock(hg);
		}
		GlobalFree(hg);
	     }
	}
	return bRet;
}
static BOOL IsCorrectVersionVFW(void) {	// return TRUE if VFW version >= 1.1
	char str[MAX_BUF];
	int nlen;
	DWORD dwVersion;
	BOOL bRet = FALSE;
					// get path to VFW library
	nlen = GetSystemDirectory(str,sizeof(str));
	if(nlen == 0 || nlen > sizeof(str)) return FALSE;
	if(*(str + nlen) != '\\') lstrcat(str,"\\");
	lstrcat(str,"MSVIDEO.DLL");

	if(GetFileVersion(str,&dwVersion))
		bRet = (HIWORD(dwVersion) > 1) || (LOWORD(dwVersion) >= 10);
	return bRet;
}
static BOOL WaitUntilClose(HWND hWnd) {	// returns TRUE if first setup closed
	OFSTRUCT ofs;
	DWORD dwTimeOut = TIME_OUT;
						// use PeekMessage to ensure first
	dwTimeOut += GetTickCount();		// setup gets cycle time to close
	while(dwTimeOut > GetTickCount()) {	// break after timeout
		MSG msg;			
		PeekMessage(&msg,hWnd,0,0,PM_REMOVE);
		if(IsWindow(msg.hwnd)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}				// if file doesn't exist return TRUE
		if(!szTestFile || OpenFile(szTestFile,&ofs,OF_EXIST) == HFILE_ERROR)
			return TRUE;
	}
	return FALSE;				// file still exists, return FALSE
}
static BOOL LaunchSetup(HWND hWnd) {		// return TRUE if VFW Setup launched ok
	char szCmd[MAX_BUF];
	LPSTR lpstr;
	WORD wRet;
	
	if(!WaitUntilClose(hWnd)) return FALSE;
	else {					// first setup closed, ask if ok to setup VFW
		if(MessageBox(hWnd,szAskMsg,
			szCaption,MB_YESNO|MB_ICONQUESTION) == IDNO) {
						// user declines, show msg and return
			MessageBox(hWnd,szCloseMsg,
				szCaption,MB_OK|MB_ICONEXCLAMATION);
			return TRUE;
		}				// get VFW setup pathname
		GetModuleFileName(hInst,szCmd,sizeof(szCmd));
		lpstr = szCmd + lstrlen(szCmd);
		while(lpstr != szCmd && *lpstr != '\\' && *lpstr != ':') lpstr--;
		if(*lpstr != '\\') *(++lpstr) = '\\';
		*(++lpstr) = '\0';
		lstrcat(szCmd,szSetupPath);	// execute and return result
		wRet = WinExec(szCmd,SW_SHOW);
		return (wRet >= 32);
	}
}
static void InstallVFW(HWND hWnd) {	// install VFW setup, show message on error
	if(!LaunchSetup(hWnd))
		MessageBox(hWnd,szErrMsg,szCaption,MB_OK|MB_ICONSTOP);
}
long FAR PASCAL _export WndProc(HWND hWnd, WORD message, WORD wParam, LONG lParam) {
	switch(message) {
		case WM_CREATE:	// after create main start test
			PostMessage(hWnd,WM_COMMAND,ID_STARTTEST,0L);
		return 0L;
		case WM_COMMAND:	// if not correct version, install and close
			if(wParam == ID_STARTTEST) {
			    if(!IsCorrectVersionVFW()) InstallVFW(hWnd);
			    PostMessage(hWnd,WM_CLOSE,0,0L);
			    return 0L;
			}
		break;
		case WM_DESTROY:
			PostQuitMessage(0);
		return 0L;
    }
	return DefWindowProc(hWnd,message,wParam,lParam);
}
