// Demonstration program for the microTimer class.

#include <windows.h>
#include <windowsx.h>
#include "microTimer.hpp"

HINSTANCE hInstance;

void
microSleep(DWORD microsecs)
{
	microTimer timer;

	timer.Start();

	while (timer.GetElapsedTime() < microsecs);
}

void
TestMicroSleep(HWND hwnd)
{
	microTimer timer;

	timer.Start();

	microSleep(500);

	DWORD dwElapsed = timer.GetElapsedTime();

	char message[50];
	wsprintf(message, "We slept for %lu microseconds.", dwElapsed);
	MessageBox(hwnd, message, "Sleep", MB_OK | MB_ICONINFORMATION);
}

void
DoBenchmark(HWND hwnd)
{
	int b;
	microTimer timer;
	RECT rc;
	HDC hdc;
	HBRUSH hBrush;
	DWORD dwElapsed;

	SetRect(&rc, 220, 10, 520, 320);

	hdc = GetDC(hwnd);

	timer.Start();

	for (b = 0; b < 256; b++)
	{
		hBrush = CreateSolidBrush(RGB(128, 0, b));
		FillRect(hdc, &rc, hBrush);
		DeleteBrush(hBrush);
	}
  
	dwElapsed = timer.GetElapsedTime();
	
	ReleaseDC(hwnd, hdc);
	
	char message[200];
	wsprintf(message, "FillRect() took %lu microseconds.", dwElapsed);
	MessageBox(hwnd, message, "Benchmark", MB_OK | MB_ICONINFORMATION);
}

HWND hwndSleepBtn, hwndBenchmarkBtn;

LRESULT WINAPI MainWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_CREATE:
			
			hwndSleepBtn = CreateWindow("BUTTON",
												 "Test microSleep()",
												 WS_VISIBLE | WS_CHILD,
												 10,
												 10,
												 200,
												 50,
												 hwnd,
												 NULL,
												 hInstance,
												 NULL);

			hwndBenchmarkBtn = CreateWindow("BUTTON",
													  "Benchmark test",
													  WS_VISIBLE | WS_CHILD,
													  10,
													  70,
													  200,
													  50,
												 	  hwnd,
												 	  NULL,
												 	  hInstance,
												 	  NULL);

			break;

		case WM_COMMAND:
			
			if ( (HWND)lParam == hwndSleepBtn )
				TestMicroSleep(hwnd);
			else
			{
				if ( (HWND)lParam == hwndBenchmarkBtn )
					DoBenchmark(hwnd);
			}

			break;		
		
		case WM_QUIT:
		case WM_DESTROY:					
			PostQuitMessage(0);
			break;
	}

	return DefWindowProc(hwnd, message, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
   if (!hPrevInst)
   {
	   WNDCLASS wc;

	   wc.style = CS_HREDRAW | CS_VREDRAW;		
	   wc.lpfnWndProc = MainWndProc;	
	   wc.cbClsExtra = 0;								
	   wc.cbWndExtra = 0;								
	   wc.hInstance = hInst;	
	   wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	   wc.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
	   wc.lpszMenuName = NULL;						
	   wc.lpszClassName = "DemoWindow";						

	   RegisterClass(&wc);
   }

	hInstance = hInst;

	HWND hwnd;

	hwnd = CreateWindow(
		"DemoWindow", 	          
		"microTimer Demonstration",
		WS_OVERLAPPEDWINDOW,	    
		CW_USEDEFAULT, 			 
		CW_USEDEFAULT,	   	   
		CW_USEDEFAULT,	      	
		CW_USEDEFAULT, 		   
		NULL,		   		      
		NULL,	   			      
		hInst,
		NULL			   	      
	);

	ShowWindow(hwnd, nCmdShow | SW_SHOWMAXIMIZED);
	UpdateWindow(hwnd);			

	MSG msg;

	while ( GetMessage(&msg, NULL, 0, 0) )
	{
		TranslateMessage(&msg);	
		DispatchMessage(&msg);	
	}

	return (msg.wParam);
}
