#include <iostream.h>
#include <string.h>
#include "darray.h"


// Derive a class from CDynamicArray:
class CIntArray : public CDynamicArray<int> {
protected:
    VOID Init(DWORD, DWORD);    // Override default Init()
    
public:
    CIntArray(DWORD size) : CDynamicArray<int>(size)    { }
};

VOID CIntArray::Init(DWORD from, DWORD to)
{
    for ( DWORD i = from ; i <= to ; ++i )
        (*this)(i) = i;
}


// Define a three byte class object:
#if defined(_MSC_VER)
#   pragma pack(1)
#elif defined(__BORLANDC__)
#   pragma option -a1
#endif

class foo {
    char    bar[3];
public:
    foo(void)                      { }
    foo(const int value)           { *this = value; }
    foo& operator=(const int value)
        { memcpy(bar, &value, 3); return *this; }
    operator int() const
        { int value; memcpy(&value, bar, 3);
          return (value << 8) >> 8; }
};

#if defined(_MSC_VER)
#   pragma pack()
#elif defined(__BORLANDC))
#   pragma option -a
#endif


// Access dynamically allocated array members with no
//   exception monitoring:
void Init(CIntArray& iArray)
{
    iArray.Commit(0, 19999);
    for ( int i = 0 ; i < 20000 ; ++i )
        iArray(i) = -i;
}

// Access dynamically allocated array members with exception
//  monitoring:
void Print(CIntArray& iArray)
{
    for ( int i = 0 ; i < 100 ; ++i ) {
        int temp = iArray[i+1000];
        if ( iArray.Fail() )
            return;
        if(i == 23)
            cout << "oh oh ";
        cout << temp << ' ';
    }
}

int main(void)
{
    CIntArray iArray(30000);
    cout<< "Elements per page: " << iArray.PerPage() << '\n';

    Init(iArray);
    Print(iArray);
    cout<<"\nRequested range: " << iArray.BottomRequest() <<
            " - " << iArray.TopRequest() << '\n';
    cout<< "Committed range: " << iArray.BottomCommit() <<
            " - " << iArray.TopCommit() << '\n';

    iArray.Release(50);
    iArray.fCommit = FALSE;
    Print(iArray);
    cout<< "\nRequested range: " << iArray.BottomRequest() <<
            " - " << iArray.TopRequest() << '\n';
    cout<< "Committed range: " << iArray.BottomCommit() <<
            " - " << iArray.TopCommit() << '\n';

    iArray.fCommit = TRUE;
    Print(iArray);
    cout<< "\nRequested range: " << iArray.BottomRequest() <<
            " - " << iArray.TopRequest() << '\n';
    cout<< "Committed range: " << iArray.BottomCommit() <<
            " - " << iArray.TopCommit() << '\n';
    cin.get();
    iArray.Release();

    // Use a dynamically allocated array as an argument for
    //  a function that expects a regular array:
    CDynamicArray<char> cArray(256);
    LPSTR lpsz = "String";
    cArray.Commit(0, strlen(lpsz));
    strcpy(&(*cArray), lpsz);
    cout << LPSTR(&(*cArray)) << '\n';

    // Because 4096 is not divisible by 3 one member will
    //  reside in two pages:
    CDynamicArray<foo> fooArray(30000);
    DWORD dwPerPage = fooArray.PerPage();
    fooArray[dwPerPage] = sizeof(foo);
    cout << fooArray[dwPerPage] << '\n';
    cout<< "Requested range: " << fooArray.BottomRequest() <<
            " - " << fooArray.TopRequest() << '\n';
    cout<< "Committed range: " << fooArray.BottomCommit() <<
            " - " << fooArray.TopCommit() << '\n';

    fooArray.fCommit = FALSE;
    fooArray.dummy = -17;
    cout << fooArray[20000] << '\n';
    cout<< "Requested range: " << fooArray.BottomRequest() <<
            " - " << fooArray.TopRequest() << '\n';
    cout<< "Committed range: " << fooArray.BottomCommit() <<
            " - " << fooArray.TopCommit() << '\n';
    cin.get();

    return 0;
}
