/*
    File: ListView.h
    Author: V.Ramachandran
    Date: 23/11/1995.

    ZListView Class with drag-drop support.
    Override the following functions for more control:
    HandleItemMove - Do the actual move for the item.
    AllowDragOperation - Customize which items can be moved.
    HandleDragMove - Customize the visual feedback while dragging.
*/

#ifndef __ZLISTCTRL__
#define __ZLISTCTRL__

#include "afxcmn.h"

class ZListCtrl : public CListCtrl
{
public:
    ZListCtrl ();

public:
    WORD GetView () const 
            { return (WORD)(GetStyle () & LVS_TYPEMASK); }
    void EnableDragDrop (BOOL bEnable = TRUE)
            { m_bDragDrop = bEnable; }

protected:
    afx_msg void OnLButtonDown (UINT nFlags, CPoint pt);
    afx_msg void OnLButtonUp (UINT nFlags, CPoint pt);
    afx_msg void OnMouseMove (UINT nFlags, CPoint point);
    afx_msg LRESULT OnCustomGetDlgCode (UINT wParam, 
                    LPARAM lParam);

    BOOL OnChildNotify (UINT message, WPARAM wParam, 
                LPARAM lParam, LRESULT *pLResult);

    void DoBeginDrag (NM_LISTVIEW *pLV);
    void DoEndDrag (CPoint point, BOOL bMoveItem);

// Overridables to control drag-drop.   
    virtual void HandleItemMove (int nDragItem, 
                    CPoint point);
    virtual BOOL AllowDragOperation (WORD wView, 
                    int nItem);
    virtual void HandleDragMove (CImageList *pDragImage,
                    POINT& pt);

    DECLARE_MESSAGE_MAP()
    DECLARE_DYNAMIC (ZListCtrl)

protected:
    BYTE m_bDragDrop;   // True if user wants drag-drop.
    HIMAGELIST m_hDragImageList;    // Default dragged imagelist.
    BYTE m_bDragging;   // True if user is currently dragging.
    int m_nDragItem;    // If user is dragging, this is the item no.
};

#endif // __ZLISTCTRL__

