/*
**  DETAILS.C
**	Displays the details dialog for a specific entry in the system
**	palette.  The following information is displayed:
**	    1) system palette index
**	    2) real RGB value
**	    3) logical palette index(s)
**	    4) requested RGB value(s)
*/

#include "palspy.h"

extern HINSTANCE	ghInstance;

#define IDD_DETAILS		1000
#define IDC_SYSTEM_INDEX	101
#define IDC_SYSTEM_RGB		102
#define IDC_LOGICAL_LIST	103
#define IDC_LOGICAL_COUNT	104

BOOL CALLBACK _export DetailsDlgProc(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);
static BOOL Cls_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam);
static void Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);


/*
**  ShowDetails()
**	Displays the details dialog for a specific palette hardware palette
**	entry.
*/
void ShowDetails(HWND hOwner, short systemIndex)
{
	DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_DETAILS), hOwner,
		DetailsDlgProc, MAKELONG(systemIndex, 0));
}


/*
**  DetailsDlgProc()
**	Callback for details dialog.
*/
#pragma argsused
BOOL CALLBACK _export DetailsDlgProc(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch(iMessage) {
		case WM_INITDIALOG:
			(void)HANDLE_WM_INITDIALOG(hDlg, wParam, lParam, Cls_OnInitDialog);
			return(TRUE);
		case WM_COMMAND:
			(void)HANDLE_WM_COMMAND(hDlg, wParam, lParam, Cls_OnCommand);
			break;
	}
	return(FALSE);
}


/*
**  Cls_OnInitDialog()
**	Called for WM_INITDIALOG.
*/
#pragma argsused
static BOOL Cls_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	PALETTEENTRY	pe;
	HDC		hdc;
	HPALETTE	hPalette;
	HWND		hListWnd;
	int		tabPos;
	char		formatString[60];
	BYTE		transTable[MAX_PALETTE_ENTRIES];
	short		systemIndex;
	short		numEntries;
	short		numThatMap;
	short		i;

	systemIndex = LOWORD(lParam);
	wsprintf(formatString, "%d", systemIndex);
	SetDlgItemText(hwnd, IDC_SYSTEM_INDEX, formatString);

	formatString[0] = '\0';
	hdc = GetDC(NULL);
	if (hdc != NULL) {
		if (GetSystemPaletteEntries(hdc, systemIndex, 1, &pe) == 1)
			wsprintf(formatString, "%d,%d,%d",
				pe.peRed, pe.peGreen, pe.peBlue);
		ReleaseDC(NULL, hdc);
	}
	SetDlgItemText(hwnd, IDC_SYSTEM_RGB, formatString);

	hPalette = GetCurrentPalette();
	numEntries = GetPaletteMapping(hPalette, transTable);
	numThatMap = 0;

	hListWnd = GetDlgItem(hwnd, IDC_LOGICAL_LIST);
	tabPos = 30;
	(void)ListBox_SetTabStops(hListWnd, 1, &tabPos);
	for (i=0; i < numEntries; ++i) {
		if (transTable[i] == systemIndex) {
			GetPaletteEntries(hPalette, i, 1, &pe);
			wsprintf(formatString, "%d:\tRGB(%d,%d,%d)",
				i, pe.peRed, pe.peGreen, pe.peBlue);
			(void)ListBox_InsertString(hListWnd, -1, formatString);
			++numThatMap;
		}
	}

	wsprintf(formatString, "%d logical entr%s listed", (int) numThatMap,
		(numThatMap == 1) ? (LPCSTR) "y" : (LPCSTR) "ies");
	SetDlgItemText(hwnd, IDC_LOGICAL_COUNT, formatString);

	return(TRUE);
}


/*
**  Cls_OnCommand()
**	Called for WM_COMMAND.
*/
#pragma argsused
static void Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	if ((id == IDOK || id == IDCANCEL) && codeNotify == BN_CLICKED)
		EndDialog(hwnd, 0);
}

