#include <windows.h>
#include <lm.h>
#include "rpctips.h"

/*
   LookupDomain (and the platform-specific versions it calls)
   returns TRUE for success, FALSE if a system error occurs.
   If successful, the current domain or workgroup will be
   returned in pDomain, else "Unknown" will be returned.
   For this sample code, only minimal error-checking is done,
   and in particular the buffer size of pDomain is assumed
   to be large enough to hold the returned data.
*/

BOOL LookupDomain(char *pDomain, DWORD buflen)
{
  DWORD   ret;

  // Are we running on WinNT or Win95?
  // Win95 does not support the necessary API to directly retrieve
  // the workgroup or domain, so we need to get it from the registry

  OSVERSIONINFO   osvi;
  memset(&osvi, 0, sizeof(OSVERSIONINFO));
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  if (!GetVersionEx(&osvi))
    return(FALSE);

  ret = FALSE;

  if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
    ret = LookupDomainViaNetwork(pDomain, buflen);
  else if (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
    ret = LookupDomainViaRegistry(pDomain, buflen);

  if (ret)
    return(TRUE);
  else
  {
    strcpy(pDomain, "Unknown");
    return(FALSE);
  }
}


BOOL LookupDomainViaNetwork(char * pDomain, DWORD buflen)
{
  // Define a type for PFN_NETAPI, which will be used to declare
  // a function pointer for NetWkstaGetInfo
  typedef NET_API_STATUS
    (*PFN_NETAPI)(LPTSTR serverName, DWORD level, LPBYTE *bufPtr);

  HINSTANCE         hNetApi = NULL;
  PFN_NETAPI        pfnNetWkstaGetInfo = NULL;
  PWKSTA_INFO_100   pInfo;
  DWORD             ret;

  // Load netapi32.dll, which contains NetWkstaGetInfo
  if ((hNetApi = LoadLibrary("netapi32.dll")) == NULL)
    return(FALSE);

  // Get a pointer to the NetWkstaGetInfo API, and call it
  if ( ((pfnNetWkstaGetInfo = (PFN_NETAPI)
         GetProcAddress(hNetApi, "NetWkstaGetInfo")) == NULL) ||

        ((ret = (*pfnNetWkstaGetInfo)
          (NULL, 100, (UCHAR **) &pInfo)) != 0) )
  {
    FreeLibrary(hNetApi);
    return(FALSE);
  }

  // Translate Unicode to ANSI if necessary
  // This assumes the domain or workgroup name is at
  // least 4 bytes long, which may not be safe in all cases

  ret = IsTextUnicode(pInfo->wki100_langroup, 4, NULL);
  
  if (ret)        // Convert Unicode to ANSI
    WideCharToMultiByte(CP_ACP,
      0,
      (const USHORT *) pInfo->wki100_langroup,
      -1,
      pDomain,
      buflen,
      NULL,
      NULL);
  else            // non Unicode - just copy ANSI text
    strcpy(pDomain, pInfo->wki100_langroup);

  FreeLibrary(hNetApi);
  return(TRUE);
}


BOOL LookupDomainViaRegistry(char * pDomain, DWORD buflen)
{
  DWORD	  dwValueSize = buflen;
  DWORD	  dwValueType;
  HKEY	  hKey;
  DWORD   ret;

  char * keyName =
    "System\\CurrentControlSet\\Services\\VxD\\VNETSUP";
  char * valueName = "Workgroup";

  if ((ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0,
    KEY_QUERY_VALUE, &hKey)) != ERROR_SUCCESS)
      return(FALSE);

  ret = RegQueryValueEx(hKey, valueName, NULL, &dwValueType,
    pDomain, &dwValueSize);

  RegCloseKey(hKey);

  if (ret == ERROR_SUCCESS)
    return(TRUE);
  else
    return(FALSE);
}
