#define  STRICT
#include <windows.h>
#include <string.h>
#include <commctrl.h>

/*****************************************
  Toolbar Demo Program by Eric Heimburg
  Abbreviations used in the code:
  TB = toolbar    CB = combo box
*****************************************/

/* global variables */
HINSTANCE hInstance;
HWND hMainWnd, hTB, hCB;
LONG OldProc = 0;

/* toolbar definitions */
#define CBWIDTH         150
#define CBEXTRAHEIGHT   200
#define ID_TB           10
#define ID_CB           110
#define IDX_CB          1     /*index of CB in Btns array*/
#define NUMBTNS         (sizeof(Btns) / sizeof(TBBUTTON))

/* The master TBBUTTON list */
TBBUTTON Btns[] =
{
  { STD_COPY, 100, TBSTATE_ENABLED, TBSTYLE_CHECK,"",0,0},
  { CBWIDTH, ID_CB, TBSTATE_ENABLED, TBSTYLE_SEP,"",0,1},
  { STD_CUT, 101, TBSTATE_ENABLED, TBSTYLE_BUTTON,"",0,2},
  { STD_PASTE, 102, TBSTATE_ENABLED, TBSTYLE_BUTTON,"",0,3},
  { STD_DELETE, 103, TBSTATE_ENABLED,TBSTYLE_BUTTON,"",0,4},
};

/* Labels of each entry in the master TBBUTTON list */
LPCSTR BtnLabels[NUMBTNS] =
{ "Copy", "ComboBox", "Cut", "Paste", "Delete" };

/* function to position combobox over toolbar */
void FixCBPos(void)
{
  RECT r;
  int n = SendMessage(hTB, TB_COMMANDTOINDEX, ID_CB, 0);
  if (n == -1)
    ShowWindow(hCB, SW_HIDE);
  else {
    SendMessage(hTB, TB_GETITEMRECT, n, (LPARAM)&r);
    SetWindowPos(hCB, NULL, r.left, r.top,
        r.right - r.left, r.bottom - r.top + CBEXTRAHEIGHT,
        SWP_NOZORDER | SWP_SHOWWINDOW);
  }
}

/* WNDPROC to subclass "Customize Toolbar" Dialog */
LRESULT CALLBACK CustomizeProc(HWND hwnd, UINT Msg,
    WPARAM WParam, LPARAM LParam)
{
  DWORD result;
  DRAWITEMSTRUCT *pDraw;

  if (Msg == WM_DRAWITEM) {
    pDraw = (DRAWITEMSTRUCT*)LParam;
    if (pDraw->itemData == MAKELPARAM(0xFFFF, IDX_CB)) {
      SendMessage(pDraw->hwndItem, LB_SETITEMDATA,
         pDraw->itemID, MAKELPARAM(STD_PROPERTIES, IDX_CB));
      InvalidateRect(pDraw->hwndItem, NULL, FALSE);
    }
  }
  result = CallWindowProc((WNDPROC)OldProc, hwnd,
      Msg, WParam, LParam);
  if (Msg == WM_DESTROY) {
    SetWindowLong(hwnd, GWL_WNDPROC, OldProc);
    OldProc = NULL;
  }
  return result;
}

/* function to process notifications coming from hTB. */
LRESULT DoToolbarNotify(NMHDR* pHdr)
{
  static char buf[128];
  HWND htemp;
  UINT dwtemp;
  TBNOTIFY *pTBHdr = (TBNOTIFY*)pHdr;

  if ((pHdr->hwndFrom==hTB) || (pHdr->code==TTN_NEEDTEXT)) {
    switch (pHdr->code) {
      case TBN_QUERYDELETE:
        if (OldProc == NULL) {
          htemp = FindWindow("#32770", "Customize Toolbar");
          if (htemp)
            OldProc = SetWindowLong(htemp, GWL_WNDPROC,
              (LONG)CustomizeProc);
        }
        return TRUE;
      case TBN_QUERYINSERT:
        return TRUE;
      case TBN_GETBUTTONINFO:
        if (pTBHdr->iItem < NUMBTNS) {
          pTBHdr->tbButton = Btns[pTBHdr->iItem];
          dwtemp = SendMessage(pHdr->hwndFrom, TB_GETSTATE,
            Btns[pTBHdr->iItem].idCommand, 0);
          if (dwtemp != -1)
            pTBHdr->tbButton.fsState = (BYTE)dwtemp;
          if (pTBHdr->pszText == NULL) {
            pTBHdr->pszText = buf;
            pTBHdr->cchText = sizeof(buf);
          }
          strcpy(pTBHdr->pszText, BtnLabels[pTBHdr->iItem]);
          return TRUE;
        }
        else
          return FALSE;
      case TBN_TOOLBARCHANGE:
        FixCBPos();
        break;
    }
  }
  return 0;
}

/* main window procedure */
LRESULT CALLBACK _export MainWndProc( HWND hwnd, UINT Msg,
    WPARAM WParam, LPARAM LParam )
{
  switch (Msg)
  {
    case WM_CREATE:
      hTB = CreateToolbarEx(hwnd,
        WS_VISIBLE | WS_CHILD | CCS_ADJUSTABLE, ID_TB, 0,
        HINST_COMMCTRL, IDB_STD_LARGE_COLOR, Btns, NUMBTNS,
        0, 0, 0, 0, sizeof(TBBUTTON));
      hCB = CreateWindow("COMBOBOX", "ComboBox",
        WS_CHILD | WS_VSCROLL | WS_BORDER | CBS_DROPDOWN,
        0, 0, 0, 0, hwnd, (HMENU)ID_CB, hInstance, NULL);
      SendMessage(hCB, CB_DIR, 0, (LPARAM)"*.*");
      SetParent(hCB, hTB);
      FixCBPos();
      break;
    case WM_DESTROY:
      PostQuitMessage( 0 );
      break;
    case WM_SIZE:
      SendMessage(hTB, WM_SIZE, 0, 0);
      break;
    case WM_NOTIFY:
      return DoToolbarNotify((NMHDR*)LParam);
    default:
      return DefWindowProc( hwnd, Msg, WParam, LParam );
  }
  return 0;
}

#pragma argsused
int PASCAL WinMain( HINSTANCE hInst, HINSTANCE hPrevInst,
    LPSTR CmdLine, int nCmdShow )
{
  WNDCLASS wndclass;
  MSG msg;

  hInstance = hInst;
  if (!hPrevInst) {
    wndclass.style         = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc   = MainWndProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 0;
    wndclass.hInstance     = hInstance;
    wndclass.hIcon         = LoadIcon( NULL, IDI_WINLOGO );
    wndclass.hCursor       = LoadCursor( NULL, IDC_ARROW );
    wndclass.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = "TBDEMO";
    if ( !RegisterClass( &wndclass ) )
      return 0;
  }
  hMainWnd = CreateWindowEx( 0, "TBDEMO", "Toolbar Demo",
      WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, CW_USEDEFAULT,
      0, NULL, NULL, hInstance, NULL );
  ShowWindow(hMainWnd, nCmdShow);
  UpdateWindow(hMainWnd);
  while( GetMessage( &msg, NULL, 0, 0 ) ) {
    TranslateMessage( &msg );
    DispatchMessage( &msg );
  }
  return msg.wParam;
}

