//------------------------------------------------------------------
//  Notify.c - Int 41h debug notification hooking device
//------------------------------------------------------------------
#include <string.h>
#include <stdarg.h>
#undef   WANTVXDWRAPS
#include <basedef.h>
#include <vmm.h>
#include <vmmreg.h>
#include <vwin32.h>
#include "dbwin.h"

//------------------------------------------------------------------
#pragma VxD_LOCKED_CODE_SEG

typedef struct Client_Word_Reg_Struc * PCWRS;
typedef DIOCPARAMETERS *LPDIOC;

DWORD  Notify_W32_DeviceIOControl(DWORD, DWORD, DWORD, LPDIOC);
DWORD  Notify_W32_RegisterProc(DWORD, DWORD, LPDIOC);
DWORD  Notify_W32_DeallocList(DWORD, DWORD, LPDIOC);

//------------------------------------------------------------------
#pragma VxD_LOCKED_DATA_SEG

static  DWORD (*Notify_W32_Proc[])(DWORD, DWORD, LPDIOC) = {
        Notify_W32_RegisterProc,
        Notify_W32_DeallocList };
#define MAX_W32_API     (sizeof(Notify_W32_Proc)/sizeof(DWORD))

DWORD  _hList           = 0;    // List handle
DWORD  _hRing3Thread    = 0;    // User app thread for callback
DWORD  _pfnRing3Proc    = 0;    // User app notify callback
DWORD  _Int41CallBack   = 0;    // Int 41h PM callback

//------------------------------------------------------------------
DWORD  Notify_Device_Init(void);
void   Int41_PM_Hook(void);
void   InitEvent(void);
void  *List_Allocate(DWORD hList);
void   List_Deallocate(DWORD hList, void *pNode);
void  *_fstrcpy(void *pDest, WORD wSel, DWORD dwOffset);

//------------------------------------------------------------------
//  C O N T R O L   P R O C 
//------------------------------------------------------------------

void __declspec(naked) Notify_Control(void)
{
    DWORD dwEBX, dwECX, dwEDX, dwESI;
    DWORD dwControlCode;

    // Set up stack
    _asm    mov     ebp, esp
    _asm    sub     esp, __LOCAL_SIZE
    _asm    mov     [dwControlCode], eax

    switch (dwControlCode) {

    case DEVICE_INIT:
        Notify_Device_Init();
        break;

    case W32_DEVICEIOCONTROL:
        _asm mov [dwEBX], ebx
        _asm mov [dwECX], ecx
        _asm mov [dwEDX], edx
        _asm mov [dwESI], esi
        Notify_W32_DeviceIOControl(dwECX, dwEBX,
                                   dwEDX, (LPDIOC)dwESI);
        break;

    default: 
        break;
    }

    _asm    mov     esp, ebp
    _asm    clc
    _asm    ret
}

//------------------------------------------------------------------
 
DWORD Notify_Device_Init()
{
    // Set up a PM callback and create a list.

    _asm mov    esi, offset Int41_PM_Hook
    _asm mov    edx, 0                  // Reference data
    VMMCall(Allocate_PM_Call_Back)      // Allocate PM callback
    _asm mov    _Int41CallBack, eax     // Seg:off for callback

    _asm mov    eax, _Int41CallBack     //
    _asm movzx  edx, ax                 // EDX = offset
    _asm mov    ecx, eax                //
    _asm shr    ecx, 10h                // CX = segment
    _asm mov    eax, 41h                //
    VMMCall(Set_PM_Int_Vector);         // Only for current VM

    _asm mov    eax, LF_ASYNC           // Usable at int time
    _asm mov    ecx, SIZE NOTIFYINFO    //
    VMMCall(List_Create);               // Create list
    _asm mov    _hList, esi             //   and save the handle

    return VXD_SUCCESS;
}

//------------------------------------------------------------------

void __declspec(naked) Int41_PM_Hook(void)
{
    // ebx: current VM handle
    // edi: current thread handle
    // edx: reference data
    // ebp: points to Client_Reg_Struc
    static LPNOTIFYINFO pInfo = NULL;
    static PCWRS        pCWRS = NULL;

    VMMCall(Simulate_Iret);

    _asm mov    pCWRS, ebp

    if (pCWRS->Client_AX == DS_DebLoaded) {
        // Reply to de debugger loaded inquiry
        pCWRS->Client_AX = DS_DebPresent;
    }
    else if (_pfnRing3Proc != 0) {

        switch (pCWRS->Client_AX) {

        case DS_Out_Str: // DS:ESI
            pInfo = List_Allocate(_hList);
            pInfo->wFunction = pCWRS->Client_AX;
            _fstrcpy(pInfo->szInfo, ((PCRS)pCWRS)->Client_DS,
                                    ((PCRS)pCWRS)->Client_ESI);
            _asm xor    eax, eax
            _asm xor    ebx, ebx
            _asm mov    ecx, PEF_ALWAYS_SCHED | PEF_WAIT_NOT_CRIT
            _asm mov    edx, pInfo
            _asm mov    esi, offset InitEvent
            VMMCall(Call_Restricted_Event);
            break;

        default:
            break;
        }

    }

    _asm ret
}

//------------------------------------------------------------------

void InitEvent()
{
    LPNOTIFYINFO pInfo;

    _asm mov    pInfo, edx
    _asm push   _hRing3Thread
    _asm push   pInfo
    _asm push   _pfnRing3Proc
    VxDCall(_VWIN32_QueueUserApc);
    _asm add    esp, 0x1C

    return ;
}

//------------------------------------------------------------------

void *List_Allocate(DWORD hList)
{
    DWORD pNode;

    _asm mov    esi, _hList
    _asm pushfd
    _asm cli
    VMMCall(List_Allocate);
    _asm popfd
    _asm mov    pNode, eax

    return (void *)pNode;
}

//------------------------------------------------------------------

void List_Deallocate(DWORD hList, void *pNode)
{
    _asm mov    esi, hList
    _asm mov    eax, dword ptr pNode
    _asm pushfd
    _asm cli
    VMMCall(List_Deallocate);
    _asm popfd

    return ;
}

//------------------------------------------------------------------
//  W32 Services
//------------------------------------------------------------------

DWORD  Notify_W32_DeviceIOControl(DWORD dwService,
                        DWORD dwDDB, DWORD hDev, LPDIOC lpParms)
{
    DWORD dwRc = 0;

    if (dwService == DIOC_OPEN) {
        dwRc = 0; // Return OK
    }
    else if (dwService == DIOC_CLOSEHANDLE) {
        dwRc = 0; // Return OK
    }
    else if (dwService > MAX_W32_API) {
        // A positive value signals an error to DeviceIOControl
        dwRc = 1; // ERROR_INVALID_FUNCTION
    }
    else {
        // CALL requested service. Do not use [dwService - 1], the
        // compiler (MSC 9.10) will generate invalid code for this.
        dwService--;
        dwRc = (Notify_W32_Proc[dwService])(dwDDB, hDev, lpParms);
    }

    return dwRc;
}

//------------------------------------------------------------------

DWORD Notify_W32_RegisterProc(DWORD dwDDB,
                                DWORD hDev, LPDIOC lpParms)
{
    if ((lpParms->lpvInBuffer == 0L) || (lpParms->cbInBuffer < 4))
        return ERROR_INVALID_PARAMETER;

    VMMCall(Get_Cur_Thread_Handle);
    _asm mov _hRing3Thread, edi

    _pfnRing3Proc = *(DWORD *)lpParms->lpvInBuffer;

    return 0;
}

//------------------------------------------------------------------

DWORD  Notify_W32_DeallocList(DWORD dwDDB,
                        DWORD hDev, LPDIOC lpParms)
{
    LPNOTIFYINFO pInfo;

    pInfo = *(LPNOTIFYINFO *)lpParms->lpvInBuffer;
    List_Deallocate(_hList, pInfo);

    return 0;
}

//------------------------------------------------------------------

void *_fstrcpy(void *pDest, WORD wSel, DWORD dwOffset)
{
    // Far strcpy for 32-bit environs.
    _asm    mov   es,  wSel     ; 
    _asm    mov   edi, dwOffset ; es:[esi] points to src
    _asm    mov   ecx, 0xFFFF   ; Strlen < 64K
    _asm    xor   eax, eax
    _asm    repnz scasb         ; Scan string es:edi for byte ax
    _asm    not   cx            ; strlen

    _asm    mov   ax,  ds       ; Set es to ds
    _asm    mov   es,  ax
    _asm    mov   edi, pDest    ; es:[edi] points to dest

    _asm    push  ds            ; Save ds
    _asm    mov   ds,  wSel     ; 
    _asm    mov   esi, dwOffset ; ds:[esi] points to src

    _asm    mov   eax, ecx
    _asm    and   eax, 03h      ; 
    _asm    shr   ecx, 2        ; Bytes to dwords
    _asm    rep   movsd         ; Copy ds:[esi] to es:[edi]
    _asm    mov   ecx, eax      ; Last 0-3 bytes
    _asm    rep   movsb         ; Copy ds:[esi] to es:[edi]
    _asm    pop   ds            ; Restore ds

    return pDest;
}

