#include <windows.h>
#include <malloc.h>
#include <stdio.h>
#include "fiber.h"

void main(void)
{
    PVOID pScheduler;
    TURN_SCHEDULER_DATA SchedData;
    CAR_FIBER_DATA      Data[NUM_ITEMS];
    int Item = 0, i = 0;

    //
    // create the "turn scheduler" fiber
    //
    pScheduler = CreateFiber(0, TurnSchedulerFiber, &SchedData);

    //
    // create a fiber to represent each element in the simulation
    //
    for (i = 0; i < NUM_CARS_PER_TYPE; i++) {
        Data[Item].pTurnScheduler = pScheduler;
        Data[Item].CarID = i;
        SchedData.pItemFiber[Item] =
                CreateFiber(4096, SportsCarFiber, &Data[Item]);
        Item++;

        Data[Item].pTurnScheduler = pScheduler;
        Data[Item].CarID = i;
        SchedData.pItemFiber[Item] =
                CreateFiber(4096, EconomyCarFiber, &Data[Item]);
        Item++;
    }

    //
    // convert main thread to a fiber so can schedule the "scheduler"
    //
    SchedData.pMainFiber = ConvertThreadToFiber(NULL);
    SwitchToFiber(pScheduler);

    //
    // The turn scheduler fiber has returned, cleanup fibers
    //
    for (i = 0; i < NUM_ITEMS; i++) {
        DeleteFiber(SchedData.pItemFiber[i]);
    }
    DeleteFiber(pScheduler);
}

VOID WINAPI TurnSchedulerFiber(LPTURN_SCHEDULER_DATA pSchedData)
{
    int i, Turn;

    //
    // simple turn-based-round-robin scheduler
    //
    for (Turn = 0; Turn < NUM_TURNS; Turn++) {
        for (i = 0; i < NUM_ITEMS; i++) {
            SwitchToFiber(pSchedData->pItemFiber[i]);
        }
    }
    SwitchToFiber(pSchedData->pMainFiber);
}

VOID WINAPI SportsCarFiber(LPCAR_FIBER_DATA pData)
{
    pData->Cost = pData->Distance = 0;

    while (TRUE) {
        pData->Distance += SPORTS_SPEED;
        pData->Cost += (int)(SPORTS_GPH * FUEL_COST);

        printf("SportsCar-%d: %d miles, %d cents\n",
               pData->CarID, pData->Distance, pData->Cost);

        SwitchToFiber(pData->pTurnScheduler);
    }
}

VOID WINAPI EconomyCarFiber(LPCAR_FIBER_DATA pData)
{
    pData->Cost = pData->Distance = 0;

    while (TRUE) {
        pData->Distance += ECONOMY_SPEED;
        pData->Cost += (int)(ECONOMY_GPH * FUEL_COST);

        printf("EconomyCar-%d: %d miles, %d cents\n",
               pData->CarID, pData->Distance, pData->Cost);

        SwitchToFiber(pData->pTurnScheduler);
    }
}

