/*
    File: ZDialog.cpp
    Author: V.Ramachandran
    Date: 18 September, 1996
*/

#include "stdafx.h"
#include "zdialog.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define WM_USER_FOCUS_CHANGE    (WM_USER+25)    // 25 arbitrary

IMPLEMENT_DYNAMIC (ZDialog, CDialog)

#define new DEBUG_NEW

int ZDialog::m_nNofInstances = 0;
HHOOK ZDialog::m_hOldHook = NULL;
ZDialog * ZDialog::m_pCurrentDlg = NULL;

BEGIN_MESSAGE_MAP(ZDialog, CDialog)
    //{{AFX_MSG_MAP(ZDialog)
    ON_WM_DESTROY()
    ON_MESSAGE(WM_USER_FOCUS_CHANGE, OnFocusChangeRequest)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

ZDialog::ZDialog () : CDialog (), m_bByPass (FALSE)
{
}

ZDialog::ZDialog (LPCSTR lpszTemplateName, CWnd *pParent)
    : CDialog (lpszTemplateName, pParent), m_bByPass (FALSE)
{
}

ZDialog::ZDialog (UINT nID, CWnd *pParent)
    : CDialog (nID, pParent), m_bByPass (FALSE)
{
}

/****************************************************************/
// OnInitDialog
// If this is the first instance, set the windows hook.
/****************************************************************/

BOOL ZDialog::OnInitDialog ()
{
    BOOL bRet = CDialog::OnInitDialog ();
    if (m_nNofInstances == 0) {
        if (!SetCBTHook ()) {
            AfxMessageBox ("CBT Hook could not be installed.");
            return bRet;
        }
    }
    m_nNofInstances ++;
    return bRet;
}

/****************************************************************/
// OnDestroy - remove the hook if this is the last instance.
/****************************************************************/

void ZDialog::OnDestroy ()
{
    if ((--m_nNofInstances == 0) && (m_hOldHook))
        UnhookWindowsHookEx (m_hOldHook);   
    CDialog::OnDestroy ();
}

/****************************************************************/
// CBTProc - the hook function.
// See CBTProc SDK documentation for parameters/return value.
// Handles focus change notifications, by first checking if 
// validation is required. If so, posts a user message and 
// returns TRUE to disallow focus shift. In the user message 
// handler, do the validation and explicitly SetFocus.
/****************************************************************/

#ifdef WIN32               // _export not required for Win32
#define _export 
#endif

LRESULT CALLBACK _export ZDialog::CBTProc (int nCode, 
                            WPARAM wParam, LPARAM lParam)
{
    HWND hWndLost, hWndGain;

// Mechanism to opt out of hook processing.
    if ((m_pCurrentDlg) && (m_pCurrentDlg->m_bByPass))
        return CallNextHookEx (m_hOldHook, nCode, wParam, lParam);

// If the to-be-activated window is a ZDialog, store it in 
// m_pCurrentDlg
    if (nCode == HCBT_ACTIVATE) {
        CWnd *pDlg = CWnd::FromHandlePermanent ((HWND)wParam);
        if ((pDlg) && (pDlg->IsKindOf (RUNTIME_CLASS (ZDialog))))
            m_pCurrentDlg = (ZDialog *)pDlg;
        else
            m_pCurrentDlg = NULL;
    }
   else if (nCode == HCBT_SETFOCUS) {
        hWndGain = (HWND)wParam;
#ifdef WIN32
        hWndLost = (HWND)lParam;
#else		// Win16
        hWndLost = (HWND)LOWORD (lParam);
#endif 

// Check if the dialog requests validation. If so, post a 
// user defined message and disallow focus shift.
        if ((m_pCurrentDlg) && (m_pCurrentDlg->IsValidationReqd 
                            (hWndLost, hWndGain))) {
            m_pCurrentDlg->PostMessage (WM_USER_FOCUS_CHANGE, 
                            (WPARAM)hWndLost, (LPARAM)hWndGain);
            return TRUE;    // Disallow focus shift         
        }
    }                       // If HCBT_SETFOCUS
    return CallNextHookEx (m_hOldHook, nCode, wParam, lParam);
}

/****************************************************************/
// IsValidationReqd - ZDialog virtual function
// Parameters: Handle of window losing focus, and window
// gaining focus.
// Return TRUE if validation is requried for the window losing 
// focus. This version returns TRUE if both the child windows 
// belong to the dialog. In derived versions, remember to
// call base class first.
/****************************************************************/

BOOL ZDialog::IsValidationReqd (HWND hWndLost, HWND hWndGain)
{
    if ((::GetParent (hWndGain) == m_hWnd) &&
            (::GetParent (hWndLost) == m_hWnd))
        return TRUE;
    else
        return FALSE;
}

/****************************************************************/
// DoValidation - ZDialog virtual function
// Parameter: ID of child window losing focus.
// Return: TRUE if validation succeeds, FALSE otherwise.
// This version just returns TRUE.
/****************************************************************/

BOOL ZDialog::DoValidation (UINT nLostID)
{
    return TRUE;
}

/****************************************************************/
// NotifyError - ZDialog virtual function
// Called whenever field validation fails.
// Parameter: ID of child window losing focus.
/****************************************************************/

void ZDialog::NotifyError (UINT nLostID)
{
}

/****************************************************************/
// OnFocusChangeRequest - WM_USER_FOCUS_CHANGE handler
// Parameters: hWndLost = wParam, hWndGain = lParam 
// Calls out to DoValidation virtual function. If validation
// succeeds, set focus to window gaining focus explicitly. 
// Else call NotifyError.
/****************************************************************/

LRESULT ZDialog::OnFocusChangeRequest (WPARAM wParam, 
                            LPARAM lParam)
{
    HWND hWndLost = (HWND)wParam;
    HWND hWndGain = (HWND)lParam;

#ifndef WIN32
    UINT nLostID = ::GetWindowWord (hWndLost, GWW_ID);
#else
    UINT nLostID = ::GetWindowLong (hWndLost, GWL_ID);
#endif

    if (DoValidation (nLostID)) {       // Call out to virtual
        m_bByPass = TRUE;           // Bypass hook code
        ::SetFocus (hWndGain);
        m_bByPass = FALSE;      // Reset bypass flag
    }
    else
        NotifyError (nLostID);
    return 0;
}

#include "windowsx.h"   // For GetInstanceModule!

/****************************************************************/
// SetCBTHook - sets the CBT hook. Call only once. Otherwise 
// latest hook is activated.
// Returns: TRUE if hook is set, FALSE otherwise.
/****************************************************************/

BOOL ZDialog::SetCBTHook ()
{
    ASSERT (!m_nNofInstances);
#ifdef WIN32
    CWinThread *pThread = AfxGetThread ();
    m_hOldHook = SetWindowsHookEx (WH_CBT, 
                    (HOOKPROC)ZDialog::CBTProc, 
                    GetInstanceModule (AfxGetInstanceHandle ()), 
                    (DWORD)pThread->m_nThreadID);
#else
    m_hOldHook = SetWindowsHookEx (WH_CBT, 
                    (HOOKPROC)ZDialog::CBTProc, 
                    GetInstanceModule (AfxGetInstanceHandle ()), 
                    GetCurrentTask ());
#endif      
    return (m_hOldHook) ? TRUE : FALSE;
}

