#ifndef Win32Exception_defined
#define Win32Exception_defined

class Win32Exception;
    class AccessViolation;
    class Breakpoint;
    class DatatypeMisalignment;
    class SingleStep;
    class ArrayBoundsExceeded;
    class MathException;
        class FltException;
            class FltDenormalOperand;
            class FltDivideByZero;
            class FltInexactResult;
            class FltInvalidOperation;
            class FltOverflow;
            class FltStackCheck;
            class FltUnderflow;
        class IntException;
            class IntDivideByZero;
            class IntOverflow;
    class PrivInstruction;
    class NoncontinuableException;
    class Win32UserException;

struct _EXCEPTION_POINTERS;

class Win32Exception
    {
    public:
        virtual ~Win32Exception();

        enum
            {
            tr_AccessViolation = 1 << 0,
            tr_Breakpoint = 1 << 1,
            tr_DatatypeMisalignment = 1 << 2,
            tr_SingleStep = 1 << 3,
            tr_ArrayBoundsExceeded = 1 << 4,
            tr_FltDenormalOperand = 1 << 5,
            tr_FltDivideByZero = 1 << 6,
            tr_FltInexactResult = 1 << 7,
            tr_FltInvalidOperation = 1 << 8,
            tr_FltOverflow = 1 << 9,
            tr_FltStackCheck = 1 << 10,
            tr_FltUnderflow = 1 << 11,
            tr_FltException = 
                tr_FltDenormalOperand |
                tr_FltDivideByZero |
                tr_FltInexactResult |
                tr_FltInvalidOperation |
                tr_FltOverflow |
                tr_FltStackCheck |
                tr_FltUnderflow,
            tr_IntDivideByZero = 1 << 12,
            tr_IntOverflow = 1 << 13,
            tr_IntException = 
                tr_IntDivideByZero | 
                tr_IntOverflow,
            tr_MathException = tr_FltException | tr_IntException,
            tr_PrivInstruction = 1 << 14,
            tr_NoncontinuableException = 1 << 15,
            tr_UserException = 1 << 16,
            tr_All = 0xffffffff
            };

        static void SetTranslator();
        static unsigned Translate(unsigned exceptions, unsigned mask = tr_All);
        static void Translator(unsigned code, _EXCEPTION_POINTERS* p);

    private:
        static unsigned translated;
        static int installed;
    };

class AccessViolation         : public Win32Exception { };
class Breakpoint              : public Win32Exception { };
class DatatypeMisalignment    : public Win32Exception { };
class SingleStep              : public Win32Exception { };
class ArrayBoundsExceeded     : public Win32Exception { };
class MathException           : public Win32Exception { };

class FltException : public MathException
    {
    public: 
        FltException();
    };

class FltDenormalOperand      : public FltException { };
class FltDivideByZero         : public FltException { };
class FltInexactResult        : public FltException { };
class FltInvalidOperation     : public FltException { };
class FltOverflow             : public FltException { };
class FltStackCheck           : public FltException { };
class FltUnderflow            : public FltException { };
class IntException            : public MathException { };
class IntDivideByZero         : public IntException { };
class IntOverflow             : public IntException { };
class PrivInstruction         : public Win32Exception { };
class NoncontinuableException : public Win32Exception { };

class Win32UserException : public Win32Exception
    {
    public:
        Win32UserException(unsigned code)
            : code(code) { }
        unsigned GetExceptionCode() const
            { return code; }
    private:
         unsigned code;
    };

#endif