
#include "windows.h"
HANDLE hInst;

int PASCAL WinMain(HANDLE, HANDLE, LPSTR, int);
BOOL InitApplication(HANDLE);
BOOL InitInstance(HANDLE, int);

LRESULT CALLBACK MainWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ThinCaptWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK BorderCaptWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK TriangularCaptWndProc(HWND, UINT, WPARAM, LPARAM);

/**************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes
             message loop
***************************************************************/
int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR lpCmdLine;
int nCmdShow;
{
   MSG msg;

   if (!hPrevInstance)
      if (!InitApplication(hInstance))
      {
         MessageBox(NULL, "InitApp Failed", NULL, MB_OK);
         return (FALSE);
      }

   if (!InitInstance(hInstance, nCmdShow))
   {
      MessageBox(NULL, "InitInst Failed", NULL, MB_OK);
      return (FALSE);
   }

   while (GetMessage(&msg, NULL, NULL, NULL))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return (msg.wParam);
}


/**************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers
             window classes
***************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;
{
   WNDCLASS  wc;

   wc.style = CS_VREDRAW | CS_HREDRAW | CS_DBLCLKS;
   wc.lpfnWndProc = MainWndProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInstance;
   wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = COLOR_WINDOW+1;
   wc.lpszMenuName =  NULL;
   wc.lpszClassName = "MainClass";
   if(!RegisterClass(&wc))
      return FALSE;

   wc.lpfnWndProc = ThinCaptWndProc;
   wc.lpszClassName = "ThinCaptionClass";
   if(!RegisterClass(&wc))
      return FALSE;

   wc.lpfnWndProc = BorderCaptWndProc;
   wc.lpszClassName = "BorderCaptionClass";
   if(!RegisterClass(&wc))
      return FALSE;

   wc.lpfnWndProc = TriangularCaptWndProc;
   wc.lpszClassName = "TriangularCaptionClass";

   return (RegisterClass(&wc));
}


/*************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
**************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;
int             nCmdShow;
{
   HWND            hwnd;

   hInst = hInstance;

   hwnd = CreateWindow("MainClass",
              "Custom Caption Bars",
              WS_OVERLAPPED|WS_VISIBLE|WS_CLIPSIBLINGS
              |WS_BORDER|WS_THICKFRAME|WS_MINIMIZEBOX|WS_SYSMENU,
              CW_USEDEFAULT, CW_USEDEFAULT,
              CW_USEDEFAULT, CW_USEDEFAULT,
              NULL, NULL, hInstance, NULL);

   if (!hwnd)
      return (FALSE);

   ShowWindow(hwnd, nCmdShow);
   UpdateWindow(hwnd);
   return (TRUE);

}


/**************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Create Custom Caption Child Windows
***************************************************************/

LRESULT CALLBACK MainWndProc(hwnd, message, wParam, lParam)
HWND hwnd;
UINT message;
WPARAM wParam;
LPARAM lParam;
{
   switch (message)
   {
      case WM_CREATE:
         CreateWindow("ThinCaptionClass", "",
                     WS_VISIBLE|WS_POPUP|WS_BORDER,
                     10, 10, 250, 50,
                     hwnd, NULL, hInst, NULL);

         CreateWindow("BorderCaptionClass", "",
                     WS_VISIBLE|WS_POPUP|WS_BORDER,
                     10, 100, 250, 50,
                     hwnd, NULL, hInst, NULL);

         CreateWindow("TriangularCaptionClass", "",
                     WS_VISIBLE|WS_POPUP|WS_BORDER,
                     10, 200, 250, 50,
                     hwnd, NULL, hInst, NULL);
         break;

      case WM_DESTROY:
         PostQuitMessage(0);
         break;

      default:
         return (DefWindowProc(hwnd, message, wParam, lParam));
    }
    return (NULL);
}


/**************************************************************
    FUNCTION: ThinCaptWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Thin Caption Child Window Procedure
***************************************************************/
LRESULT CALLBACK ThinCaptWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
UINT message;
WPARAM wParam;
LPARAM lParam;
{
   switch(message)
   {
      case WM_SETCURSOR:
         if(LOWORD(lParam) == HTCAPTION)
         {
            SetCursor(LoadCursor(NULL, IDC_UPARROW));
            return NULL;
         }
         break;
      case WM_PAINT:
      {
         HDC hdc;
         PAINTSTRUCT ps;
         HBRUSH hbr;
         HPEN hpen;
         RECT rect;
         int nCaption;
         COLORREF crCaptionColor;

         nCaption = GetSystemMetrics(SM_CYCAPTION) / 4;
         crCaptionColor = GetSysColor(COLOR_ACTIVECAPTION);

         GetClientRect(hWnd, &rect);

         hdc = BeginPaint(hWnd, &ps);
         hbr = SelectObject(hdc, CreateSolidBrush(crCaptionColor));
         hpen = SelectObject(hdc, CreatePen(PS_SOLID, 1, crCaptionColor));

         //Draw the caption
         Rectangle(hdc, rect.left, rect.top, rect.right, rect.top + nCaption);

         DeleteObject(SelectObject(hdc, hpen));
         DeleteObject(SelectObject(hdc, hbr));

         //To display anything in client area, include code here. Don't
         //forget that the caption is now occupying our client area, and
         //it should not be overwritten.

         EndPaint(hWnd, &ps);

         break;
      }
      case WM_NCHITTEST:
      {
         RECT rect;
         int nCaption;
         POINT pt;

         nCaption = GetSystemMetrics(SM_CYCAPTION) / 4;

         GetWindowRect(hWnd, &rect);
         rect.bottom = rect.top + nCaption;

         pt.x = LOWORD(lParam);
         pt.y = HIWORD(lParam);

         //Check if the point is on our caption
         if(PtInRect(&rect, pt))
            return HTCAPTION;

         break;
      }
   }
   return (DefWindowProc(hWnd, message, wParam, lParam));
}


/**************************************************************
    FUNCTION: BorderCaptWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Border Caption Child Window Procedure
***************************************************************/
LRESULT CALLBACK BorderCaptWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
UINT message;
WPARAM wParam;
LPARAM lParam;
{
   switch(message)
   {
      case WM_SETCURSOR:
         if(LOWORD(lParam) == HTCAPTION)
         {
            SetCursor(LoadCursor(NULL, IDC_UPARROW));
            return NULL;
         }
         break;
      case WM_PAINT:
      {
         HDC hdc;
         PAINTSTRUCT ps;
         HBRUSH hbr;
         HPEN hpen;
         RECT rect;
         int nCaption;
         COLORREF crCaptionColor;

         nCaption = GetSystemMetrics(SM_CXBORDER) * 2;
         crCaptionColor = GetSysColor(COLOR_ACTIVECAPTION);

         GetClientRect(hWnd, &rect);

         hdc = BeginPaint(hWnd, &ps);
         hbr = SelectObject(hdc, GetStockObject(NULL_BRUSH));
         hpen = SelectObject(hdc, CreatePen(PS_INSIDEFRAME, nCaption, crCaptionColor));

         //Draw the caption
         Rectangle(hdc, rect.left, rect.top, rect.right, rect.bottom);

         DeleteObject(SelectObject(hdc, hpen));
         DeleteObject(SelectObject(hdc, hbr));

         //To display anything in client area, include code here. Don't
         //forget that the caption is now occupying our client area, and
         //it should not be overwritten.

         EndPaint(hWnd, &ps);
         break;
      }
      case WM_NCHITTEST:
      {
         RECT rect, rectInner;
         POINT pt;
         int nCaption;

         pt.x = LOWORD(lParam);
         pt.y = HIWORD(lParam);
         nCaption = GetSystemMetrics(SM_CXBORDER) * 2;

         GetWindowRect(hWnd, &rect);
         CopyRect(&rectInner, &rect);
         InflateRect(&rectInner, -nCaption-1, -nCaption-1);

         //Check if the point is on our caption
         if(PtInRect(&rect, pt) && !PtInRect(&rectInner, pt))
            return HTCAPTION;

         break;
      }
   }
   return (DefWindowProc(hWnd, message, wParam, lParam));
}


/**************************************************************
    FUNCTION: TriangularCaptWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Triangular Caption Child Window Procedure
***************************************************************/
LRESULT CALLBACK TriangularCaptWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
UINT message;
WPARAM wParam;
LPARAM lParam;
{
   switch(message)
   {
      case WM_SETCURSOR:
         if(LOWORD(lParam) == HTCAPTION)
         {
            SetCursor(LoadCursor(NULL, IDC_UPARROW));
            return NULL;
         }
         break;
      case WM_PAINT:
      {
         HDC hdc;
         PAINTSTRUCT ps;
         HBRUSH hbr;
         HPEN hpen;
         RECT rect;
         int nCaption;
         COLORREF crCaptionColor;
         POINT apt[3];

         nCaption = GetSystemMetrics(SM_CYCAPTION)/2;
         crCaptionColor = GetSysColor(COLOR_ACTIVECAPTION);

         GetClientRect(hWnd, &rect);

         hdc = BeginPaint(hWnd, &ps);
         hbr = SelectObject(hdc, CreateSolidBrush(crCaptionColor));
         hpen = SelectObject(hdc, CreatePen(PS_SOLID, 1, crCaptionColor));

         apt[0].x = rect.right;              apt[0].y = rect.top;
         apt[1].x = rect.right - nCaption,   apt[1].y = rect.top;
         apt[2].x = rect.right;              apt[2].y = rect.top + nCaption;

         //Draw the caption
         Polygon(hdc, apt, 3);

         DeleteObject(SelectObject(hdc, hpen));
         DeleteObject(SelectObject(hdc, hbr));
         
         //To display anything in client area, include code here. Don't
         //forget that the caption is now occupying our client area, and
         //it should not be overwritten.

         EndPaint(hWnd, &ps);
         break;
      }
      case WM_NCHITTEST:
      {
         RECT rect;
         int nCaption;
         POINT pt;

         nCaption = GetSystemMetrics(SM_CYCAPTION) / 2;

         pt.x = LOWORD(lParam);
         pt.y = HIWORD(lParam);
         ScreenToClient(hWnd, &pt);

         GetClientRect(hWnd, &rect);
         //Check if the point is on our caption
         if((pt.x - rect.right + nCaption) >= pt.y)
            return HTCAPTION;

         break;
      }
   }
   return (DefWindowProc(hWnd, message, wParam, lParam));
}

