
// __________________________________________________________
//
//                         NicAddr.c
//              Get NIC Address via WinSock V1.00
//                06-02-1997 Sven B. Schreiber
//                       sbs@orgon.com
// __________________________________________________________

#include <winsock.h>
#include <wsipx.h>

#pragma comment (linker, "/entry:\"WinStartup\"")

// =================================================================
// REVISION HISTORY
// =================================================================

/*

06-02-1997  V1.00  SBS  Original version.

*/

// =================================================================
// CONSTANTS & TYPES
// =================================================================

#define WSAMAXHOSTNAME 256      // max host name length
#define WS_VERSION     0x0101   // preferred WinSock version
#define MAX_WVSPRINTF  0x0400   // max wvsprintf buffer size

#define NIC_ADDRESS_ 6                     // size of NIC address
typedef BYTE NIC_ADDRESS [NIC_ADDRESS_];   // NIC address array

#define IP_ADDRESS_  4                     // size of IP address
typedef BYTE IP_ADDRESS  [IP_ADDRESS_];    // IP address array

// =================================================================
// GLOBAL VARIABLES
// =================================================================

HANDLE hStdOut = INVALID_HANDLE_VALUE;

// =================================================================
// FUNCTION DEFINITIONS
// =================================================================

DWORD WINAPI printf (PBYTE pbFormat,
                     ...)
    {
    PBYTE pbBuffer;
    PVOID pArgumemts;
    DWORD l, n;

    n = MAXDWORD;
    pbBuffer = LocalAlloc (LMEM_FIXED, MAX_WVSPRINTF+1);

    if (pbBuffer != NULL)
        {
        pArgumemts = (PVOID) ((DWORD) &pbFormat + sizeof (DWORD));
        wvsprintf (pbBuffer, pbFormat, pArgumemts);
        pbBuffer [MAX_WVSPRINTF] = 0;

        n = 0;
        if ((l = lstrlen (pbBuffer)) &&
            !WriteFile (hStdOut, pbBuffer, l, &n, NULL))
            {
            n = MAXDWORD;
            }
        LocalFree (pbBuffer);
        }
    return n;
    }
    
// -----------------------------------------------------------------

BOOL WINAPI GetNicAddress (NIC_ADDRESS NicAddress)
    {
    WSADATA      wd;
    SOCKET       s;
    SOCKADDR_IPX sa;
    DWORD        n;
    BOOL         fOk = FALSE;

    if (NIC_ADDRESS_ == sizeof (sa.sa_nodenum))
        {
        if (!WSAStartup (WS_VERSION, &wd))
            {
            if (wd.wVersion == WS_VERSION)
                {
                if ((s = socket (AF_IPX, SOCK_DGRAM, NSPROTO_IPX))
                    != INVALID_SOCKET)
                    {
                    ZeroMemory (&sa, sizeof (sa));
                    sa.sa_family = AF_IPX;

                    n = sizeof (sa);
                    if ((!bind (s, (PSOCKADDR) &sa, n)) &&
                        (!getsockname (s, (PSOCKADDR) &sa, &n)))
                        {
                        CopyMemory (NicAddress,
                                    sa.sa_nodenum,
                                    NIC_ADDRESS_);
                        fOk = TRUE;
                        }
                    closesocket (s);
                    }
                }
            WSACleanup ();
            }
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI GetIpAddress (IP_ADDRESS IpAddress)
    {
    WSADATA  wd;
    PHOSTENT phe;
    BYTE     abHostName [WSAMAXHOSTNAME];
    BOOL     fOk = FALSE;

    if (!WSAStartup (WS_VERSION, &wd))
        {
        if (wd.wVersion == WS_VERSION)
            {
            if ((!gethostname (abHostName, WSAMAXHOSTNAME))  &&
                ((phe = gethostbyname (abHostName)) != NULL) &&
                (phe->h_length == IP_ADDRESS_))
                {
                CopyMemory (IpAddress,
                            phe->h_addr,
                            IP_ADDRESS_);
                fOk = TRUE;
                }
            }
        WSACleanup ();
        }
    return fOk;
    }

// =================================================================
// PROGRAM ENTRY POINT
// =================================================================

void WinStartup (void)
    {
    NIC_ADDRESS na;
    IP_ADDRESS  ia;
    DWORD       i;

    hStdOut = GetStdHandle (STD_OUTPUT_HANDLE);

    printf ("\n  NIC address:  ");
    if (GetNicAddress (na))
        {
        for (i = 0; i < NIC_ADDRESS_; i++)
            {
            printf ("%02X", na [i]);
            }
        }
    else
        {
        printf ("N/A");
        }
    printf ("\n  IP  address:  ");
    if (GetIpAddress (ia))
        {
        for (i = 0; i < IP_ADDRESS_; i++)
            {
            printf ("%s%lu", (i ? "." : ""), ia [i]);
            }
        }
    else
        {
        printf ("N/A");
        }
    printf ("\n");
    ExitProcess (0);
    return;
    }

// =================================================================
// END OF PROGRAM
// =================================================================
