//==================================================================
//  Main.cpp
//    Bret S. Pehrson for Windows Developer's Journal
//==================================================================

#if !defined(__BORLANDC__)
#   pragma comment(lib, "user32.lib")
/* disable unnecessary level 4 warnings */
#   pragma warning (disable : 4201)  //nonstandard extension used
#   pragma warning (disable : 4514)  //unreferenced inline function
#endif

/* include files */
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>
#include <windowsx.h>
#include "DerefLnk.h"

/* local manifest constants */
#define IDC_EDIT_WINDOW 1

//------------------------------------------------------------------
//OnCreate()
//------------------------------------------------------------------
static BOOL OnCreate(HWND hwnd, LPCREATESTRUCT /*lpCreateStruct*/)
{
    /* enable drag and drop for the main window */
    DragAcceptFiles(hwnd, TRUE);
    /* create edit window for output */
    CreateWindow(
        TEXT("EDIT"),
        TEXT(""),
        WS_CHILD|WS_VISIBLE|ES_MULTILINE|ES_WANTRETURN,
        0, 0, 0, 0,
        hwnd,
        (HMENU)IDC_EDIT_WINDOW,
        GetWindowInstance(hwnd),
        NULL
    );
    /* done */
    return TRUE;
}

//------------------------------------------------------------------
//OnSize()
//------------------------------------------------------------------
static void OnSize(HWND hwnd, UINT /*state*/, int cx, int cy)
{
    /* resize edit window */
    SetWindowPos(
        GetDlgItem(hwnd, IDC_EDIT_WINDOW), 
        NULL, 0, 0, cx, cy, SWP_NOZORDER|SWP_NOMOVE);
}

//------------------------------------------------------------------
//OnDropFiles()
//------------------------------------------------------------------
static void OnDropFiles(HWND hwnd, HDROP hDrop)
{
    /* STACK variables */
    const UINT cFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
    const HWND hwndEdit = GetDlgItem(hwnd, IDC_EDIT_WINDOW);
    TCHAR      szFileName[MAX_PATH+1];
    TCHAR      szDerefName[MAX_PATH+1];
    TCHAR      szText[MAX_PATH*3];
    BOOL       fValidName;

    /* loop through all dropped files */
    for (UINT index=0; index<cFiles; index++)
    {
        szFileName[0]  = NULL;
        szDerefName[0] = NULL;
        DragQueryFile(hDrop, index, szFileName, MAX_PATH);
        fValidName = DerefShellLink(
            szFileName, szDerefName, MAX_PATH);
        wsprintf(szText, TEXT("%s -> %s\r\n"), 
            szFileName, fValidName ? szDerefName : TEXT("FAILED"));
        Edit_SetSel(hwndEdit, (UINT)-1, (UINT)-1);
        Edit_ReplaceSel(hwndEdit, szText);
    }
    /* clean up */
    DragFinish(hDrop);
}

//------------------------------------------------------------------
//OnDestroy()
//------------------------------------------------------------------
static void OnDestroy(HWND /*hwnd*/)
{
    /* exit message pump */
    PostQuitMessage(0);
}

//------------------------------------------------------------------
//WndProc()
//------------------------------------------------------------------
static LRESULT CALLBACK WndProc(
    HWND hwnd, 
    UINT uMessage, 
    WPARAM wParam, 
    LPARAM lParam)
{
    /* process window messages */
    switch (uMessage) 
    {
        HANDLE_MSG(hwnd, WM_CREATE, OnCreate);
        HANDLE_MSG(hwnd, WM_SIZE, OnSize);
        HANDLE_MSG(hwnd, WM_DROPFILES, OnDropFiles);
        HANDLE_MSG(hwnd, WM_DESTROY, OnDestroy);
    }
    return DefWindowProc(hwnd, uMessage, wParam, lParam);
}

//------------------------------------------------------------------
//WinMain()
//------------------------------------------------------------------
int WINAPI WinMain(
    HINSTANCE hInstance, 
    HINSTANCE /*hPrevInstance*/, 
    LPSTR /*lpCmdLine*/, 
    INT /*nCmdShow*/)
{
    /* STACK variables */
    #ifdef UNICODE
    const LPCTSTR lpszWindowName = TEXT("DerefShellLink UNICODE");
    #else //!UNICODE
    const LPCTSTR lpszWindowName = TEXT("DerefShellLink");
    #endif //!UNICODE
    WNDCLASS      wc = { 0 };
    MSG           msg;

    /* define and register the window class */
    wc.lpfnWndProc   = WndProc;
    wc.hInstance     = hInstance;
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.lpszClassName = TEXT("Class_DSLWindow");
    RegisterClass(&wc);
    /* create main window */
    CreateWindow(
        wc.lpszClassName,
        lpszWindowName,
        WS_OVERLAPPEDWINDOW|WS_VISIBLE,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL
    );
    /* message pump */
    while (GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    /* done */
    return 0;
}
