//===============================================================
//  Rebuild.cpp
//===============================================================

/* disable unnecessary level 4 warnings */
#pragma warning (disable : 4201)  //nonstandard extension used
#pragma warning (disable : 4514)  //unreferenced inline fn...

/* include files */
#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <tchar.h>
#include <stdlib.h>
#include "Rebuild.h"

//---------------------------------------------------------------
//BOOL RebuildShellIcons(void)
//  Forces the shell to rebuild its icons.
//  Return Value: TRUE on success, FALSE on failure
//---------------------------------------------------------------
BOOL RebuildShellIcons(void)
{
  /* CONSTANTS */
  static const LPCTSTR lpszWindowMetrics = 
    TEXT("Control Panel\\Desktop\\WindowMetrics");
  static const LPCTSTR lpszShellIconSize = 
    TEXT("Shell Icon Size");
  static const DWORD   dwTimeout = 10000;  //10 seconds
  /* STACK variables */
  DWORD dwValueType = 0;
  DWORD cbBuffer = 0;
  DWORD dwResult = 0;
  BYTE  buffer[256+1];
  LONG  result = 0;
  HKEY  hKey = NULL;
  UINT  uShellIconSize = 0;
  BOOL  fSuccess = FALSE;

  /* method to the madness:  set the shell icon size value to 
   * one less than its current value, send a WM_SETTINGCHANGE, 
   * restore the shell icon size, and send another 
   * WM_SETTINGCHANGE. this technique was stolen from the 
   * TweakUI control panel applet. */

  /* open the window metrics key */
  result = RegOpenKeyEx(
    HKEY_CURRENT_USER, 
    lpszWindowMetrics, 
    0, 
    KEY_QUERY_VALUE|KEY_SET_VALUE, &hKey);
  if (result == ERROR_SUCCESS)
  {
    /* get the current value for the shell icon size */
    cbBuffer = sizeof(buffer);
    result = RegQueryValueEx(
      hKey, 
      lpszShellIconSize, 
      NULL, 
      &dwValueType, 
      buffer, 
      &cbBuffer); 
    if (result == ERROR_SUCCESS)
    {
      /* decrement the icon size value */
      switch (dwValueType)
      {
        case REG_DWORD:
          uShellIconSize = *(DWORD *)buffer;
          *(DWORD *)buffer = uShellIconSize - 1;
          cbBuffer = sizeof(DWORD);
          break;
        case REG_SZ:
          uShellIconSize = _ttoi((LPCTSTR)buffer);
          _itot(uShellIconSize-1, (LPTSTR)buffer, 10);
          cbBuffer = 
            (lstrlen((LPTSTR)buffer) + 1) * sizeof(TCHAR);
          break;
        default:
          result = !ERROR_SUCCESS;
          break;
      }
      if (result == ERROR_SUCCESS)
      {
        /* set the new (temporary) icon size value */
        result = RegSetValueEx(
          hKey, 
          lpszShellIconSize, 
          0, 
          dwValueType, 
          buffer, 
          cbBuffer);
        if (result == ERROR_SUCCESS)
        {
          /* notify everyone of change */
          SendMessageTimeout(
            HWND_BROADCAST, 
            WM_SETTINGCHANGE, 
            SPI_SETNONCLIENTMETRICS, 
            0, 
            SMTO_ABORTIFHUNG, 
            dwTimeout, 
            &dwResult);
          /* restore the icon size value */
          switch (dwValueType)
          {
            case REG_DWORD:
              *(DWORD *)buffer = uShellIconSize;
              cbBuffer = sizeof(DWORD);
              break;
            case REG_SZ:
              _itot(uShellIconSize, (LPTSTR)buffer, 10);
              cbBuffer = 
                (lstrlen((LPTSTR)buffer) + 1) * sizeof(TCHAR);
              break;
          }
          result = RegSetValueEx(
            hKey, 
            lpszShellIconSize, 
            0, 
            dwValueType, 
            buffer, 
            cbBuffer);
          /* notify everyone of change */
          SendMessageTimeout(
            HWND_BROADCAST, 
            WM_SETTINGCHANGE, 
            SPI_SETNONCLIENTMETRICS, 
            0, 
            SMTO_ABORTIFHUNG, 
            dwTimeout, 
            &dwResult);
          fSuccess = result == ERROR_SUCCESS;
        }
      }
    }
    /* close key */
    RegCloseKey(hKey); hKey = NULL;
  }
  /* done */
  return fSuccess;
}
