Attribute VB_Name = "Globals"
Option Explicit

'---- Copy in memory
Public Declare Sub MemCopy Lib "kernel32" Alias "RtlMoveMemory" (pDest As Any, pSource As Any, ByVal ByteLen As Long)

'---- API Functions
Type POINTAPI
        x As Long
        y As Long
End Type

Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
End Type

Type SIZE
        cx As Long
        cy As Long
End Type

' Sizing
Public Const WM_SIZING = &H214

Public Const WMSZ_LEFT = 1
Public Const WMSZ_RIGHT = 2
Public Const WMSZ_TOP = 3
Public Const WMSZ_TOPLEFT = 4
Public Const WMSZ_TOPRIGHT = 5
Public Const WMSZ_BOTTOM = 6
Public Const WMSZ_BOTTOMLEFT = 7
Public Const WMSZ_BOTTOMRIGHT = 8

' Hittest
Public Const WM_NCHITTEST = &H84

' WM_NCHITTEST and MOUSEHOOKSTRUCT Mouse Position Codes
Public Const HTERROR = (-2)
Public Const HTTRANSPARENT = (-1)
Public Const HTNOWHERE = 0
Public Const HTCLIENT = 1
Public Const HTCAPTION = 2
Public Const HTSYSMENU = 3
Public Const HTGROWBOX = 4
Public Const HTSIZE = HTGROWBOX
Public Const HTMENU = 5
Public Const HTHSCROLL = 6
Public Const HTVSCROLL = 7
Public Const HTMINBUTTON = 8
Public Const HTMAXBUTTON = 9
Public Const HTLEFT = 10
Public Const HTRIGHT = 11
Public Const HTTOP = 12
Public Const HTTOPLEFT = 13
Public Const HTTOPRIGHT = 14
Public Const HTBOTTOM = 15
Public Const HTBOTTOMLEFT = 16
Public Const HTBOTTOMRIGHT = 17
Public Const HTBORDER = 18
Public Const HTREDUCE = HTMINBUTTON
Public Const HTZOOM = HTMAXBUTTON
Public Const HTSIZEFIRST = HTLEFT
Public Const HTSIZELAST = HTBOTTOMRIGHT

' Window field offsets for GetWindowLong() and GetWindowWord()
Public Const GWL_WNDPROC = (-4)
Public Const GWL_HINSTANCE = (-6)
Public Const GWL_HWNDPARENT = (-8)
Public Const GWL_STYLE = (-16)
Public Const GWL_EXSTYLE = (-20)
Public Const GWL_USERDATA = (-21)
Public Const GWL_ID = (-12)


' GetSystemMetrics() codes
Public Const SM_CXSCREEN = 0
Public Const SM_CYSCREEN = 1
Public Const SM_CXVSCROLL = 2
Public Const SM_CYHSCROLL = 3
Public Const SM_CYCAPTION = 4
Public Const SM_CXBORDER = 5
Public Const SM_CYBORDER = 6
Public Const SM_CXDLGFRAME = 7
Public Const SM_CYDLGFRAME = 8
Public Const SM_CYVTHUMB = 9
Public Const SM_CXHTHUMB = 10
Public Const SM_CXICON = 11
Public Const SM_CYICON = 12
Public Const SM_CXCURSOR = 13
Public Const SM_CYCURSOR = 14
Public Const SM_CYMENU = 15
Public Const SM_CXFULLSCREEN = 16
Public Const SM_CYFULLSCREEN = 17
Public Const SM_CYKANJIWINDOW = 18
Public Const SM_MOUSEPRESENT = 19
Public Const SM_CYVSCROLL = 20
Public Const SM_CXHSCROLL = 21
Public Const SM_DEBUG = 22
Public Const SM_SWAPBUTTON = 23
Public Const SM_RESERVED1 = 24
Public Const SM_RESERVED2 = 25
Public Const SM_RESERVED3 = 26
Public Const SM_RESERVED4 = 27
Public Const SM_CXMIN = 28
Public Const SM_CYMIN = 29
Public Const SM_CXSIZE = 30
Public Const SM_CYSIZE = 31
Public Const SM_CXFRAME = 32
Public Const SM_CYFRAME = 33
Public Const SM_CXMINTRACK = 34
Public Const SM_CYMINTRACK = 35
Public Const SM_CXDOUBLECLK = 36
Public Const SM_CYDOUBLECLK = 37
Public Const SM_CXICONSPACING = 38
Public Const SM_CYICONSPACING = 39
Public Const SM_MENUDROPALIGNMENT = 40
Public Const SM_PENWINDOWS = 41
Public Const SM_DBCSENABLED = 42
Public Const SM_CMOUSEBUTTONS = 43
Public Const SM_CMETRICS = 44
Public Const SM_CYSMCAPTION = 51

Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long) As Long
Declare Function AdjustWindowRectEx Lib "user32" (lpRect As RECT, ByVal dsStyle As Long, ByVal bMenu As Long, ByVal dwEsStyle As Long) As Long
Declare Function GetMenu Lib "user32" (ByVal hwnd As Long) As Long
Declare Function CallWindowProc Lib "user32" Alias _
    "CallWindowProcA" (ByVal lpPrevWndFunc As Long, _
    ByVal hwnd As Long, ByVal Msg As Long, _
    ByVal wParam As Long, ByVal lParam As Long) As Long
Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long
Declare Function GetWindowRect Lib "user32" (ByVal hwnd As Long, _
    lpRect As RECT) As Long

'---- Compute a form dimensions for a given client area
Public Function ComputeFormDimensions(hwnd As Long, ByVal ClientWidth As Long, ByVal ClientHeight As Long, Optional ByVal Menu As Boolean = False) As SIZE
    ' Client area rectangle
    Dim rc As RECT
    rc.Left = 0
    rc.Top = 0
    rc.Right = ClientWidth / Screen.TwipsPerPixelX
    rc.Bottom = ClientHeight / Screen.TwipsPerPixelY
    
    ' Compute global rectangle using style
    AdjustWindowRectEx _
                rc, _
                GetWindowLong(hwnd, GWL_STYLE), _
                Menu, _
                GetWindowLong(hwnd, GWL_EXSTYLE)
    
    ' Window dimensions
    Dim sz As SIZE
    sz.cx = (rc.Right - rc.Left) * Screen.TwipsPerPixelX
    sz.cy = (rc.Bottom - rc.Top) * Screen.TwipsPerPixelY
    ' Return
    ComputeFormDimensions = sz
End Function

'---- Convert a rectangle from Twips to Pixels
Public Sub rcTwipsToPixels(rc As RECT)
    rc.Left = rc.Left / Screen.TwipsPerPixelX
    rc.Right = rc.Right / Screen.TwipsPerPixelX
    rc.Top = rc.Top / Screen.TwipsPerPixelY
    rc.Bottom = rc.Bottom / Screen.TwipsPerPixelY
End Sub

'---- Convert a rectangle from Pixels to Twips
Public Sub rcPixelsToTwips(rc As RECT)
    rc.Left = rc.Left * Screen.TwipsPerPixelX
    rc.Right = rc.Right * Screen.TwipsPerPixelX
    rc.Top = rc.Top * Screen.TwipsPerPixelY
    rc.Bottom = rc.Bottom * Screen.TwipsPerPixelY
End Sub

'---- Return a long low word
Public Function LOWORD(l As Long) As Integer
    LOWORD = l And &HFFFF&
End Function

'---- Return a long high word
Public Function HIWORD(l As Long) As Integer
    HIWORD = CInt((l And &HFFFF0000) / 65536)
End Function
