Attribute VB_Name = "WindowProc"
Option Explicit

'---- Win32 functions declarations
Declare Function CallWindowProc Lib "user32" Alias _
    "CallWindowProcA" (ByVal lpPrevWndFunc As Long, _
    ByVal hwnd As Long, ByVal Msg As Long, _
    ByVal wParam As Long, ByVal lParam As Long) As Long

Declare Function SetWindowLong Lib "user32" Alias _
    "SetWindowLongA" (ByVal hwnd As Long, _
    ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

Declare Function IsWindow Lib "user32" (ByVal hwnd As Long) As Long

' Window field offsets for GetWindowLong() and GetWindowWord()
Public Const GWL_WNDPROC = (-4)
Public Const GWL_HINSTANCE = (-6)
Public Const GWL_HWNDPARENT = (-8)
Public Const GWL_STYLE = (-16)
Public Const GWL_EXSTYLE = (-20)
Public Const GWL_USERDATA = (-21)
Public Const GWL_ID = (-12)

' Collection to store references to calling objects
Private mcolObject As New Collection
' Collection to store previous window procedures
Private mcolWndProc As New Collection

'---- Set subclassing
Public Sub modSubClass(sc As SubClasser)
    ' Window handle
    Dim hwnd As Long: hwnd = sc.hwnd
    
    Dim lpPrevWindowProc As Long
    ' Subclass the window
    lpPrevWindowProc = SetWindowLong(hwnd, GWL_WNDPROC, AddressOf WindowProc)
    ' Store previous WndProc in collection
    mcolWndProc.Add lpPrevWindowProc, Hex(hwnd)
    ' Store calling object in collection
    mcolObject.Add sc, Hex(hwnd)
End Sub

'---- Remove subclassing
Public Sub modReClass(sc As SubClasser)
    ' Window handle
    Dim hwnd As Long: hwnd = sc.hwnd
    
    ' Previous window procedure
    Dim lpPrevWindowProc As Long
    lpPrevWindowProc = mcolWndProc(Hex(hwnd))
    If lpPrevWindowProc Then
        ' Remove subclassing
        SetWindowLong hwnd, GWL_WNDPROC, lpPrevWindowProc
        ' Remove from collections
        mcolWndProc.Remove Hex(hwnd)
        mcolObject.Remove Hex(hwnd)
    End If
End Sub

'---- Call original Window Procedure
Public Function modCallPrevWindowProc(ByVal hwnd As Long, ByVal uMsg As Long, _
        ByVal wParam As Long, ByVal lParam As Long) As Long
    modCallPrevWindowProc = CallWindowProc(mcolWndProc(Hex(hwnd)), hwnd, uMsg, wParam, lParam)
End Function

'---- New Window Procedure
Private Function WindowProc(ByVal hwnd As Long, ByVal uMsg As Long, _
        ByVal wParam As Long, ByVal lParam As Long) As Long
    ' Flag to call previous Window Procedure
    Dim fCall As Boolean: fCall = True

    ' Retrieve object for window
    Dim sc As SubClasser
    On Error Resume Next
    Set sc = mcolObject(Hex(hwnd))
    
    ' Generate event
    If Not sc Is Nothing Then _
        WindowProc = sc.Message(hwnd, uMsg, wParam, lParam, fCall)
    
    ' Call original procedure
    If fCall Then _
        WindowProc = CallWindowProc(mcolWndProc(Hex(hwnd)), hwnd, uMsg, wParam, lParam)
End Function
