/* Borland C++: bcc32  -w -DSTRICT startsvc.c
 * Visual C++:  cl  /DSTRICT /W3 startsvc.c
 */

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <tlhelp32.h>

#define SERVICE_KEY "Software\\Microsoft\\Windows" \
    "\\CurrentVersion\\RunServices"

/* AddService() - install a Win95 "service".
 */
BOOL AddService(const char* Name, const char *CmdLine)
    {
    HKEY    hKey;
    LONG    Status;
    BOOL    Result  = FALSE;    /* assume failure           */

    Status = RegCreateKey(HKEY_LOCAL_MACHINE, SERVICE_KEY, &hKey);
    
    if(ERROR_SUCCESS == Status)
        {
        Status  = RegSetValueEx(hKey, Name, 0, REG_SZ,
                    (CONST BYTE*)CmdLine, strlen(CmdLine)+1);
        if(ERROR_SUCCESS == Status)
            Result  = TRUE;
        RegCloseKey(hKey);
        }

    return Result;
    }


/* StartWin95Service() - start .exe with no window, specific priority
 */

#define NUM_ATTEMPTS_EXISTS     20
#define SLEEP_TIME              500

DWORD StartWin95Service(char chPriorityClass, char* file_name)
    {
    PROCESS_INFORMATION piProcInfo; 
    STARTUPINFO         siStartInfo; 
    DWORD               dwPriority;
    DWORD               dwResult = (DWORD)-1;
    int                 ii;

    /* Set priority */
    if(chPriorityClass == 'h')
        dwPriority = HIGH_PRIORITY_CLASS;
    else if(chPriorityClass == 'i')
        dwPriority = IDLE_PRIORITY_CLASS;
    else if(chPriorityClass == 'n')
        dwPriority = NORMAL_PRIORITY_CLASS;

    /* Now create the child process.                    *
     * Set up members of STARTUPINFO structure.         */
 
    siStartInfo.cb          = sizeof(STARTUPINFO); 
    siStartInfo.lpReserved  = NULL; 
    siStartInfo.lpReserved2 = NULL; 
    siStartInfo.cbReserved2 = 0; 
    siStartInfo.lpDesktop   = NULL; 
    siStartInfo.dwFlags     = STARTF_USESHOWWINDOW;

    /* Create the process   */
    if(FALSE == CreateProcess(file_name, 
        NULL,                 // command line
        NULL,                 // process security attributes
        NULL,                 // primary thread security attributes
        FALSE,                // handles are inherited
        DETACHED_PROCESS | dwPriority, // creation flags
        NULL,                 // use parent's environment
        NULL,                 // use parent's current directory
        &siStartInfo,         // STARTUPINFO pointer
        &piProcInfo))         // receives PROCESS_INFORMATION 
        {
        return -1;
        }
    else
        {
        /* Check if process has been started    */
        for(ii = 0; ii < NUM_ATTEMPTS_EXISTS; ii++)
            {
            GetExitCodeProcess(piProcInfo.hProcess, &dwResult);
            if(dwResult == STILL_ACTIVE)
                break;
            else
                Sleep(SLEEP_TIME);
            }
        if(dwResult == STILL_ACTIVE)
            {
             /* Set priority.  
              * For console apps dwCreationFlags in CreateProcess
              * doesn't work so we need to do this again
              * We might want to sleep here if the process sets its
              * priority by itself  and we want to change it
              */
            Sleep(2000);
            SetPriorityClass(piProcInfo.hProcess, dwPriority);
            dwResult = piProcInfo.dwProcessId;
            }
        else
            dwResult    = (DWORD)-1;
        }
    return dwResult;
    }


/* RegisterService() - mark a process as a Win95 "service"
 */
BOOL RegisterService (DWORD AppProcessId)
    {
    BOOL    Result  = FALSE;        /* assume failure */
    int     nstat;
    HMODULE hKernelLib;
    typedef DWORD (WINAPI*ExRegSrvProc) (DWORD dwProcessId, DWORD dwType);
    ExRegSrvProc RegSrvProc;

    hKernelLib = GetModuleHandle("KERNEL32.DLL");
    if(NULL != hKernelLib)
        {
        RegSrvProc = (ExRegSrvProc)GetProcAddress(hKernelLib, 
                     "RegisterServiceProcess");
        if(NULL != RegSrvProc)
            {
            /* 0x00000001 == RSP_SIMPLE_SERVICE */
            nstat = RegSrvProc(AppProcessId, 0x00000001);
            if(1 == nstat)
                Result  = TRUE;
            }
        }
    return Result;
    }


void usage(void) 
    {
    printf("svc95 option name\n\n");
    printf("                  option name\n");
    printf("add service       -a      svcname arg0 arg1 ...\n");
    printf("start service     -s[hin] file name of the executable\n");
    printf("show window       -f      file name of the executable\n");
    }

void main(int argc, char *argv[])
    {
    char            pString[512];
    DWORD           dwProcID;
    HANDLE          hProcess, hProcessSnap, hModuleSnap;
    PROCESSENTRY32  pe32; 
    MODULEENTRY32   me32; 
    BOOL            bModuleFound = FALSE;

    /* Check if arguments are valid         */
    if(argc < 3 || (argv[1][1] != 'a' && argv[1][1] != 's' &&
                     argv[1][1] != 'f'))
        {
        usage();
        return;
        }

    /* Add service to the registry      */
    if(argv[1][1] == 'a')
        {
        int     i;

        pString[0]  = '\0';
        for(i = 3; i < argc; ++i)
            {
            if(i > 3)
                strcat(pString, " ");
            strcat(pString, argv[i]);
            }
        if(AddService(argv[2], pString))
            printf("Added service '%s' ('%s')\n", argv[2], pString);
        else
            printf("Could not add service!\n");
        }
    /* Start the service with the normal priority and register it */
    if(argv[1][1] == 's')
        {
        char Priority = 'n';    /* default is normal priority */
        if(argv[1][2])
            Priority    = argv[1][2];
        dwProcID = StartWin95Service(Priority, argv[2]);
        RegisterService(dwProcID); 
        }

    /* Find the window, show it and then stop the process           */
    if(argv[1][1] == 'f')
        {
        strupr(argv[2]);
        /* Take snapshot of all processes currently in the system   */
        hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS,0); 
        pe32.dwSize = sizeof(PROCESSENTRY32); 
        if (Process32First(hProcessSnap, &pe32))
            { 
            do  {
                hModuleSnap = CreateToolhelp32Snapshot(
                             TH32CS_SNAPMODULE, pe32.th32ProcessID); 
                me32.dwSize = sizeof(MODULEENTRY32); 
                Module32First(hModuleSnap, &me32);
                do {
                    if(strcmp(me32.szExePath, argv[2]) == 0)
                        {
                        bModuleFound = TRUE;
                        break;  
                        }
                   }
                   while (Module32Next(hModuleSnap, &me32));
                CloseHandle (hModuleSnap); 
                if(bModuleFound)
                    break;
                }
                while (Process32Next(hProcessSnap, &pe32));
            } 
        CloseHandle (hModuleSnap); 

        if(bModuleFound)
            {
            hProcess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, 
                                 pe32.th32ProcessID);
            TerminateProcess(hProcess, -1);
            printf("Process terminated");
            }
        else 
            printf("Process not found");
        }
    }
