//
// Listing 1 - MAY98.CPP
//
// This program demonstrates a problem in the implementation
// of the STL when using Visual C++ 5.0.  I created a couple
// of routines called DisplayVector() and DisplaySet() that
// iterate through a container full of strings, printing
// each one on the way.  Both of these routines work properly
// when they are included in the EXE I create here.  However,
// if the routines are moved into a DLL, the routine that
// iterates through the set crashes badly.
//
// To demonstrate this I created companion routines with
// identical functionality in DISPLAY.CPP, with the slightly
// modified names DisplayVectorDll() and DisplaySetDll().
// I compile these routines and place them in a DLL.  Calling
// DisplayVectorDll() from this EXE works fine, but the
// routine that displays sets crashes the first time we
// attempt to increment its iterator.
//
// Before building the EXE you must build the DLL.  This is
// done using the following command line:
//
//   cl /GX /LD Display.cpp
//
// This will create Display.lib and Display.dll.  From the
// same directory, you can then create the executable using
// this command line:
//
//   cl /GX May98.cpp Display.lib
//
// Running the program should cause a fatal exception when you
// reach the code in DisplaySetDll().
//

#include <set>
#include <vector>
#include <string>
#include <iostream>

using namespace std;

//
// Microsoft gives a warning when creating most common sets,
// because the final class name is greater than 255 characters,
// which confuses the debugger.  We agree that this is a problem,
// but issuing a warning is pointless in this context, because
// we can't do anything about it!
//
#pragma warning( disable : 4786 )

//
// These two routines are found in the Display.cpp
// module, and are compiled and linked to Display.dll
//

__declspec( dllimport )
int DisplayVectorDll( const vector< string > &s );
__declspec( dllimport )
int DisplaySetDll( const set< string > &s );

//
// Display the contents of the set on a single line
//
int DisplaySet( const set< string > &s )
{
    cout << "\nListing set from EXE: ";
    set<string>::const_iterator ii = s.begin();
	while ( ii != s.end() ) {
		cout << *ii << "  ";
                ii++;
        }
        cout << "\n\n";
	return 1;
}

//
// Display the contents of the vector on a single line
//
int DisplayVector( const vector < string > &s )
{
    cout << "\nListing vector from EXE: ";
    vector<string>::const_iterator ii = s.begin();
	while ( ii != s.end() ) {
		cout << *ii << "  ";
                ii++;
        }
        cout << "\n\n";
	return 1;
}

main()
{
    set< string > name_set;
    name_set.insert( "WDJ" );
    name_set.insert( "CUJ" );
    name_set.insert( "DDJ" );
    name_set.insert( "MSJ" );
    vector< string > name_vector( name_set.size() );
    copy( name_set.begin(), name_set.end(), name_vector.begin() );

    DisplayVector( name_vector );
    DisplayVectorDll( name_vector );
    DisplaySet( name_set );
    DisplaySetDll( name_set );
    return 0;
}

//EOF
