// OptTree.cpp : implementation file
//

#include "stdafx.h"
#include "OptTree.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifdef _DEBUG

#define ASSERT_RETURN(x)            if(!(x))            \
                                    {                   \
                                        ASSERT(FALSE);  \
                                        return;         \
                                    }
#define ASSERT_RETURN_VALUE(x,v)    if(!(x))            \
                                    {                   \
                                        ASSERT(FALSE);  \
                                        return (v);     \
                                    }
#else

#define ASSERT_RETURN(x)
#define ASSERT_RETURN_VALUE(x,v)

#endif

////////////////////////////////////////////////////////////////
// COptionsTree

COptionsTree::COptionsTree()
{
    m_nCheck0 = 0;
    m_nCheck1 = 1;
    m_nRadio0 = 2;
    m_nRadio1 = 3;
}

COptionsTree::~COptionsTree()
{
}


BEGIN_MESSAGE_MAP(COptionsTree, CTreeCtrl)
    //{{AFX_MSG_MAP(COptionsTree)
    ON_NOTIFY_REFLECT(TVN_KEYDOWN, OnKeydown)
    ON_NOTIFY_REFLECT(NM_CLICK, OnClick)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

////////////////////////////////////////////////////////////////
// COptionsTree message handlers

// Handler for mouse click.
void COptionsTree::OnClick(NMHDR* /*pNMHDR*/, LRESULT* pResult)
{
    *pResult = 0;

    HTREEITEM hItem = GetClickedItem();
    if (hItem)
    {
        ToggleItem(hItem);
    }
}

// Handler for key down.
void COptionsTree::OnKeydown(NMHDR* pNMHDR, LRESULT* pResult) 
{
    TV_KEYDOWN* pTVKeyDown = (TV_KEYDOWN*)pNMHDR;
    *pResult = 0;

    if (pTVKeyDown->wVKey == VK_SPACE)
    {
        HTREEITEM hItem = GetSelectedItem();
        if (hItem)
        {
            ToggleItem(hItem);
        }
    }
}

// Function toggles the status of a checkbox
// or a radiobutton item.
void COptionsTree::ToggleItem(HTREEITEM hItem)
{
    ASSERT(hItem);

    if (IsCheckBox(hItem))
    {
        if (GetCheck(hItem))
        {
            SetCheckBox(hItem, 0);
        }
        else
        {
            SetCheckBox(hItem, 1);
        }
        // Send MODIFIED message to parent if needed.
    }
    else
    if (IsRadioButton(hItem))
    {
        if (!GetCheck(hItem))
        {
            SetRadioButton(hItem, 1);
            // Send MODIFIED message to parent if needed.
        }
    }
}

// Function gets the handle to the item the cursor points to.
HTREEITEM COptionsTree::GetClickedItem()
{
    // Get the current cursor coord.
    CPoint pt;
    ::GetCursorPos(&pt);
    ScreenToClient(&pt);

    // Loop thru' just the visible items.
    HTREEITEM hItem = GetFirstVisibleItem();
    if (!hItem)
    {
        return NULL;
    }

    do
    {
        if (IsPtOnItem(hItem, pt))
        {
            return hItem;
        }
        hItem = GetNextVisibleItem(hItem);
    }while (hItem);

    return NULL;
}

// Function gets the handle to the item the cursor points to.
BOOL COptionsTree::IsPtOnItem(HTREEITEM hItem, const CPoint& pt)
{
    ASSERT(hItem);

    CRect rcItem;
    GetItemRect(hItem, rcItem, TRUE );

    if (pt.y >= rcItem.top && pt.y <= rcItem.bottom)
    {
        // Adjust for any offset.
        int nIndent = (int)GetIndent();
        if (pt.x >= rcItem.left-nIndent && pt.x <= rcItem.right)
        {
            return TRUE;
        }
    }
    return FALSE;
}

// Function adds a parent item to the options tree.
HTREEITEM COptionsTree::AddGroup(LPCSTR pszText,
                                int nImageIndex,
                                HTREEITEM hParent/*=NULL*/,
                                BOOL bExpanded/*=TRUE*/)
{
    TV_INSERTSTRUCT	tvs;

    tvs.hParent = hParent;
    tvs.hInsertAfter = TVI_LAST;
    tvs.item.mask = TVIF_CHILDREN | TVIF_IMAGE |
                    TVIF_SELECTEDIMAGE | TVIF_STATE | TVIF_TEXT;

    tvs.item.state = tvs.item.stateMask =
                        (bExpanded) ? TVIS_EXPANDED : 0;
    tvs.item.cChildren = 1;

    tvs.item.pszText = (LPSTR)pszText;
    tvs.item.iImage = tvs.item.iSelectedImage = nImageIndex;

    return InsertItem(&tvs);
}

// Function adds a CheckBox item to the options tree.
BOOL COptionsTree::AddCheckBox(  UINT uID,
                                LPCSTR pszText,
                                HTREEITEM hParent,
                                BOOL bChecked/*=FALSE*/)
{
#ifdef _DEBUG
    if (!uID || GetItemFromID(uID))
    {
        AfxMessageBox("COptionsTree::AddCheckBox-Invalid/Dup ID");
        ASSERT(FALSE);// Give a chance to debug.
        return FALSE;
    }
#endif

    TV_INSERTSTRUCT	tvs;

    tvs.hParent = hParent;
    tvs.hInsertAfter = TVI_LAST;
    tvs.item.mask = TVIF_IMAGE |TVIF_SELECTEDIMAGE| TVIF_TEXT;

    tvs.item.pszText = (LPSTR)pszText;
    tvs.item.iImage = tvs.item.iSelectedImage =
                        (bChecked) ? m_nCheck1 : m_nCheck0;

    return AddItem(uID, &tvs);
}

// Function adds a RadioButton item to the options tree.
BOOL COptionsTree::AddRadioButton(   UINT uID,
                                    LPCSTR pszText,
                                    HTREEITEM hParent,
                                    BOOL bSelected/*=FALSE*/)
{
#ifdef _DEBUG
    if (!uID || GetItemFromID(uID))
    {
        AfxMessageBox("COptionsTree::AddRadioButt-Invalid/Dup ID");
        ASSERT(FALSE);// Give a chance to debug.
        return FALSE;
    }
#endif

    TV_INSERTSTRUCT	tvs;

    tvs.hParent = hParent;
    tvs.hInsertAfter = TVI_LAST;
    tvs.item.mask = TVIF_IMAGE |TVIF_SELECTEDIMAGE| TVIF_TEXT;

    tvs.item.pszText = (LPSTR)pszText;
    tvs.item.iImage = tvs.item.iSelectedImage =
                        (bSelected) ? m_nRadio1 : m_nRadio0;

    return AddItem(uID, &tvs);
}

// Function pass-thru to InsertItem. Also adds the ID to the map.
BOOL COptionsTree::AddItem(UINT uID, LPTV_INSERTSTRUCT ptvs)
{
    HTREEITEM hItem = InsertItem(ptvs);
    ASSERT(hItem);
    m_muhItems.SetAt(uID, hItem);

    return TRUE;
}

// Function gets the handle of the item for a given ID
HTREEITEM COptionsTree::GetItemFromID(UINT uID)
{
    HTREEITEM hItem = 0;
    m_muhItems.Lookup(uID, (void*&)hItem);
    return hItem;
}

// Function gets the ID of the item for a given handle
UINT COptionsTree::GetIDFromItem(HTREEITEM hItemFrom)
{
    ASSERT_RETURN_VALUE(hItemFrom, 0);
    UINT uID;
    HTREEITEM hItem;

    for(POSITION pos = m_muhItems.GetStartPosition(); pos; )
    {
        m_muhItems.GetNextAssoc(pos, uID, (void*&)hItem);

        if (hItem == hItemFrom)
        {
            return uID;
        }
    }

    ASSERT(FALSE);
    return 0;
}

// Function sets the item ON/OFF
void COptionsTree::SetCheck(UINT uID, int nCheck)
{
    ASSERT(!nCheck || nCheck==1);

    HTREEITEM hItem = GetItemFromID(uID);
    SetCheck(hItem, nCheck);
}

// Function sets the item ON/OFF
void COptionsTree::SetCheck(HTREEITEM hItem, int nCheck)
{
    ASSERT(!nCheck || nCheck==1);
    ASSERT(hItem);

    if (GetCheck(hItem) == nCheck)
    {
        // Already Set. Just return.
        return;
    }

    if (IsCheckBox(hItem))
    {
        SetCheckBox(hItem, nCheck);
    }
    else
    if (IsRadioButton(hItem))
    {
        SetRadioButton(hItem, nCheck);
    }
}

// Function sets the CheckBox item ON/OFF
void COptionsTree::SetCheckBox(HTREEITEM hItem, int nCheck)
{
    int iImage;
    ASSERT(IsCheckBox(hItem));

    if (nCheck)
    {
        iImage = m_nCheck1;
    }
    else
    {
        iImage = m_nCheck0;
    }

    VERIFY(SetItemImage(hItem, iImage, iImage));
}

// Function sets the RadioButton item ON/OFF
void COptionsTree::SetRadioButton(HTREEITEM hItem, int nCheck)
{
    int iImage;
    ASSERT_RETURN(IsRadioButton(hItem));

    if (nCheck)
    {
        iImage = m_nRadio1;
        HTREEITEM hSibbling = hItem;

        // Set all other radio butts ABOVE to OFF
        while ((hSibbling = GetPrevSiblingItem(hSibbling)) != NULL)
        {
            VERIFY(SetItemImage(hSibbling,m_nRadio0,m_nRadio0));
        }

        hSibbling = hItem;

        // Set all other radio butts BELOW to OFF
        while ((hSibbling = GetNextSiblingItem(hSibbling)) != NULL)
        {
            VERIFY(SetItemImage(hSibbling,m_nRadio0,m_nRadio0));
        }
    }
    else
    {
        iImage = m_nRadio0;
    }
    VERIFY(SetItemImage(hItem, iImage, iImage));
}

// Function Gets the status of the item
int COptionsTree::GetCheck(UINT uID)
{
    HTREEITEM hItem = GetItemFromID(uID);
    return GetCheck(hItem);
}

// Function Gets the status of the item
int COptionsTree::GetCheck(HTREEITEM hItem)
{
    ASSERT_RETURN_VALUE(hItem, 0);
    ASSERT_RETURN_VALUE(IsCheckBox(hItem) ||
                            IsRadioButton(hItem), 0);

    int iImage, iSelectedImage;
    VERIFY(GetItemImage(hItem, iImage, iSelectedImage));

    if (iImage==m_nCheck0 || iImage==m_nRadio0)
    {
        return 0;
    }

    if (iImage==m_nCheck1 || iImage==m_nRadio1)
    {
        return 1;
    }

    ASSERT(FALSE);
    return 0;
}

// Function returns TRUE if given item is a CheckBox.
BOOL COptionsTree::IsCheckBox(HTREEITEM hItem)
{
    int iImage, iSelectedImage;

    ASSERT(hItem);
    VERIFY(GetItemImage(hItem, iImage, iSelectedImage));

    if (iImage==m_nCheck0 || iImage==m_nCheck1)
    {
        return TRUE;
    }

    return FALSE;
}

// Function returns TRUE if given item is a RadioButton.
BOOL COptionsTree::IsRadioButton(HTREEITEM hItem)
{
    int iImage, iSelectedImage;

    ASSERT(hItem);
    VERIFY(GetItemImage(hItem, iImage, iSelectedImage));

    if (iImage==m_nRadio0 || iImage==m_nRadio1)
    {
        return TRUE;
    }

    return FALSE;
}

// Function Retrieves the ID of the currently checked
// RadioButton in the specified group.
UINT COptionsTree::GetCheckedRadioButton(UINT uIDFirstButton,
                                        UINT uIDLastButton)
{
    HTREEITEM hItemFirst = GetItemFromID(uIDFirstButton);
    HTREEITEM hItemLast = GetItemFromID(uIDLastButton);

    // Just checking....
    ASSERT_RETURN_VALUE(IsRadioButton(hItemFirst), 0);
    ASSERT_RETURN_VALUE(IsRadioButton(hItemLast), 0);
    ASSERT_RETURN_VALUE(GetParentItem(hItemFirst)==
                            GetParentItem(hItemLast), 0);

    HTREEITEM hSibbling = hItemFirst;
    do
    {
        ASSERT_RETURN_VALUE(IsRadioButton(hSibbling), 0);

        if (GetCheck(hSibbling))
        {
            return GetIDFromItem(hSibbling);
        }

        // I don't think this check is required.
        if (hSibbling == hItemLast)
        {
            break;
        }

        hSibbling = GetNextSiblingItem(hSibbling);
    }while(hSibbling);

    ASSERT(FALSE);
    return 0;
}

// Function Selects (adds a check mark to) a given radio button 
// in a group and clears (removes a check mark from) all other 
// radio buttons in the group. 
void COptionsTree::CheckRadioButton(UINT /*uIDFirstButton*/,
                                   UINT /*uIDLastButton*/,
                                   UINT uIDCheckButton)
{
    // We don't need the First and Last button IDs.
    // SetCheck will take care of setting the other buttons OFF.
    SetCheck(uIDCheckButton, 1);
}
