#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>

#define VWIN32_DIOC_DOS_DRIVEINFO   6

#include <pshpack1.h>
typedef struct {        // from MSDN-CD
    WORD  ExtFree_Size;
    WORD  ExtFree_Level;
    DWORD ExtFree_SectorsPerCluster;
    DWORD ExtFree_BytesPerSector;
    DWORD ExtFree_AvailableClusters;
    DWORD ExtFree_TotalClusters;
    DWORD ExtFree_AvailablePhysSectors;
    DWORD ExtFree_TotalPhysSectors;
    DWORD ExtFree_AvailableAllocationUnits;
    DWORD ExtFree_TotalAllocationUnits;
    DWORD ExtFree_Rsvd[2];
} EXTGETDISKFREESPACE;
#include <poppack.h>
 
typedef struct {        // from DDK vwin32.h
    DWORD reg_EBX;
    DWORD reg_EDX;
    DWORD reg_ECX;
    DWORD reg_EAX;
    DWORD reg_EDI;
    DWORD reg_ESI;
    DWORD reg_Flags;
} DIOC_REGISTERS;


BOOL GetFat32FreeSpace(LPCTSTR lpRootPathName,
            LPDWORD lpSectorsPerCluster, LPDWORD lpBytesPerSector,
            LPDWORD lpFreeClusters, LPDWORD lpTotalClusters)
{
    HANDLE              hDevice;
    BOOL                bResult;
    DIOC_REGISTERS      regs;
    EXTGETDISKFREESPACE efs;
    DWORD               cb;

    bResult = FALSE;
    ZeroMemory(&efs, sizeof(EXTGETDISKFREESPACE));
    ZeroMemory(&regs, sizeof(DIOC_REGISTERS));

    //
    // get device handle for VWIN32 (win9x only, fails on NT)
    //
    hDevice = CreateFile("\\\\.\\vwin32", 0, 0, NULL, 0,
                        FILE_FLAG_DELETE_ON_CLOSE, NULL);
    if (hDevice != NULL) {
        // 
        // issue an int 21h function 7303h
        //
        regs.reg_EAX = 0x7303;      // Get_ExtFreeSpace (FAT32)
        regs.reg_ECX = sizeof(EXTGETDISKFREESPACE);
        regs.reg_EDX = (DWORD) lpRootPathName;
        regs.reg_EDI = (DWORD) &efs;
        regs.reg_Flags = 0x0001;    // assume error (carry flag)

        bResult = DeviceIoControl(hDevice,
                VWIN32_DIOC_DOS_DRIVEINFO, &regs, sizeof(regs),
                &regs, sizeof(regs), &cb, 0);

        //
        //  only successful if DeviceIoControl() returns success
        //  *and* carry flag was not set by int 21h 7303h
        //
        if (bResult && !(regs.reg_Flags & 0x0001)) {
            *lpSectorsPerCluster = efs.ExtFree_SectorsPerCluster;
            *lpBytesPerSector = efs.ExtFree_BytesPerSector;
            *lpFreeClusters = efs.ExtFree_AvailableClusters;
            *lpTotalClusters = efs.ExtFree_TotalClusters;
        } else
            bResult = FALSE;        // error if carry flag set

        CloseHandle(hDevice);
    }

    return(bResult);
}

BOOL IsFat32Available(void)
{
    OSVERSIONINFO   osInfo;

    osInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    if (GetVersionEx(&osInfo)) {
        if (osInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
            if (LOWORD(osInfo.dwBuildNumber) > 1000)
                return(TRUE);
    }

    return(FALSE);
}

// ============== example for GetFat32FreeSpace() ==============
int main(int argc, char *argv[])
{
    LPTSTR  lpRootPath;
    BOOL    bResult;
    DWORD   dwSectorsPerCluster;
    DWORD   dwBytesPerSector;
    DWORD   dwFreeClusters;
    DWORD   dwTotalClusters;

    if (argc > 1)
        lpRootPath = argv[1];
    else {
        printf("Usage: fat32 <drive>:\\\n");
        return(0);
    }

    bResult = GetDiskFreeSpace(lpRootPath,
                        &dwSectorsPerCluster, &dwBytesPerSector,
                        &dwFreeClusters, &dwTotalClusters);
    if (bResult) {
        printf("GetDiskFreeSpace() returned:\n");
        printf("  dwSectorsPerCluster = %d\n", \
                dwSectorsPerCluster);
        printf("  dwBytesPerSector = %d\n", dwBytesPerSector);
        printf("  dwTotalFreeClusters = %d\n", dwFreeClusters);
        printf("  dwTotalClusters = %d\n", dwTotalClusters);
    } else
        printf("GetDiskFreeSpace() failed\n");

    if (IsFat32Available()) {
        bResult = GetFat32FreeSpace(lpRootPath,
                        &dwSectorsPerCluster, &dwBytesPerSector,
                        &dwFreeClusters, &dwTotalClusters);
        if (bResult) {
            printf("\nGetFat32FreeSpace() returned:\n");
            printf("  dwSectorsPerCluster = %d\n", \
                    dwSectorsPerCluster);
            printf("  dwBytesPerSector = %d\n", dwBytesPerSector);
            printf("  dwTotalFreeClusters = %d\n", dwFreeClusters);
            printf("  dwTotalClusters = %d\n", dwTotalClusters);
        } else
            printf("\nGetFat32FreeSpace() failed\n");
    } else
        printf("\nFAT32 not supported\n");

    return(0);
}
