; File CDEBGREG.INC
; to be INCluded after Manish Apte's CDEBUGD.INC from April 1997 WDJ

;**************************************************************************

ConsoleDebugOut macro AString, nocrlf

;  Displays a string on screen, via Manish Apte's CDBG.VXD (WDJ 4/97, p.54)

;  INPUT: AString = quoted string to display, may contain the following
;                   register placeholders, which will be replaced by their
;                   actual values: #(E)AX, #(E)BX, #(E)CX, #(E)DX, #(E)SI,
;                   #(E)DI, #(E)BP, #(E)SP, #AL, #BL, #CL, #DL
;         nocrlf = 1 to suppress a newline after the string

;  NOTE: limitations:
;        - each register placeholder may only appear once
;        - AString may not contain '#' other than in register names, nor
;          characters that have a special meaning to MASM, i.e. semicolons

;**************************************************************************

if deblevel le deblevelretail           ;for debug mode only
    exitm
    endif

    pushfd
    pushad

    InsertRegs nocrlf,AString           ;replace placeholders with actual data

    VxDCall Dbg_Out                     ;show string

    popad
    popfd
endm      ;ConsoleDebugOut


;**************************************************************************

;  InsertRegs macro nocrlf, Message:VARARG

;  Replaces register placeholders in Message with actual values

;  INPUT: Message = string to work on, see description of AString in
;                   macro ConsoleDebugOut
;         nocrlf = 1 to suppress a newline after the string

;  OUTPUT: eax = memory address containing fixed up message string

;**************************************************************************

;-----------------------------------------------------------------------

; helper macro: generates i.e. PosOfEAX = 22
SavePos macro Name
PosOf&Name = PosOfCross
endm

;-----------------------------------------------------------------------

BytesPushedSoFar = 68     ;see "if PosOfESP" below

InsertRegs macro nocrlf, Message:VARARG

        local   FinalString, MsgString, RegNameStr, FirstChar
        local   Str1, Str2, NewPos, IsExtended

MsgString   textequ <Message>          ;force string

PosOfCross  instr MsgString,<#>        ;is there a # in MsgString?
if PosOfCross eq 0                     ;if not,

_ldata segment                         ;just put the string into memory
FinalString label byte
    ifdif   MsgString, <"">      ;MASM will complain about empty strings
            db      MsgString
            endif
    ifb <nocrlf>
            db      0dh,0ah
            endif
    db      0
_ldata ends

    mov     eax, offset32 FinalString     ;and return a pointer to it

    exitm                                 ;finished
    endif

;-----------------------------------------------------------------------
; there are register placeholders in the Message, let's handle them!

; set all possible PosOfxxx to 0
irp RegName,<EAX,EBX,ECX,EDX,EDI,ESI,EBP,ESP,AX,BX,CX,DX,SI,DI,SP,BP,AL,BL,CL,DL>
    PosOf&&RegName = 0
    endm

; search all #regnames in MsgString
while PosOfCross ne 0
     ;extract 3 character register name
     RegNameStr substr MsgString,PosOfCross+1,3
     FirstChar substr RegNameStr,1,1
     IsExtended instr RegNameStr,<E>       ;if it's not an extended register...
     if IsExtended ne 1
	IsExtended instr RegNameStr,<e>
        endif
     if IsExtended ne 1
        RegNameStr substr RegNameStr,1,2   ;...its name has only 2 characters
        endif

     ;save position of this register placeholder
     SavePos %RegNameStr

     ;make room for actual register value in MsgString
     Str1 substr MsgString,1,PosOfCross-1  ;everything before placeholder
     Str2 substr MsgString,PosOfCross      ;rest
     if IsExtended eq 1
          ;four more characters for extended (32 bit) registers
          MsgString catstr Str1,<XXXX>,Str2
          NewPos = PosOfCross+4
       else
          ;..or one more character for other registers
          MsgString catstr Str1,<X>,Str2
          NewPos = PosOfCross+1
     endif

     ;NewPos = position of # in modified string; look for further #'s
     PosOfCross instr NewPos+1,MsgString,<#>

     endm  ;while
;-----------------------------------------------------------------------

_ldata segment
; dump the contents of MsgString, with or without cr/lf
FinalString db      MsgString
    ifb <nocrlf>
            db      0dh,0ah
            endif
            db      0
_ldata ends

;-----------------------------------------------------------------------

; now fill in the actual register values
       ; first: 32-bit registers
irp RegName,<EAX,EBX,ECX,EDX,EDI,ESI,EBP>
    if PosOf&&RegName gt 0  ;if MsgString contained this register placeholder..
       pushad
       mov      eax,RegName     ;..dump it
       mov      ecx,8           ;8 ASCII characters
       mov      edi,offset32 FinalString + PosOf&&RegName - 2
       ;note: -2: -1 for leading quote,
       ;          -1 because 1st char of string has index 1 for string macros
       call     DumpRegister
       popad
       endif  ;PosOf
    endm  ;irp

       ; then: 16-bit registers - and handle 8-bit like 16-bit
irp RegName,<AX,BX,CX,DX,SI,DI,BP,AL,BL,CL,DL>
    if PosOf&&RegName gt 0  ;if MsgString contained this register placeholder..
       pushad
       movzx    eax,RegName     ;..dump it
       mov      ecx,4           ;4 ASCII characters
       mov      edi,offset32 FinalString + PosOf&&RegName - 2
       call     DumpRegister
       popad
       endif  ;PosOf
    endm  ;irp

       ; finally: stack pointers - need special treatment
if PosOfESP gt 0  ;if MsgString contained this register placeholder..
       pushad
       mov      eax,esp                ;..dump it
       add      eax,BytesPushedSoFar   ;adjust for what we have pushed
       mov      ecx,8                  ;8 ASCII characters
       mov      edi,offset32 FinalString + PosOfESP - 2
       call     DumpRegister
       popad
       endif ;PosOfESP

if PosOfSP gt 0  ;if MsgString contained this register placeholder..
       pushad
       movzx    eax,sp                 ;..dump it
       add      eax,BytesPushedSoFar   ;adjust for what we have pushed
       mov      ecx,4                  ;4 ASCII characters
       mov      edi,offset32 FinalString + PosOfSP - 2
       call     DumpRegister
       popad
       endif ;PosOfSP

       mov     eax, offset32 FinalString    ;set pointer to output string

endm    ;InsertRegs

;**************************************************************************

if deblevel gt deblevelretail           ;debug mode only

VxD_LOCKED_CODE_SEG

;**************************************************************************

DumpRegister proc private

;  Converts up to 4 byte of binary data into ASCII string

;  INPUT: eax = data to convert
;         ecx = length of ASCII string
;         ds:edi = address where to put ASCII string

;**************************************************************************

;        assume  ds:_ldata

        pushfd
        pushad

        add     edi,ecx         ;set pointer to last character + 1

@@nextnibble:
        dec     edi             ;next char address
        mov     bl,al           ;convert lowest 4 bit of eax
        and     bl,0Fh
        add     bl,'0'
        cmp     bl,'9'
        jbe     @@DRok1
        add     bl,'A' - ('9'+1)
@@DRok1:
        mov     [edi],bl        ;store ASCII char
        shr     eax,4           ;fetch next lowest 4 bits
        loop    @@nextnibble

        popad
        popfd
        ret
DumpRegister endp

VxD_LOCKED_CODE_ENDS

endif ;deblevel le deblevelretail

;**************************************************************************

; End of CDEBGREG.INC


